/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_FILE_H
#include "file.h"
#define INC_FILE_H
#endif

#ifndef INC_AUDIOCD_H
#include "audiocd.h"
#define INC_AUDIOCD_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

using FileSpace::File;
using TrackSpace::Track;
using MediaSpace::AudioCD;

//-----------------------------------------------------------------------------------------------------------
AudioCD::AudioCD() : CD(){
    fYear = 0;
    fArtist = "";
    fGenre = "";
}
//-----------------------------------------------------------------------------------------------------------
AudioCD::AudioCD(const QString a, const QString t, const QString g, const QString c) : CD(){
    fArtist = a;
    fTitle = t;
    fGenre = g;
    fComment = c;
    fYear = 0;
}
//-----------------------------------------------------------------------------------------------------------
AudioCD::~AudioCD() {
}
//-----------------------------------------------------------------------------------------------------------
QString AudioCD::getGenre() const {
    return fGenre;
}
//-----------------------------------------------------------------------------------------------------------
QString AudioCD::getArtist() const{
    return fArtist;
}
//-----------------------------------------------------------------------------------------------------------
void AudioCD::setArtist(const QString a){
    fArtist = a;
}
//-----------------------------------------------------------------------------------------------------------
void AudioCD::setGenre(const QString g) {
    fGenre = g;
}
//-----------------------------------------------------------------------------------------------------------
int AudioCD::getYear() const{
    return fYear;
}
//-----------------------------------------------------------------------------------------------------------
void AudioCD::setYear(const int i){
    fYear = i;
}
//-----------------------------------------------------------------------------------------------------------
void AudioCD::setYear(const QString s){
    fYear = s.toInt();
}
//-----------------------------------------------------------------------------------------------------------
void AudioCD::setTracklist(Track::Tracklist& f) {
    clear();
    Track::TrackIterator FI = f.begin();
    bool setGenre = ((!fGenre.isEmpty())&&(fGenre != "Various"));
    bool setYear = (fYear != 0);
    QString album = fArtist + " - " + fTitle;
    
    while (FI != f.end()) {
	if (setGenre) (*FI)->setGenre( fGenre );
	if (setYear) (*FI)->setYear( fYear );
	(*FI)->setAlbum(album);
	(*FI)->setBitrate(128);
	(*FI)->setSamplingRate(44);
	CD::add(*FI);
	++FI;
    }
	
}
//-----------------------------------------------------------------------------------------------------------
void AudioCD::setTrackAttributes() {
    bool setGenre = ((!fGenre.isEmpty())&&(fGenre != "Various"));
    bool setYear = (fYear != 0);
    QString tmp = "";
    if (!setYear && !setGenre) return;
    
    Track* trk = 0;
    
    File::FileIterator FI = fFilelist.begin();
    int i = 1;
    while (FI != fFilelist.end()) {
	trk = dynamic_cast<Track*>(*FI);
	assert(trk != 0);
	if (setGenre) trk->setGenre( fGenre );
	if (setYear) trk->setYear( fYear );
	trk->setName("Track " + tmp.setNum(i) + ".cda");
	++FI;
	++i;
    }
}
//-----------------------------------------------------------------------------------------------------------
void AudioCD::writeTxt(TextOStream* ostr) const {
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("AudioCD::writeTxt","Pre: Ostream is 0 or not good"));
    
    CD::writeTxt(ostr);
    
    switch (ostr->getMode()) {
    case TextOStream::SHORT:
	if (!fArtist.isEmpty()) *ostr << "Artist: "  << fArtist << "\n";
	break;
    case TextOStream::LONG:
	if (!fArtist.isEmpty()) *ostr << "Artist: "  << fArtist << "\n";
	if (!fGenre.isEmpty()) *ostr << "Genre: "  << fGenre << "\n";
	if (!empty(fYear)) *ostr << "Year: "  << fYear << "\n";
	break;
    default:
	break;
    }
    *ostr << "Tracks:\n\n";
    writeTxtFileList(ostr);	
    
   if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("AudioCD::writeTxt","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void AudioCD::writeHtml(TextOStream* ostr) const{
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("AudioCD::writeHtml","Pre: Ostream is 0 or not good"));
    
    switch (ostr->getMode()) {
    case TextOStream::SHORT:
	if (ostr->isCdOnly()) {
	    *ostr << "<tr>\n";
	    CD::writeHtml(ostr);
	    *ostr << "   <td>" << fArtist << "</td>\n";
	    *ostr << "</tr>\n";
	} else {
	    CD::writeHtml(ostr);
	    if (!empty(fArtist)) *ostr << "<b>Artist: </b> "  << fArtist << "<br>\n";
	}
	break;
    case TextOStream::LONG:
	if (ostr->isCdOnly()) {
	    *ostr << "<tr>\n";
	    CD::writeHtml(ostr);
	    *ostr << "   <td>" << fArtist << "</td>\n";
	    *ostr << "   <td>" << fGenre << "</td>\n";
	    *ostr << "   <td>" << fYear << "</td>\n";
	    *ostr << "</tr>\n";
	} else {
	    CD::writeHtml(ostr);
	    if (!fArtist.isEmpty())     *ostr << "<b>Artist: </b>"  << fArtist << "<br>\n";
	    if (!fGenre.isEmpty())  *ostr << "<b>Genre: </b>"   << fGenre << "<br>\n";
	    if (!empty(fYear))     *ostr << "<b>Year: </b>"  << fYear << "<br>\n";
	}
    default:
	break;
    }
    if (!ostr->isCdOnly()) {
	*ostr << "<b>Tracks:</b><br><br>\n";
	writeHtmlFileList(ostr);	
    }
    
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("AudioCD::writeHtml","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
//The M3U format only deals with Tracks, not CDs
void AudioCD::writeM3u(TextOStream* ostr) const {
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("AudioCD::writeM3u","Pre: Ostream is 0 or not good"));
    writeM3uFileList(ostr);
}
//-----------------------------------------------------------------------------------------------------------
//The Pls format only deals with Tracks, not CDs
void AudioCD::writePls(TextOStream* ostr) const {
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("AudioCD::writePls","Pre: Ostream is 0 or not good"));
    writePlsFileList(ostr);
}
//-----------------------------------------------------------------------------------------------------------
void AudioCD::writeB4s(TextOStream* ostr) const{
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("AudioCD::writeB4s","Pre: Ostream is 0 or not good"));
    writeB4sFileList(ostr);
}
//-----------------------------------------------------------------------------------------------------------
void AudioCD::writeCsv(TextOStream* ostr, const QStringList& fields) const{
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("AudioCD::writeCsv","Ostream is 0 or not good"));
    
    CD::writeCsv(ostr,fields);
    
    if (ostr->isCdOnly()) {
	if (fields.contains("Year")) *ostr << fYear              << ";";
	if (fields.contains("Artist")) *ostr << fArtist              << ";";
	if (fields.contains("Genre")) *ostr << fGenre              << ";";
    }
}
//-----------------------------------------------------------------------------------------------------------
QString AudioCD::getTypeString() const {
    return  "AUDIO_CD";    
}
//-----------------------------------------------------------------------------------------------------------
Track::Tracklist* AudioCD::getTracklist() {
    Track::Tracklist* tlist = new Track::Tracklist();
    Track* trk = 0;
    
    File::FileIterator FI = fFilelist.begin();
    while (FI != fFilelist.end()) {
	trk = dynamic_cast<Track*>(*FI);
	assert(trk != 0);
	tlist->push_back(trk);
	++FI;
    }
    
    return tlist;
}
//-----------------------------------------------------------------------------------------------------------
