/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#ifndef INC_CD_H
#include "cd.h"
#define INC_CD_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#include <iostream>
#include "assert.h"
#include <ctype.h>

using MediaSpace::CD;
using std::cout;
//-----------------------------------------------------------------------------------------------------------
CD::CD() {
    fFileCount = 0;
    fID = "0";
    fComment = "";
    fTitle = "";
}
//-----------------------------------------------------------------------------------------------------------
QString CD::toString(const int i) const{
    QString tmp = "";
    return tmp.setNum(i);
}
//-----------------------------------------------------------------------------------------------------------
int CD::toInt(const QString s) const{
    return s.toInt();
}
//-----------------------------------------------------------------------------------------------------------
CD::CD(const int i) {
    fID = toString(i);
    fComment = "";
    fFileCount = 0;
    fTitle = "";
}
//-----------------------------------------------------------------------------------------------------------
CD::~CD() {
   clear();
}
//-----------------------------------------------------------------------------------------------------------
int CD::getID() const {
    return toInt(fID);
}
//-----------------------------------------------------------------------------------------------------------
QString CD::getIDString() const {
    return fID;
}
//-----------------------------------------------------------------------------------------------------------
void CD::setID(const int i) {
    fID = toString(i);
}
//-----------------------------------------------------------------------------------------------------------
void CD::setID(const QString i){
    fID = i;
}
//-----------------------------------------------------------------------------------------------------------
QString CD::getTitle() const{
    return fTitle;
}
//-----------------------------------------------------------------------------------------------------------
void CD::setTitle(const QString t){
    fTitle = t;
}
//-----------------------------------------------------------------------------------------------------------
int CD::getFileCount() const{
    return fFileCount;
}
//-----------------------------------------------------------------------------------------------------------
void CD::setFileCount(const int i){
    fFileCount = i;
}
//-----------------------------------------------------------------------------------------------------------
void CD::setFileCount(const QString i){
    fFileCount = i.toInt();
}
//-----------------------------------------------------------------------------------------------------------
QString CD::getComment() const {
    return fComment;
}
//-----------------------------------------------------------------------------------------------------------
void CD::setComment(const QString c) {
    fComment = c;
}
//-----------------------------------------------------------------------------------------------------------
void CD::add(File* f) {
    assert(f != 0);
    f->setCDno( toInt(fID) );
    fFilelist.push_back(f);	
    ++fFileCount;
}
//-----------------------------------------------------------------------------------------------------------
void CD::clear() {
   File::FileIterator FI = fFilelist.begin();
    while (FI != fFilelist.end()) {
	delete (*FI);
	++FI;
    }
    fFilelist.clear();
    fFileCount = 0;
}
//-----------------------------------------------------------------------------------------------------------
void CD::setFilelist(File::Filelist& f) {
	clear();
	File::FileIterator FI = f.begin();
	while (FI != f.end()) {
		add(*FI);
		++FI;
	}
	
}

//-----------------------------------------------------------------------------------------------------------
void CD::writeTxt(TextOStream* ostr) const {
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("CD::writeTxt","Pre: Ostream is 0 or not good"));
    
   // if (ostr->isShort()) cout<<"writing cd in mode short\n";
   // if (ostr->isLong()) cout<<"writing cd in mode long\n";
    
    switch (ostr->getMode()) {
    case TextOStream::SHORT:
	*ostr << "\n";
	*ostr << fID << " : ";
	if (!empty(fTitle))        *ostr << fTitle   << "\n";
	break;
    case TextOStream::LONG:
	*ostr << "ID: "             << fID           << "\n"; 
	if (!empty(fTitle))        *ostr << "Title: "          << fTitle        << "\n";
	*ostr << "File Count: " << fFileCount << "\n"; 
	if (!empty(fComment)) *ostr << "Comment: "  << fComment << "\n";
	break;
    default:
	break;
    }
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("CD::writeTxt","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void CD::writeHtml(TextOStream* ostr) const{
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("CD::writeHtml","Pre: Ostream is 0 or not good"));
    
    switch (ostr->getMode()) {
    case TextOStream::SHORT:
	if (ostr->isCdOnly()) {
	    *ostr << "   <td>" << fID << "</td>\n";
	    *ostr << "   <td>";
	    if (!empty(fTitle)) *ostr << "<a name=\"" << fID << "_" << fTitle << "\"></a>\n";   
	    *ostr    << fTitle << "</td>\n";
	} else {
	    if (!empty(fTitle)) *ostr << "<a name=\"" << fID << "_" << fTitle << "\"></a>\n";   
	    *ostr << "<h3>" << fID << " : " << fTitle << "</h3>\n"; 
	}
	break;
    case TextOStream::LONG:
	//place an anchor so the cd can be hyperlinked
	if (ostr->isCdOnly()) {
	    *ostr << "   <td>" << fID << "</td>\n";
	    *ostr << "   <td>";
	    if (!empty(fTitle)) *ostr << "<a name=\"" << fID << "_" << fTitle << "\"></a>\n";   
	    *ostr    << fTitle << "</td>\n";
	    *ostr << "   <td>" << fFileCount << "</td>\n";
	    *ostr << "   <td>" << fComment << "</td>\n";
	} else {
	    if (!empty(fTitle)) *ostr << "<a name=\"" << fID << "_" << fTitle << "\"></a>\n";   
	    *ostr << "<b>ID: </b>"             << fID             << "<br>\n"; 
	    *ostr << "<b>File Count: </b>" << fFileCount << "<br>\n"; 
	    if (!empty(fTitle))         *ostr << "<b>Title: </b>"          << fTitle          << "<br>\n";
	    if (!empty(fComment)) *ostr << "<b>Comment: </b>" << fComment   << "<br>\n";
	}
	break;
    default:
	break;
    }
    
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("CD::writeHtml","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
//The M3U format only deals with Tracks, not CDs
void CD::writeM3u(TextOStream* ostr) const{
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("CD::writeM3u","Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
//The Pls format only deals with Tracks, not CDs
void CD::writePls(TextOStream* ostr) const{
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("CD::writePls","Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void CD::writeB4s(TextOStream* ostr) const{
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("CD::writeB4s","Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void CD::writeCsv(TextOStream* ostr, const QStringList& fields) const{
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("CD::writeCsv","Ostream is 0 or not good"));
    
    if (ostr->isCdOnly()) {
	if (fields.contains("Id")) *ostr << fID              << ";";
	if (fields.contains("Title")) *ostr << fTitle              << ";";
	if (fields.contains("FileCount")) *ostr << fFileCount           << ";";
	if (fields.contains("Comment")) *ostr << fComment   << ";";
    } else {
	writeCsvFileList(ostr,fields);
    }
}
//-----------------------------------------------------------------------------------------------------------
void CD::writeTxtFileList(TextOStream* ostream) const {
    File::ConstFileIterator FI = getConstBegin();
    while (FI != getConstEnd()) {
	(*FI)->writeTxt(ostream);
	*ostream <<"\n";
	++FI;
    }
}
//-----------------------------------------------------------------------------------------------------------
void CD::writeHtmlFileList(TextOStream* ostream) const {
    File::ConstFileIterator FI = getConstBegin();
    while (FI != getConstEnd()) {
	(*FI)->writeHtml(ostream);
	++FI;
    }
}
//-----------------------------------------------------------------------------------------------------------
void CD:: writeM3uFileList(TextOStream* ostream) const {
    File::ConstFileIterator FI = getConstBegin();
    while (FI != getConstEnd()) {
	(*FI)->writeM3u(ostream);
	++FI;
    }
}
//-----------------------------------------------------------------------------------------------------------
void CD::writeB4sFileList(TextOStream* ostream) const {
   File::ConstFileIterator FI = getConstBegin();
    while (FI != getConstEnd()) {
	(*FI)->writeB4s(ostream);
	++FI;
    }
}
//-----------------------------------------------------------------------------------------------------------
void CD:: writePlsFileList(TextOStream* ostream) const {
  File::ConstFileIterator FI = getConstBegin();
  uint i = 1;
  while (FI != getConstEnd()) {
	(*FI)->writePls(ostream,i);
	++FI;
	++i;
    }
}
//-----------------------------------------------------------------------------------------------------------
void CD:: writeCsvFileList(TextOStream* ostream,const QStringList& fields) const {
  File::ConstFileIterator FI = getConstBegin();
  uint i = 1;
  while (FI != getConstEnd()) {
	(*FI)->writeCsv(ostream,fields);
	*ostream <<"\n";
	++FI;
	++i;
    }
}
//-----------------------------------------------------------------------------------------------------------
File::FileIterator CD::getBegin() {
    return fFilelist.begin();
}
//-----------------------------------------------------------------------------------------------------------
File::FileIterator CD::getEnd()  {
    return fFilelist.end();
}
//-----------------------------------------------------------------------------------------------------------
File::ConstFileIterator CD::getConstBegin() const{
    return fFilelist.begin();
}
//-----------------------------------------------------------------------------------------------------------
File::ConstFileIterator CD::getConstEnd()const {
    return fFilelist.end();
}
//-----------------------------------------------------------------------------------------------------------
QString CD::getTypeString() const {
    return  "GENERIC_CD";    
}
//-----------------------------------------------------------------------------------------------------------
