/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#ifndef INC_CDLOOKUP_H
#include "cdlookup.h"
#define INC_CDLOOKUP_H
#endif

#ifndef INC_EXCEPTION_H
#include "exception.h"
#define INC_EXCEPTION_H
#endif

#ifndef INC_TRACK_H
#include "track.h"
#define INC_TRACK_H
#endif


#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#include <stdlib.h>
#include "mb_c.h"
#include "assert.h"

using std::cout;
//-----------------------------------------------------------------------------------------------------------
CDLookup::CDLookup(const QString server, const short port, const QString device) {
    fMusicBrainzObj = new MusicBrainz();
    setServer( server , port );
    setDevice( device );   
}
//-----------------------------------------------------------------------------------------------------------
CDLookup::~CDLookup() {
    delete fMusicBrainzObj;
}
//-----------------------------------------------------------------------------------------------------------
void CDLookup::setServer(const QString server, const short port) {
    fMusicBrainzObj->SetServer( server.latin1(), port );
}
//-----------------------------------------------------------------------------------------------------------
void CDLookup::setDevice(const QString drive) {
    fMusicBrainzObj->SetDevice( drive.latin1() );
}
//-----------------------------------------------------------------------------------------------------------
bool CDLookup::authenticate(const QString username, const QString pwd) {
    return fMusicBrainzObj->Authenticate(username.latin1(), pwd.latin1());   
    
}
//-----------------------------------------------------------------------------------------------------------
void CDLookup::setProxy(const QString proxy, const int port) {
    fMusicBrainzObj->SetProxy(proxy.latin1(), (short)port);
}
//-----------------------------------------------------------------------------------------------------------
//throws Exception
void CDLookup::lookupCD(vector<AudioCD*>& albumList, const bool getTracks) {
    
    string error = "";
    string data = "";
    int numAlbums = 0;
    bool ret = false;
    int numTracks = 0;
    int trackNum = 0;
    AudioCD* cd = 0;
    Track* track = 0;
    QString title = "";
    QString artist = "";
    QString trackName = "";
    int trackLen = 0;
    QString artistName = "";
    bool compilation;
    QString tmp = "";
    QString comment = "";
    QString trackTitle = "";
	   
    // Tell the client library to return data in ISO8859-1 and not UTF-8
    fMusicBrainzObj->UseUTF8(false);

    // Execute the GetCDInfo query, which pulls the TOC from the 
    // audio CD in the cd-rom drive, calculates the disk id and 
    // requests the data from the server
    ret = fMusicBrainzObj->Query(string(MBQ_GetCDInfo));
    
    if (!ret) {
         fMusicBrainzObj->GetQueryError(error);
         throw( new Exception("Query failed: " + QString(error.c_str())) );
    }

    // Check to see how many items were returned from the server
    numAlbums = fMusicBrainzObj->DataInt(MBE_GetNumAlbums);
    
    if (numAlbums < 1) {
        throw( new Exception("This CD was not found.\n") );
    }
   
    for(int i = 1; i <= numAlbums; i++) {
	// Select the album
	fMusicBrainzObj->Select(MBS_SelectAlbum, i);
	
	// Get the number of tracks
	numTracks = fMusicBrainzObj->DataInt(MBE_AlbumGetNumTracks);
	title =  fMusicBrainzObj->Data(MBE_AlbumGetAlbumName).c_str();
	artist = fMusicBrainzObj->Data(MBE_AlbumGetArtistName, i).c_str();
	fMusicBrainzObj->GetIDFromURL(fMusicBrainzObj->Data(MBE_AlbumGetAlbumId), data);
	compilation = (artist == "Various");
	
	//cout<<"artist: " << artist << " numtracks: " << numTracks << "\n";
	cd = new AudioCD();
	cd->setTitle( artist + " - " + title );
	cd->setArtist( artist );
	cd->setID( i-1 );
	
	compilation = (artist == "Various");
	    
	if (getTracks) {
	    for(int i = 1; i <= numTracks; i++) {
		trackNum = fMusicBrainzObj->DataInt(MBE_AlbumGetTrackNum, i);
		trackName =  fMusicBrainzObj->Data(MBE_AlbumGetTrackName, i).c_str();
		trackLen = fMusicBrainzObj->DataInt(MBE_AlbumGetTrackDuration,i) / 1000;
		 	
		//cout<<"Track num: " << trackNum << " length: " << trackLen << " name: " << trackName << " artist: " << artist << "\n";
		
		if(compilation) {
		    //cout<<"Compilation track\n";
		    tmp = trackName;
		    trackName = tmp.replace("/","-");
		    //cout<<"after replace trackName is: "<< trackName << "\n";
		    splitTrackname(trackName,artistName,trackTitle, comment, 1);
		    //cout<<"After split: name: " << trackTitle << " artist: " << artistName << "\n";
		} else {
		    trackTitle = trackName;
		    comment = trim( extractComment(trackTitle) );
		    
		    trackName = artist + " - " + trackName;
		    artistName = artist;
		}
		
		track = new Track(artistName, trackTitle, i);
		track->setBitrate(128);
		track->setSamplingRate(44);
		track->setAlbum(artist + " - " + title);
		track->setName(trackName);
		track->setComment(comment);
		track->setLength(trackLen);
		track->setDescription("Audio CD track (cda)");
		track->setReadOnly(true);
		cd->add(track);
	    }
	}
	
	albumList.push_back( cd );
	
	fMusicBrainzObj->Select(MBS_Back);
    }
}
//-----------------------------------------------------------------------------------------------------------
