/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/

#include "stringmanip.h"
#include "rcman.h"
#include "qregexp.h"
#include "musicexpression.h"
#include "metainfo.h"

using ResourceSpace::RcMan;

void CleanDirDialog::init() {
	dirChooser->setOnlyDirs(true);
	dirChooser->setEnabled(false);
}

void CleanDirDialog::destroy() {
}

void CleanDirDialog::setDirectory(const QString dir) {
	dirChooser->setFileName(dir);
}

QString CleanDirDialog::getDirectory() const {
	return dirChooser->fileName();
}

QString CleanDirDialog::cleanFile(const QString filename, const QString path, const ResourceSpace::RcMan* rcman) const {
	QString ext = getExtension(filename);
	ext = ext.lower();
	QString s = stripExtension(filename);
	
	if(customReplace->isChecked()) {
		if(stringReplace->isChecked()) {
			if(useRegex->isChecked()) {
				s = s.replace(QRegExp(customBefore->text()),customAfter->text());
			} else {	
				s = s.replace(customBefore->text(),customAfter->text());
			}
		} else if(musicExpReplace->isChecked()) {
			s = MusicExpression::fitToPattern(s, musicExpBefore->text(), musicExpAfter->text());
		} else if(nameFromMetatag->isChecked()) {
			QString expression = metatagExpression->text();
			QString file = path + DomoGlobals::DIR_SEP + filename;
			s = MetaInfo::filenameFromTag(file, expression);
		} else {
		   //do nothing, should not happen
		}
	}
	
	if (replaceUnderscores->isChecked()) {
		s = s.replace(QRegExp("(\\_)+")," ");
	}
	
	if (stripDuplicatedIds->isChecked()) {
		//remove (1),(2),(3),...
		s = s.replace(QRegExp("\\(\\d\\)"),"");
	}
	
	if(simplifyDashes->isChecked()) {
		//eg: replace "bla--- -- -bla" by "bla - bla"
		s = s.replace(QRegExp("(\\-)((\\s)*(\\-)+(\\s)*)*(\\-)")," - ").simplifyWhiteSpace();
	}
	
	if(replaceBraces->isChecked()) {
		s = s.replace('[', '(');
		s = s.replace(']', ')');
	}
	
	if (stripTrackNos->isChecked()) {
		//eg: dont do this: '50 Cent - blabla' -> 'Cent - blabla'
		if ( ! rcman->startsWithSafeArtist(s)) {
		    s = stripTrackNo(s);
		}
	}
	
	if(simplifyWs->isChecked()) {
		s = s.simplifyWhiteSpace();
		s = s.stripWhiteSpace();
	}
	
	if (toLargeCase->isChecked()){
		s = largeCase(s);
	}

	return s + "." + ext; 
}

bool CleanDirDialog::getForcePrompt() const {
	return forcePrompt->isChecked();
}

bool CleanDirDialog::getCheckConsistency() const {
	return checkConsistency->isChecked();
}

bool CleanDirDialog::getStripTrackNos() const {
	return stripTrackNos->isChecked();
}

bool CleanDirDialog::writeTags() const {
	return setTagsFromFilename->isChecked();
}

void CleanDirDialog::writeTags(const QString dir, const QString fname) const {
	if (writeTags()){
		QString file = dir + DomoGlobals::DIR_SEP + fname;
		QString expression = filenameExpression->text();
		MetaInfo::writeTagsFromFilename(file, expression);
	}
}
