/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#ifndef INC_COMPARE_H
#define INC_COMPARE_H

#ifndef INC_TRACK_H
#include "track.h"
#define INC_TRACK_H
#endif

#ifndef INC_FILE_H
#include "file.h"
#define INC_FILE_H
#endif

#ifndef INC_COLLECTION_H
#include "collection.h"
#define INC_COLLECTION_H
#endif

#ifndef INC_DBASE_H
#include "dbase.h"
#define INC_DBASE_H
#endif

#ifndef INC_RCMAN_H
#include "rcman.h"
#define INC_RCMAN_H
#endif

#include <map>
#include <vector>
#include "qprogressbar.h"
#include "qstringlist.h"
#include "qlistview.h"
#include "log.h"

using DbaseSpace::Dbase;
using std::map;
using std::vector;
using FileSpace::File;
using TrackSpace::Track;
using MediaSpace::Collection;
using ResourceSpace::RcMan;

namespace CompareSpace {

class Compare {
	private:
		map<File*,File::Filelist> fYesList;
		File::Filelist fNoList;
		map<File*,File::Filelist> fMaybeList;
		map<File*,File::Filelist> fDiffVerList;
		bool fRequireEqualCom;
		int fLenTolerance;
		RcMan* fRcMan;
		Log* fLogger;
	public:
	
		Compare(RcMan* resourceManager, Log* logger);
		~Compare();
	
		typedef std::map<File*,File::Filelist>::const_iterator ConstMaybeListIterator;	
		typedef std::map<File*,File::Filelist>::const_iterator ConstYesListIterator;	
		typedef std::map<File*,File::Filelist>::const_iterator ConstDiffVerListIterator;	
		
		typedef std::map<File*,File::Filelist>::iterator MaybeListIterator;	
		typedef std::map<File*,File::Filelist>::iterator YesListIterator;	
		typedef std::map<File*,File::Filelist>::iterator DiffVerListIterator;	
		
		ConstMaybeListIterator getConstMaybeBegin();
		ConstMaybeListIterator getConstMaybeEnd();
	
		ConstYesListIterator getConstYesBegin();
		ConstYesListIterator getConstYesEnd();
		
		ConstDiffVerListIterator getConstDiffVerBegin();
		ConstDiffVerListIterator getConstDiffVerEnd();
	
		File::ConstFileIterator getConstNoBegin();
		File::ConstFileIterator getConstNoEnd();
		
		MaybeListIterator getMaybeBegin();
		MaybeListIterator getMaybeEnd();
	
		YesListIterator getYesBegin();
		YesListIterator getYesEnd();
		
		DiffVerListIterator getDiffVerBegin();
		DiffVerListIterator getDiffVerEnd();
	
		File::FileIterator getNoBegin();
		File::FileIterator getNoEnd();

		void clear();
		
		void setRequireEqualCom(const bool b);
		bool getRequireEqualCom() const;
		
		int getLenTolerance() const;
		void setLenTolerance(const int i);
		
		void getKeywords(const QString s, QStringList& list);
		
		void compare(Track::Tracklist& tlist,Dbase* DB,const QString table="Track",QProgressBar* pBar=0);
		void displayResults(QListView* list);
		void deleteCompareSelection(QListView* compareResults, 
					    const QString comparePath,
					    const bool noEntries, 
					    const bool diffVerEntries,
					    const bool maybeEntries,
					    const bool yesEntries,
					    const bool selectedEntries);
		
		void exportCompareSelection(QListView* compareResults,
						     const QString filename,
						     const bool noEntries,
						     const bool diffVerEntries,
						     const bool maybeEntries,
						     const bool yesEntries,
						     const bool selectedEntries);
    };
	

} //Close namespace CompareSpace;

#endif
