/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)
   
   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   
----------------------------------------------------------------------------*/

#include "musicexpression.h"
#include "consistencychecker.h"
#include "assert.h"
#include "skiptrack.h"
#include "splittrackdiag.h"
#include "metainfo.h"
#include "stringmanip.h"
#include "qdialog.h"
#include "fileio.h"
#include "domoglobals.h"
#include "filechooser.h"
#include "qlineedit.h"
#include "qcheckbox.h"
#include "qradiobutton.h"
#include "stringmanip.h"
#include "qcombobox.h"
#include "qspinbox.h"

using TrackSpace::Track;
using FileSpace::File;
using ResourceSpace::RcMan;
//-----------------------------------------------------------------------------------------------------------
ConsistencyChecker::ConsistencyChecker(Log* logger, RcMan* rcman, Settings* settings) {
        assert(logger != 0);
        assert(rcman != 0);
        assert(settings != 0);
        fLogger = logger;
        fRcMan = rcman;
        fSettings = settings;
}
//-----------------------------------------------------------------------------------------------------------
ConsistencyChecker::~ConsistencyChecker() {
}
//-----------------------------------------------------------------------------------------------------------
bool ConsistencyChecker::splitFilename(Track* t, ConsistencyProp& properties) {
        assert(t != 0);
        QString title = "";
        QString artist = "";
        QString comment = "";
        QString mood = "";
        QString album = "";
        QString genre = "";
        int year = 0;
        int score = 0;
        
        SplitTrackDiag* dialog = new SplitTrackDiag();
        dialog->setProperties(properties);
        dialog->setGenreList(fRcMan->getGenreListRef());
        
        dialog->trackLine->setText(t->absPath());
        dialog->initCustomFields(t);
        dialog->enableTrm();
        
        Track* trk = new Track();
        trk->setName( t->getName() );
        trk->setPath( t->getPath() );
        MetaInfo::getMetaInfo(trk, true);
        dialog->initMetatagFields(trk);
        
        if((properties.fApplyToNone) || (properties.fApplyToAllInAlbum && !properties.fSameAlbum)) {
                if (dialog->exec() == QDialog::Rejected) {
                        delete dialog;
                        delete trk;
                        return false;
                }
        }
        
        if (dialog->custom->isChecked()) {
                artist = dialog->customArtist->text();
                title = dialog->customTitle->text();
                comment = dialog->customComment->text();
                album = dialog->customAlbum->text();
                genre = dialog->customGenre->currentText();
                year = dialog->customYear->value();
        } else if (dialog->meta->isChecked()) {
                artist = dialog->metaArtist->text();
                title = dialog->metaTitle->text();
                comment = dialog->metaComment->text();
                album = dialog->metaAlbum->text();
                genre = dialog->metaGenre->currentText();
                year = dialog->metaYear->value();
        } else if (dialog->trm->isChecked()) {
                artist = dialog->trmArtist->text();
                title = dialog->trmTitle->text();
                comment = dialog->trmComment->text();
                album = dialog->trmAlbum->text();
                genre = dialog->trmGenre->currentText();
                year = dialog->trmYear->value();
        } else if (dialog->onExpression->isChecked()) {
                QString exp = dialog->expression->text();
                Track* tk = MusicExpression::makeTrack(exp,stripExtension(t->getName()));
                artist = tk->getArtist();
                title = tk->getTitle();
                comment = tk->getComment();
                album = tk->getAlbum();
                genre = tk->getGenre();
                year = tk->getYear();
                
                if (tk->getTrackNo() > 0) t->setTrackNo(tk->getTrackNo());
                
                delete tk;
        } else {
                QString artist = t->getName();
                if (dashCount(artist) < dialog->getDashNum()) {
                        fLogger->warning("ConsistencyChecker::splitFilename",
                                         "Too large dash count specified, splitting on first instead (artist = '" + artist + "')");
                        splitTrackname(t->getName(),artist,title,comment, 1);
                } else {
                        splitTrackname(t->getName(),artist,title,comment, dialog->getDashNum());
                }
        }
        
        if(!empty(artist)) t->setArtist(artist);
        if(!empty(title)) t->setTitle(title);
        t->setComment(comment);
        t->setAlbum(album);
        t->setGenre(genre);
        t->setYear(year);
        t->setMood(mood);
        t->setScore(score);
        
        if (dialog->renameFile()) {
                QString newname = "";
                QString newpath = "";
                QString oldpath = t->getPath() + DomoGlobals::DIR_SEP + t->getName();
                QString ext = getExtension( t->getName() );
                
                if (!comment.isEmpty()) {
                        newname = artist + " - " + title + " (" + comment + ")." + ext;
                        newpath = t->getPath() + DomoGlobals::DIR_SEP + newname;
                } else {
                        newname = artist + " - " + title + "." + ext;
                        newpath = t->getPath() + DomoGlobals::DIR_SEP + newname;
                }
                
                if ( FileIO::renameFile( oldpath,  newpath ) ) {
                        fLogger->info("ConsistencyChecker::splitFilename",
                                      "File '" + oldpath + "' renamed to '" + newpath + "'");
                        t->setName(newname);
                } else {
                        fLogger->error("ConsistencyChecker::splitFilename","Rename of file '" 
                                       + oldpath + "' to " + newpath + "' failed...");    
                }
        }
        
        if(dialog->flushMetatag()) {
                MetaInfo::writeMetaInfo(t);
        }
        
        dialog->getProperties(properties);
        delete dialog;
        delete trk;
        return true;
}
//-----------------------------------------------------------------------------------------------------------
bool ConsistencyChecker::unknownTrackField(Track* t, ConsistencyProp& properties) {
        properties.fEnableSplitMode = false;
        return splitFilename(t,properties); 
}
//-----------------------------------------------------------------------------------------------------------
bool ConsistencyChecker::skipTrack(const Track* t) {
        if (t == 0) {
                fLogger->error("ConsistencyChecker::skipTrack","Track is 0, serious error!!");
                return false;
        }
        
        SkipTrack* dialog = new SkipTrack();
        dialog->setTrack(t);
        int res = dialog->exec();
        delete dialog;
        
        if (res == 1) {
                QString path = t->absPath();
                if (FileIO::deleteFile(path)) {
                        fLogger->info("ConsistencyChecker::skipTrack","'" + path + "' deleted");
                } else {
                        fLogger->warning("ConsistencyChecker::skipTrack","Delete '" + path + "' failed");
                }
                return true;
        } else if (res == 2) {
                return true;
        } else if (res == 3) {
                return false;
        } else {
                fLogger->error("ConsistencyChecker::skipTrack","Invalid return code!");
                return false;
        }
        
}
//-----------------------------------------------------------------------------------------------------------
bool ConsistencyChecker::makeConsistent(Track::Tracklist& tlist, QProgressBar* progressBar) {
        if (fSettings->getBool(Settings::fSkipConsCheck)) {
                fLogger->warning("ConsistencyChecker::makeConsistent","Skipping consistency check");
                return true;
        }
        
        ConsistencyProp properties;
        properties.fRenameFile = fSettings->getBool(Settings::fAutoRename);
        
        //force a user prompt on every track
        bool forcePrompt = fSettings->getBool(Settings::fForcePrompt);
        progressBar->reset();
        progressBar->setTotalSteps(tlist.size());
        QString s1 = "";
        QString s2 = "";
        QString s3 = "";
        QString prevAlbum = "";
        QString curAlbum = "";
        
        //Did the user specify a custom format that filenames are in
        bool useCustomTrackFormat = fSettings->getBool(Settings::fUseCustomTrackFormat);
        QString customTrackFormat = fSettings->get(Settings::fCustomTrackFormat);
        if(useCustomTrackFormat) 
                DomoGlobals::logger->debug("DirScanner::indexMusicDirectory","Using custom track format '" + customTrackFormat + "'");
        
        fLogger->info("ConsistencyChecker::makeConsistent","Checking consistency...");
        Track::TrackIterator TI = tlist.begin();
        Track::TrackIterator end = tlist.end();
        
        if(TI != end) prevAlbum = (*TI)->getAlbum();
        
        int i = 1;
        while (TI != end) {
                curAlbum = (*TI)->getAlbum();
                properties.fSameAlbum = ((prevAlbum == curAlbum) && !empty(curAlbum));
                if(!properties.fSameAlbum) {
                        if(!empty(curAlbum)) fLogger->debug("ConsistencyChecker::makeConsistent","Changing running album from '" 
                                                            + prevAlbum + "' to '" + curAlbum + "'");
                        prevAlbum = curAlbum;
                } else {
                        fLogger->debug("ConsistencyChecker::makeConsistent","Still in the same album '" + prevAlbum + "'");   
                }
                
                if((properties.fApplyToNone) || (properties.fApplyToAllInAlbum && !properties.fSameAlbum)) {
                        properties.fSkipFile = false;
                        properties.fDoNothing = true;
                }
                
                if (! fSettings->getBool(Settings::fSkipQualityCheck)) {
                        //check for very short, very long, low bitrate tracks
                        if ((*TI)->fishyQuality()) {
                                properties.fSkipFile = this->skipTrack(*TI);
                        }	
                } else {
                        fLogger->warning("ConsistencyChecker::makeConsistent","Skipping low quality check");
                }
                
                if (properties.fSkipFile == false) {
                        //This is where the actual consistency check starts, a number of different situations exist.
                        
                        //(1) The user forced a prompt on every track
                        if(forcePrompt) {
                                DomoGlobals::logger->info("ConsistencyChecker::makeConsistent","User prompt forced on track '" + (*TI)->getName() + "'");
                                if ( !splitFilename(*TI,properties) ) return false;
                                
                                //(2) Regardless of track structure , if the trackname is all in uppercase eg: "2 UNLIMITED - RAIN" we should double check with
                                //the user
                        } else if(isUpperCase((*TI)->getName(),true)) {
                                DomoGlobals::logger->info("ConsistencyChecker::makeConsistent","Detected an all uppercase track ('" 
                                                          + (*TI)->getName() + "'), prompting user");
                                if ( !splitFilename(*TI,properties) ) return false;
                                
                                //(3) if the user defined a custom track format and the current track matches that format there is no need to bother the user,
                                //we can safely go to the next track
                        } else if(useCustomTrackFormat && MusicExpression::matchesPattern(customTrackFormat,(*TI)->getName())) {
                                //custom track format, do nothing
                                DomoGlobals::logger->info("ConsistencyChecker::makeConsistent","Track '" + (*TI)->getName() 
                                                          + "' matches custom track format, not prompting user.");		  
                                
                                //(4) else assume the track format is "artist - title (comment)" start checking the consistency of the current track
                        } else {
                                //Check dashcount   
                                //get the number of '-' in the filename
                                i = dashCount((*TI)->getName());
                                if (i > 1){
                                        //get the number of visible dashes (visible = not enclosed by round/square brackets)
                                        int k = visibleDashCount((*TI)->getName());
                                        if (k < 1) {
                                                //no visible dashes -> unkown artist (eg: 03 My Song (Ice-T remix) )
                                                if ( !unknownTrackField(*TI,properties) ) return false;
                                        } else if (k == 1) {
                                                //only one visible dash, split on it
                                                s1 = ""; s2 = ""; s3 = "";
                                                splitOnVisibleDash((*TI)->getName(), s1, s2,s3,1);
                                                (*TI)->setArtist(s1);
                                                (*TI)->setTitle(s2);
                                                (*TI)->setComment(s3);
                                        } else {
                                                if (k == 2) {
                                                        //two visible dashes
                                                        s1 = ""; s2 = ""; s3 = "";
                                                        //try to split on the second one
                                                        splitOnVisibleDash((*TI)->getName(), s1, s2,s3,2);
                                                        //if we are lucky the first part turns out to be a hypened artist (eg: Ice-T - Gangsta)
                                                        if ( fRcMan->containsHyphenedArtist(s1) ) {
                                                                //if so, no need to consult the user
                                                                (*TI)->setArtist(s1);
                                                                (*TI)->setTitle(s2);
                                                                (*TI)->setComment(s3);
                                                        } else {
                                                                //user intervention needed
                                                                properties.fRememberArtist = true;
                                                                properties.fEnableSplitMode = true;
                                                                if ( !splitFilename(*TI,properties) ) return false;
                                                                rememberArtist(*TI, properties);
                                                        }
                                                } else {
                                                        //more than two visible dashes, user intervention needed
                                                        properties.fEnableSplitMode = true;
                                                        if ( !splitFilename(*TI,properties) ) return false;
                                                        rememberArtist(*TI, properties);
                                                }
                                        }
                                } else if (i == 0) {
                                        //no dashes at all, need to ask user
                                        if ( !unknownTrackField(*TI,properties) ) return false;
                                } else if (i == 1) {
                                        /*only one dash, don't automatically split on this one dash to
                to prevent tracks like "02 - This is the title" being added */
                                        QString name = (*TI)->getName();
                                        bool b = fRcMan->containsHyphenedArtist(name);
                                        
                                        if ( ! fRcMan->startsWithSafeArtist(name)) {
                                                name = stripTrackNo(name);
                                        }
                                        
                                        //does name contain a hypended artist or is the dash part of the track no?
                                        if ((dashCount(name) == 0) || b) {
                                                if ( !unknownTrackField(*TI,properties) ) return false;
                                        }
                                        
                                } else {
                                        assert(i >= 0);
                                }
                        }
                }	
                //did the user want to skip the track?
                if (properties.fSkipFile) {
                        //erase current track from the list
                        fLogger->info("ConsistencyChecker::makeConsistent","Skipping track '" + (*TI)->getName() + "'");
                        //Did the user want to delete the file
                        if(properties.fDeleteFile) {
                                QString path = (*TI)->absPath();
                                if (FileIO::deleteFile(path)) {
                                        fLogger->info("ConsistencyChecker::makeConsistent","'" + path + "' deleted");
                                } else {
                                        fLogger->warning("ConsistencyChecker::makeConsistent","Delete '" + path + "' failed");
                                }
                        }
                        delete (*TI);
                        TI = tlist.erase(TI);
                } else {
                        //try to strip the track number from the artist field
                        const QString artist = (*TI)->getArtist();
                        if ( ! fRcMan->startsWithSafeArtist(artist)) {
                                (*TI)->setArtist(stripTrackNo(artist));
                        } else {
                                fLogger->debug("ConsistencyChecker::makeConsistent","Track number not stripped from: " + artist);
                        }
                        ++TI;
                }
                progressBar->setProgress(i);
                ++i;
        }
        progressBar->reset();
        return true;
}
//-----------------------------------------------------------------------------------------------------------
void ConsistencyChecker::rememberArtist(const Track* trk, ConsistencyProp properties) {
        assert(trk != 0);
        if (properties.fRememberArtist) {
                QString s1 = trk->getArtist();
                if (dashCount(s1) == 1) {
                        fRcMan->addHyphenedArtist(s1);
                        fLogger->info("ConsistencyChecker::rememberArtist",
                                      "Added hyphened artist '" + s1 + "'");
                } else {
                        fLogger->warning("ConsistencyChecker::rememberArtist",
                                         "Artist '" + s1 + "' doesn't contain 1 dash, artist not added");
                }
        }
}
//-----------------------------------------------------------------------------------------------------------
