/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#include "assert.h"

#ifndef INC_DBASE_H
#include "dbase.h"
#define INC_DBASE_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#ifndef INC_DBASEEXCEPTION_H
#include "dbaseexception.h"
#define INC_DBASEEXCEPTION_H
#endif

using std::vector;
using std::cout;
using TrackSpace::Track;
using MediaSpace::Mp3CD;
using MediaSpace::MovieCD;
using MediaSpace::DataCD;
using MediaSpace::AudioCD;
using FileSpace::File;
using MediaSpace::Collection;
using DbaseSpace::Dbase;

//-----------------------------------------------------------------------------------------------------------
Dbase::Dbase() {
    fDefaultDB = 0;
    fDbaseDriver = "<UNK>";
}
//-----------------------------------------------------------------------------------------------------------
Dbase::~Dbase() {
 //fDefaultDB->close();
}
//-----------------------------------------------------------------------------------------------------------
bool Dbase::open() {
 return fDefaultDB->open();
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::close() {
 fDefaultDB->close();
}
//-----------------------------------------------------------------------------------------------------------
/**
*  Warning: if a default connection already exists that one is returned and no new connection is made
*/
void Dbase::createConnections(const QString name, const QString user, const QString pwd, const QString host) {
    
    //does a default connection already exist?
    fDefaultDB = QSqlDatabase::database( "defaultConnection", false);
    
    if (fDefaultDB == 0) {
 fDefaultDB = QSqlDatabase::addDatabase( fDbaseDriver );
 if ( ! fDefaultDB ) {
     throw (new DbaseException("Dbase::createConnections","Failed to connect to database driver '" + fDbaseDriver + "'"));
     assert(false);
 }

 fDefaultDB->setDatabaseName( name );
 fDefaultDB->setUserName( user );
 if (!pwd.isEmpty()) fDefaultDB->setPassword( pwd );
 fDefaultDB->setHostName( host );
    }   
    
    if ( ! fDefaultDB->open() ) {
 throw (new DbaseException("Dbase::createConnections",
         "Failed to open database '" + name + "@" + host 
         + "' as user '" + user + "' using driver '" + fDbaseDriver + "'"
        + "\n - Driver error: '" + lastError() + "'"));
    }
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::query(const QString s,QSqlQuery& query) {
    if (isOpen()) {
 query = QSqlQuery(s, fDefaultDB);
 if ( !query.isActive() ) throw (new DbaseException("Dbase::query(QString,QSqlQuery)","Query '" + s + "' not active"));
    } else {
 throw (new DbaseException("Dbase::query(QString,QSqlQuery)","Database not open"));
    }
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::query(const QString s,QStringList& slist) {
    if (isOpen()) {
 QSqlQuery q(s, fDefaultDB);
 if ( q.isActive() ) {
     while (q.next()) {
  QString res = q.value(0).toString();
  if (res != "") slist.push_back(res);
     }
 } else {
     throw (new DbaseException("Dbase::query(QString,QStringList)","Query '" + s + "' not active"));
 }
            
    } else {
 throw (new DbaseException("Dbase::query(QString,QStringList)","Database not open"));
    }
}
//-----------------------------------------------------------------------------------------------------------
bool Dbase::isOpen() {
 return fDefaultDB->isOpen();
}
//-----------------------------------------------------------------------------------------------------------
int Dbase::queryNum(const QString s){

 QSqlQuery query(s, fDefaultDB);

 if (query.isActive()) {
  if (query.first()) {
   return (query.value(0).toInt());
  } else {
   return -1;
  }  
 } else {
     throw (new DbaseException("Dbase::queryNum","Query '" + s + "' not active"));
 }
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::query(const QString s){
    
    if (isOpen()) {
 QSqlQuery query(s, fDefaultDB);

 if ( !query.isActive() ) {
    throw (new DbaseException("Dbase::query(QString)","Query '" + s + "' not active"
                                                                + "\n - Driver error: '" + lastError() + "'"));
}
            
    } else {
            throw (new DbaseException("Dbase::query(QString)","Database not open"));
    }

}
//-----------------------------------------------------------------------------------------------------------
Track* Dbase::getTrack(QSqlCursor& cur) {
    assert(cur.isValid());
    
    QString artist = cur.value( "artist" ).toString();
    QString title = cur.value( "title" ).toString();
    int no = cur.value( "trackno" ).toInt();
 
    Track* t = new Track(artist,title,no);
    
    t->setCDno( cur.value( "cdno" ).toInt() );
    t->setName( cur.value( "name" ).toString() );
    t->setPath( cur.value( "path" ).toString() );
    t->setDescription( cur.value( "descr" ).toString() );
    t->setSize( cur.value( "size" ).toInt() );
    t->setCreated( cur.value( "created" ).toString() );
    t->setModified( cur.value( "modified" ).toString() );
    t->setAccessed( cur.value( "accessed" ).toString() );
    t->setAttributes( cur.value( "attributes" ).toString() );
    t->setAlbum( cur.value( "album" ).toString() );
    t->setLength( cur.value( "length" ).toInt() );
    t->setBitrate( cur.value( "bitrate" ).toInt() );
    t->setSamplingRate( cur.value( "samplingrate" ).toInt() );
    t->setYear( cur.value( "year" ).toInt() );
    t->setMood( cur.value( "mood" ).toString() );
    t->setGenre( cur.value( "genre" ).toString() );
    t->setComment( cur.value( "comment" ).toString() );
    t->setScore( cur.value( "score" ).toInt() );
    t->setID( cur.value( "id" ).toInt() );
    
    return t;
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::queryTracks(const QString filter, File::Filelist& flist, const QString name){
    bool ok = false;
    
    QSqlCursor cur( name, fDefaultDB ); // Specify the table/view name
    QSqlIndex index = cur.index( "trackno" ); 
    
    if (empty(filter)) {
 ok = cur.select(index); // We'll retrieve every record
    } else {
 ok = cur.select( filter , index );   
    }
    //  cout<<"last query was: " << cur.lastQuery() << "\n";
    
    if (!ok) throw (new DbaseException("Dbase::queryTracks(..,Filelist,..)","Select query '" + cur.lastQuery() + "' failed"));
    
    while ( cur.next() ) {
 Track* t = getTrack(cur);
 //cout<<"Added track: " << (*t) << "\n";
 flist.push_back(t);
    }
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::queryTracks(const QString filter, Track::Tracklist& tlist, const QString name){
    bool ok = false;
    
    QSqlCursor cur( name, fDefaultDB ); // Specify the table/view name
    QSqlIndex index = cur.index( "trackno" ); 
 
    if (empty(filter)) {
 ok = cur.select(index); // We'll retrieve every record
    } else {
 ok = cur.select( filter, index );   
    }
    
    //cout<<"last query was: " << cur.lastQuery() << "\n";
    if (!ok) throw (new DbaseException("Dbase::queryTracks(..,Tracklist,..)","Select query '" + cur.lastQuery() + "' failed"));
    
    while ( cur.next() ) {
 Track* t = getTrack(cur);
 tlist.push_back(t);
 //cout<<"Added track: " << (*t) << "\n";
    }
    
}
//-----------------------------------------------------------------------------------------------------------
QSqlRecord* Dbase::trackToRec(const int id) {
    
    QString tmp = "";
    
    QSqlCursor cur( "Track", fDefaultDB ); // Specify the table/view name
    bool ok = cur.select("id = " + tmp.setNum(id)); 
    
    if (!ok) throw (new DbaseException("Dbase::trackToRec","Select query '" + cur.lastQuery() + "' failed"));
     
    if ( cur.next() ) {
	QSqlRecord* rec = new QSqlRecord(*cur.editBuffer(true));
	return rec;
    } else {
	//z << "edit buff returns 0\n";
    }
    
    return 0;
}
//-----------------------------------------------------------------------------------------------------------
Track* Dbase::getTrack(const int id) {
    assert(id <= 0);

    QString filter = "";
    filter = "id = " + filter.setNum(id);
    
    QSqlCursor cur( "Track", fDefaultDB ); // Specify the table/view name
    
    if ( !cur.select( filter ) ) throw (new DbaseException("Dbase::getTrack","Select query '" + cur.lastQuery() + "' failed"));   
    if ( cur.next() ) return getTrack(cur);
    
    return 0;
}
//-----------------------------------------------------------------------------------------------------------
/*bool Dbase::queryFiles(const QString s, File::Filelist& flist, const QString name) {
	return true;
}*/
//-----------------------------------------------------------------------------------------------------------
void Dbase::add(const File* f){
 assert(f != 0);
    
 QSqlCursor cur( "File", fDefaultDB ); // Specify the table/view name
    QSqlRecord *buffer = cur.primeInsert();
    
    assert(buffer != 0);
    
    double size = f->getSize();
    buffer->setValue( "cdno", f->getCDno() );
    buffer->setValue( "name", f->getName() );
    buffer->setValue( "path", f->getPath() );
    buffer->setValue( "descr", cleanSQL(f->getDescription()) );
    buffer->setValue( "size", size );
    buffer->setValue( "created", f->getCreated() );
    buffer->setValue( "modified", f->getModified() );
    buffer->setValue( "accessed", f->getAccessed() );
    buffer->setValue( "attributes", f->getAttributes() );
    
    bool ok = cur.insert();
    if (!ok) throw (new DbaseException("Dbase::add(File*)","Insert query '" + cur.lastQuery() + "' failed"));
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::add(const Track* f,const QString table){
    assert(f != 0);
    if (table.isEmpty()) throw (new DbaseException("Dbase::add(Track*)","Empty table pased"));
    
    QSqlCursor cur( table, fDefaultDB ); // Specify the table/view name
    QSqlRecord *buffer = cur.primeInsert();

    assert(buffer != 0);

    double size = f->getSize();

    buffer->setValue( "cdno", f->getCDno() );
    buffer->setValue( "name", f->getName() );
    buffer->setValue( "path", f->getPath() );
    buffer->setValue( "descr", cleanSQL(f->getDescription()) );
    buffer->setValue( "size", size );
    buffer->setValue( "created", f->getCreated() );
    buffer->setValue( "modified", f->getModified() );
    buffer->setValue( "accessed", f->getAccessed() );
    buffer->setValue( "attributes", f->getAttributes() );
    buffer->setValue( "trackno", f->getTrackNo());
    buffer->setValue( "artist", cleanSQL(f->getArtist()) );
    buffer->setValue( "title", cleanSQL(f->getTitle()) );
    buffer->setValue( "album", cleanSQL(f->getAlbum()) );
    buffer->setValue( "bitrate", f->getBitrate());
    buffer->setValue( "samplingrate", f->getSamplingRate());
    buffer->setValue( "year", f->getYear());
    buffer->setValue( "mood", cleanSQL(f->getMood()) );
    buffer->setValue( "genre", cleanSQL(f->getGenre()) );
    buffer->setValue( "comment", cleanSQL(f->getComment()) );
    buffer->setValue( "score", f->getScore());
    buffer->setValue( "length", f->getLength());

    bool ok = cur.insert();
 if (!ok) throw (new DbaseException("Dbase::add(Track*)","Insert query '" + cur.lastQuery() + "' failed"));
}
//-----------------------------------------------------------------------------------------------------------
QString Dbase::getCDTitle(const int id) {
    
    QString filter = "";
    filter.setNum(id);
    filter = "id=" + filter;
	
    QSqlCursor cur( "CD", fDefaultDB ); // Specify the table/view name
   
    if ( !cur.select( filter ) ) {
	throw (new DbaseException("Dbase::getCDTitle","Select query '" + cur.lastQuery() + "' failed"));
	return "";   
    }
    
    if ( cur.next() ) return ( cur.value( "title" ).toString() );
      
    return "";
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::updateAlbums(const int cdID,const int occurence,const QString format) {
    QString tmp = "";
    QString tmp2 = "";
        
    //give me all albums of which I have at least 'occurence' Tracks
    QString q =  "INSERT INTO Album (id, title,cdno,format,tracks) SELECT NULL,t.album,t.cdno,'" + format + "' AS format, COUNT(t.album)"
  " FROM Track AS t WHERE (t.album <> '') AND (t.album IS NOT NULL) AND (t.cdno = " + tmp.setNum(cdID) + 
  ") GROUP BY (t.album) HAVING (COUNT(t.album) >= " + tmp2.setNum(occurence) + ");";
  
    
    try {
 query(q);
    } catch (DbaseException* e) {
 throw (new DbaseException("Dbase::updateAlbums","Insert query '" + q + "' failed\n Root cause: " + e->msg()));  
    }
}
//-----------------------------------------------------------------------------------------------------------
CD* Dbase::getCD(const int id, const bool getTracks) {
    QString filter = "";
    filter.setNum(id);
    filter = "id=" + filter;
    QString type = "";
    
    QString q = "SELECT type FROM CD WHERE id = " + filter.setNum(id) + ";";
    QSqlQuery query(q, fDefaultDB);
    if (!query.isActive()) throw (new DbaseException("Dbase::getCD","Select query '" + q + "' failed"));  
    
    if (query.first()) {
	type = (query.value(0).toString());
	//cout<<"type1 is: " << type <<"\n";
    } else {
	assert(false);
	//cout<<"no type\n";
    }
    
    if (type == "MP3_CD") {
	Mp3CD* m = new Mp3CD();
	getMp3CD(m,id,getTracks);
	return m;
    } else if (type == "AUDIO_CD") {
	AudioCD* m = new AudioCD();
	getAudioCD(m,id,getTracks);
	return m;
    } else if (type == "DATA_CD") {
	DataCD* m = new DataCD();
	getDataCD(m,id,getTracks);
	return m;
    } else if (type == "MOVIE_CD") {
	MovieCD* m = new MovieCD();
	getMovieCD(m,id,getTracks);
	return m;
    } else  {
	assert(false);
	return 0;
    }
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::getMp3CD(Mp3CD* m,const int id, const bool getTracks) {
    assert(m != 0);
    QString filter = "";
    filter.setNum(id);
    filter = "id=" + filter;
    
    QSqlCursor cur( "Mp3CD", fDefaultDB ); // Specify the table/view name
    if ( !cur.select( filter ) ) throw (new DbaseException("Dbase::getMp3CD","Select query '" + cur.lastQuery() + "' failed"));  
    
    File::Filelist flist;
    if ( cur.next() ) {
 flist.clear();
 m->setID( cur.value( "id" ).toInt() );
 m->setFileCount( cur.value( "trackcount" ).toInt() );
 m->setTitle( cur.value( "title" ).toString() );
 m->setGenre( cur.value( "genre" ).toString() );
 m->setComment( cur.value( "comment" ).toString() );
 
 if (getTracks) {
     queryTracks("cdno = " + filter.setNum(m->getID()), flist);
     m->setFilelist(flist);
 }
    }
    
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::getAudioCD(AudioCD* m,const int id, const bool getTracks) {
    assert(m != 0);
    QString filter = "";
    filter.setNum(id);
    filter = "id=" + filter;
    
    QSqlCursor cur( "AudioCD", fDefaultDB ); // Specify the table/view name
    if ( !cur.select( filter ) ) throw (new DbaseException("Dbase::getAudioCD","Select query '" + cur.lastQuery() + "' failed"));  
    
    File::Filelist flist;
    if ( cur.next() ) {
 flist.clear();
 m->setID( cur.value( "id" ).toInt() );
 m->setFileCount( cur.value( "trackcount" ).toInt() );
	m->setYear( cur.value( "year" ).toInt() );
	m->setTitle( cur.value( "title" ).toString() );
	m->setArtist( cur.value( "artist" ).toString() );
	m->setGenre( cur.value( "genre" ).toString() );
	m->setComment( cur.value( "comment" ).toString() );

	if (getTracks) {
	    queryTracks("cdno = " + filter.setNum(m->getID()), flist);
	    m->setFilelist(flist);
	}
    }
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::getDataCD(DataCD* m,const int id, const bool getTracks) {
    assert(m != 0);
    QString filter = "";
    filter.setNum(id);
    filter = "id=" + filter;
    
    QSqlCursor cur( "DataCD", fDefaultDB ); // Specify the table/view name
    if ( !cur.select( filter ) ) throw (new DbaseException("Dbase::getDataCD","Select query '" + cur.lastQuery() + "' failed"));  
    
    File::Filelist flist;
    if ( cur.next() ) {
	flist.clear();
 m->setID( cur.value( "id" ).toInt() );
 //do more
 if (getTracks) {
     m->setFilelist(flist);
 }
    }
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::getMovieCD(MovieCD* m,const int id, const bool getTracks) {
    assert(m != 0);
    QString filter = "";
    filter.setNum(id);
    filter = "id=" + filter;
    
    QSqlCursor cur( "MovieCD", fDefaultDB ); // Specify the table/view name
    if ( !cur.select( filter ) ) throw (new DbaseException("Dbase::getMovieCD","Select query '" + cur.lastQuery() + "' failed"));  
    
    File::Filelist flist;
    if ( cur.next() ) {
 flist.clear();
 m->setID( cur.value( "id" ).toInt() );
 //do more
 if (getTracks) {
     m->setFilelist(flist);
 }
    }
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::addAlbum(const int cdno, const QString title, const QString format,const int tracks){
        
    QSqlCursor cur( "Album", fDefaultDB ); // Specify the table/view name
    QSqlRecord *buffer = cur.primeInsert();
    
    assert(buffer != 0);
    
    buffer->setValue( "cdno", cdno );
    buffer->setValue( "title", cleanSQL(title) );
    buffer->setValue( "format", format );
    buffer->setValue( "tracks", tracks );
    
   bool ok = cur.insert();
   if (!ok) throw (new DbaseException("Dbase::addAlbum","Insert query '" + cur.lastQuery() + "' failed")); 
  
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::addAlbum(const AudioCD* cd) {
    assert(cd != 0);
    addAlbum(cd->getID(), cd->getTitle(), "CDA", cd->getFileCount());
}
//-----------------------------------------------------------------------------------------------------------
CD* Dbase::getAlbumAsCD(const int id, const bool getTracks) {
    
    AudioCD* m = new AudioCD();
    QString filter = "";
    filter.setNum(id);
    filter = "id=" + filter;
    int cdno = 0;
    
    QSqlCursor cur( "Album", fDefaultDB ); // Specify the table/view name
    if ( !cur.select( filter ) )  throw (new DbaseException("Dbase::getAlbumAsCD","Select query '" + cur.lastQuery() + "' failed"));
    
    
    File::Filelist flist;
    if ( cur.next() ) {
	flist.clear();
	cdno = cur.value( "cdno" ).toInt();
	m->setID( cur.value( "id" ).toInt() );
	m->setFileCount( cur.value( "tracks" ).toInt() );
	m->setYear( 0 );
	m->setTitle( cur.value( "title" ).toString() );
	m->setArtist( "" );
	m->setGenre( "" );
	m->setComment( "" );
	
	if (getTracks) {
	    queryTracks("cdno = " + filter.setNum(cdno) + " AND album = \"" + m->getTitle() + "\"", flist);
	    m->setFilelist( flist );
	}
    }
    
    return m;

}
//-----------------------------------------------------------------------------------------------------------
void Dbase::addToCollection(CD* m, const Collection* c){
    assert(m != 0);
    assert(c != 0);
    
    QString colTit = cleanSQL(c->getTitle());
    QString tmp = "";
    
    if (colTit != "") {
 
 QString q = "INSERT INTO Contains (title,id) VALUES (\"";
 
 q += colTit + "\",";
 
 tmp.setNum( m->getID() );  
 
 q += tmp + ");";
 
 query(q);
 
 QString type = m->getTypeString();
 if (type == "MP3_CD") {
     q = "UPDATE Collection SET mp3CDcount = mp3CDcount + 1 WHERE title = \"";
 } else if (type == "AUDIO_CD") {
     q = "UPDATE Collection SET audioCDcount = audioCDcount + 1 WHERE title = \"";
 } else if (type == "MOVIE_CD") {
     q = "UPDATE Collection SET movieCDcount = movieCDcount + 1 WHERE title = \"";
 } else if (type == "DATA_CD") {
     q = "UPDATE Collection SET dataCDcount = dataCDcount + 1 WHERE title = \"";
 } else {
     assert(false);
 }
 q += colTit + "\";";
 query(q);
    }
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::add(const Track::Tracklist& tlist,const QString table) {

    Track::ConstTrackIterator TI = tlist.begin();
    while ( TI != tlist.end()) {
 add(*TI,table);
 ++TI;
    }
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::addCD(const CD* cd, const QString type) {
    assert(cd != 0);
    
    QString tmp = cleanSQL(cd->getTitle());
    query("INSERT INTO CD (title, type) VALUES (\"" + tmp + "\",\"" + type + "\");");
    
/*    QSqlCursor cur( "CD", fDefaultDB ); // Specify the table/view name
    QSqlRecord *buffer = cur.primeInsert();
    
    assert(buffer != 0);
    
    buffer->setValue( "type", type );
    buffer->setValue( "title", cd->getTitle() ); 
    
   return cur.insert(); */
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::add(Mp3CD* const m ,QProgressBar* pBar){
    
    assert(m != 0);
    bool ispbar = (pBar != 0);
    bool ok = false;
    try {
 addCD(m,"MP3_CD");
    } catch (DbaseException* e) {
 throw (new DbaseException("Dbase::add(Mp3CD,..)","Failed to add CD '" + QString(m->getTitle()) 
      + "' to the collection\n Root cause: " + e->msg()));
    }
        
    int id = lastInsertID("CD");
    //cout<<"Last insert Id is: " << id <<"\n";
    
    if (id < 0) throw (new DbaseException("Dbase::add(Mp3CD,..)","Last insert ID is < 0"));
    
    m->setID(id);
    
    QSqlCursor cur( "Mp3CD", fDefaultDB ); // Specify the table/view name
    QSqlRecord *buffer = cur.primeInsert();
    
    assert(buffer != 0);
  
    buffer->setValue( "id",  id);
    buffer->setValue( "trackcount", m->getFileCount() );
    buffer->setValue( "title", cleanSQL(m->getTitle()) );
    buffer->setValue( "genre", cleanSQL(m->getGenre()) );
    buffer->setValue( "comment", cleanSQL(m->getComment()) );
    
   ok = cur.insert();

   if (!ok) throw (new DbaseException("Dbase::add(Mp3CD,..)","Insert query '" + cur.lastQuery() + "' failed"));
   if (ispbar) {
       pBar->reset();
       pBar->setTotalSteps(m->getFileCount());
   }
   
   int i = 1;
   File::ConstFileIterator TI = m->getConstBegin();
   int cdno = m->getID();
   while (TI != m->getConstEnd()) {
       (*TI)->setCDno(cdno);
       Track* t = dynamic_cast<Track*>(*TI);
       assert(t != 0);
       this->add(t);
       ++TI; 
       if (ispbar) pBar->setProgress(i);
       ++i;
   }
   
   if (ispbar) pBar->reset();
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::add(AudioCD* const m, QProgressBar* pBar){
        
    assert(m != 0);
    bool ispbar = (pBar != 0);
    bool ok = false;
     try {
 addCD(m,"AUDIO_CD");
 
    } catch (DbaseException* e) {
 throw (new DbaseException("Dbase::add(AudioCD,..)","Failed to add CD '" + m->getTitle() + "' to the collection"));
    }
    
    int id = lastInsertID("CD");
    //cout<<"Last insert Id is: " << id <<"\n";
    
    if (id < 0) throw (new DbaseException("Dbase::add(AudioCD,..)","Last insert ID is < 0"));
        
    m->setID(id);
    
    QSqlCursor cur( "AudioCD", fDefaultDB ); // Specify the table/view name
    QSqlRecord *buffer = cur.primeInsert();
    
    assert(buffer != 0);
    
    buffer->setValue( "id",  id );
    buffer->setValue( "trackcount", m->getFileCount() );
    buffer->setValue( "title", cleanSQL(m->getTitle()) );
    buffer->setValue( "artist", cleanSQL(m->getArtist()) );
    buffer->setValue( "genre", cleanSQL(m->getGenre()) );
    buffer->setValue( "year", m->getYear() );
    buffer->setValue( "comment", cleanSQL(m->getComment()) );
    
   ok = cur.insert();
   
   if (!ok) throw (new DbaseException("Dbase::add(AudioCD,..)","Insert query '" + cur.lastQuery() + "' failed"));
   if (ispbar) {
       pBar->reset();
       pBar->setTotalSteps(m->getFileCount());
   }
   
   int i = 1;
   File::ConstFileIterator TI = m->getConstBegin();
   int cdno = m->getID();
   while (TI != m->getConstEnd()) {
       (*TI)->setCDno(cdno);
       Track* t = dynamic_cast<Track*>(*TI);
       assert(t != 0);
       this->add(t);
       ++TI; 
       if (ispbar) pBar->setProgress(i);
       ++i;
   }
   
   if (ispbar) pBar->reset();
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::add(MovieCD* const m){
    assert(m != 0);
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::add(DataCD* const m){
    assert(m != 0);
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::add(const Collection* c){
    assert(c != 0);
    
    QSqlCursor cur( "Collection", fDefaultDB ); // Specify the table/view name
    QSqlRecord *buffer = cur.primeInsert();
    
    assert(buffer != 0);
    
    buffer->setValue( "owner", cleanSQL(c->getOwner()) );
    buffer->setValue( "mp3CDcount",c->getMp3CDCount() );
    buffer->setValue( "audioCDcount",c->getAudioCDCount() );
    buffer->setValue( "dataCDcount",c->getDataCDCount() );
    buffer->setValue( "movieCDcount",c->getMovieCDCount() );
    buffer->setValue( "title", cleanSQL(c->getTitle()) );
    buffer->setValue( "descr", cleanSQL(c->getDesc()) );
    buffer->setValue( "comment", cleanSQL(c->getComment()) );
    
    bool ok = cur.insert();
    if (!ok) throw (new DbaseException("Dbase::add(Collection*,..)","Insert query '" + cur.lastQuery() + "' failed"));
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::deleteCD(const int id,const QString type,const QString collection) {
    QString tmp = "";
    QString col = cleanSQL(collection);
    
    try {
 query("DELETE FROM Contains WHERE id = " + tmp.setNum(id) + ';'); 
 if (type=="MP3_CD") {
     clearTable("Track","cdno = " + tmp.setNum(id));
     clearTable("Album", "cdno = " + tmp.setNum(id));
     clearTable("Mp3CD", "id = " + tmp.setNum(id));
     query("UPDATE Collection SET mp3CDcount = mp3CDcount - 1 WHERE (title = \"" + col + "\") AND (mp3CDcount > 0);"); 
 } else if (type=="AUDIO_CD") {
     clearTable("Track","cdno = " + tmp.setNum(id));
     clearTable("Album","cdno = " + tmp.setNum(id));
     clearTable("AudioCD" ,"id = " + tmp.setNum(id));
     query("UPDATE Collection SET audioCDcount = audioCDcount - 1 WHERE (title = \"" + col + "\") AND (audioCDcount > 0);");    } else if (type=="MOVIE_CD") {
     clearTable("File", "cdno = " + tmp.setNum(id));
     clearTable("MovieCD", "id = " + tmp.setNum(id));
     query("UPDATE Collection SET movieCDcount = movieCDcount - 1 WHERE (title = \"" + col + "\") AND (movieCDcount > 0);");    } else if (type=="DATA_CD") {
	    clearTable("File", "cdno = " + tmp.setNum(id));
	    clearTable("DataCD", "id = " + tmp.setNum(id));
	    query("UPDATE Collection SET dataCDcount = dataCDcount - 1 WHERE (title = \"" + col + "\") AND (dataCDcount > 0);");	
	} else  {
	    assert(false);
 }
 
 clearTable("CD","id = " + tmp.setNum(id));
    } catch (DbaseException* e) {
 throw (new DbaseException("Dbase::deleteCD","Delete CD (id=" + tmp.setNum(id) + ") failed, error: '" + QString(e->msg()) + "'"));
    }
}
//-----------------------------------------------------------------------------------------------------------
QString Dbase::getDriver() const{
 return fDbaseDriver;
}
//-----------------------------------------------------------------------------------------------------------
QString Dbase::getName() const{
 return fDefaultDB->databaseName();
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::setName(const QString s){
 fDefaultDB->setDatabaseName(s);
}
//-----------------------------------------------------------------------------------------------------------
QString Dbase::getUser() const{
 return fDefaultDB->userName();
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::setUser(const QString s){
 fDefaultDB->setUserName(s);
}
//-----------------------------------------------------------------------------------------------------------
QString Dbase::getPwd() const{
 return fDefaultDB->password();
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::setPwd(const QString s){
 fDefaultDB->setPassword(s);
}
//-----------------------------------------------------------------------------------------------------------
QString Dbase::getHost() const{
 return fDefaultDB->hostName();
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::setHost(const QString s){
 fDefaultDB->setHostName(s);
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::setDriver(const QString driver) {
    fDbaseDriver = driver;
}
//-----------------------------------------------------------------------------------------------------------
QString Dbase::lastErrorDriver() const {
    return fDefaultDB->lastError().driverText();
}
//-----------------------------------------------------------------------------------------------------------
QString Dbase::lastErrorDbase() const {
    return fDefaultDB->lastError().databaseText();
}
//-----------------------------------------------------------------------------------------------------------
QString Dbase::lastError() const {
    return "Driver error: '" + lastErrorDriver() + "' Dbase error: '" + lastErrorDbase() + "'";
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::clearTable(const QString table, const QString filter) {
    if (empty(filter)) {
 query("DELETE FROM " + table + ";");
    } else {
 query("DELETE FROM " + table + " WHERE " + filter + ";");
    }
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::dropTable(const QString table) {
    query("DROP TABLE " + table + ";");
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::getCollection(const QString title, Collection* col) {
    assert(col != 0);
    QString filter = "title = '" + cleanSQL(title) + "'";
    
    QSqlCursor cur( "Collection", fDefaultDB ); // Specify the table/view name
    if ( !cur.select( filter ) ) throw (new DbaseException("Dbase::getCollection","Select query '" + cur.lastQuery() + "' failed"));
    
    if ( cur.next() ) {
	col->setOwner( cur.value( "owner" ).toString() );
	col->setDesc( cur.value( "descr" ).toString() );
	col->setTitle( cur.value( "title" ).toString() );
	col->setMp3CDCount( cur.value( "mp3CDcount" ).toInt() );
	col->setAudioCDCount( cur.value( "audioCDcount" ).toInt() );
	col->setMovieCDCount( cur.value( "movieCDcount" ).toInt() );
	col->setDataCDCount( cur.value( "dataCDcount" ).toInt() );
	col->setComment( cur.value( "comment" ).toString() );
    }
 
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::getCollectionTitles(QStringList& slist) {
    query("SELECT title FROM Collection",slist);
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::getGenres(QStringList& slist, QString where) {
 if(where.isEmpty()) {
  query("SELECT genre FROM Track GROUP BY genre",slist);
 } else {
  query("SELECT genre FROM Track WHERE " + where + " GROUP BY genre",slist);
 }
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::getArtists(QStringList& slist, QString where) {
 if(where.isEmpty()) {
  query("SELECT artist FROM Track GROUP BY artist",slist);
 } else {
  query("SELECT artist FROM Track WHERE " + where + " GROUP BY artist",slist);
 }
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::getAlbums(QStringList& slist, QString where) {
 if(where.isEmpty()) {
  query("SELECT album FROM Track GROUP BY album",slist);
 } else {
  query("SELECT album FROM Track WHERE " + where + " GROUP BY album",slist);
 }
}
//-----------------------------------------------------------------------------------------------------------
QString Dbase::cleanSQL(const QString s){
   QString out = s;
   //NB: Qt generated queries use single quotes
   //To be safe, remove all quotes
   out = out.replace('\'',"");
   out = out.replace('"',"");
   
   out = out.replace('(',"");
   out = out.replace(')',"");
   out = out.replace('[',"");
   out = out.replace(']',"");
   return out;
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::deleteFromTable(const QString filter, const QString table){
    query("DELETE FROM " + table + " WHERE " + filter);
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::getIdTitles(QStringList& idList, QStringList& titleList, const QString table){
    
   QSqlCursor cur( table, fDefaultDB ); // Specify the table/view name
   QSqlIndex index = cur.index( "id" ); 
   QString id = "";
   QString title = "";
   
    if ( !cur.select( index ) ) throw (new DbaseException("Dbase::getIdTitles","Select query '" + cur.lastQuery() + "' failed"));   
    
    while ( cur.next() ) {
 id = cur.value( "id" ).toString();
 title = cur.value( "title" ).toString();
 idList.append(id);
 titleList.append(title);
    }
}
//-----------------------------------------------------------------------------------------------------------
int Dbase::getResultSetSize(const QString s){
    QSqlQuery query;
    this->query(s,query);
   // cout<<"size " << query.size() <<"\n";
    return query.size();
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::deleteCollection(const QString title) {
    QStringList slist;
    query("SELECT id FROM Contains WHERE title = \"" + title + "\";",slist);
    //cout<<"title is '" << title << "'\n";
    QStringList::const_iterator SI = slist.begin();
    
    query("DELETE FROM Contains WHERE title = \"" + title + "\";");
    query("DELETE FROM Collection WHERE title = \"" + title + "\";");
    
    while (SI != slist.end()) {
	query("DELETE FROM File WHERE cdno = " + *SI + ";");
	query("DELETE FROM Track WHERE cdno = " + *SI + ";");
	query("DELETE FROM Album WHERE cdno = " + *SI + ";");
 query("DELETE FROM Mp3CD WHERE id = " + *SI + ";");
 query("DELETE FROM AudioCD WHERE id = " + *SI + ";");
 query("DELETE FROM DataCD WHERE id = " + *SI + ";");
 query("DELETE FROM MovieCD WHERE id = " + *SI + ";");
 query("DELETE FROM CD WHERE id = " + *SI + ";");
 ++SI;
    }
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::getIDsFromTable(const QString table, QStringList& list) {
    query("SELECT id FROM " + table + ";",list);
} 
//-----------------------------------------------------------------------------------------------------------
bool Dbase::cdTitleExists(const QString title) {
    if (isOpen()) {
     QString q = "SELECT title FROM CD WHERE title=\"" + title + "\";";
 //cout<<"query: " << q << "\n";
 return (this->getResultSetSize(q) > 0);
    } else {
 throw (new DbaseException("Dbase::cdTitleExists","Database not open"));
    }
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::scoreTrack(const int id, const uint score) {
 query("UPDATE Track SET score = " + QString::number(score) + " WHERE id = " + QString::number(id) + ";");
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::scoreSameArtist(const int id, const uint score) {
 query("UPDATE Track AS t1,Track AS t2 SET t2.score =  " + QString::number(score) 
       + " WHERE t2.artist=t1.artist AND t1.id = " + QString::number(id) + ";");
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::scoreSameCD(const int id, const uint score) {
 query("UPDATE Track AS t1,Track AS t2 SET t2.score =  " + QString::number(score) 
       + " WHERE t2.cdno=t1.cdno AND t1.id = " + QString::number(id) + ";");
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::scoreSameAlbum(const int id, const uint score) {
 query("UPDATE Track AS t1,Track AS t2 SET t2.score =  " + QString::number(score) 
       + " WHERE t2.album=t1.album AND t1.id = " + QString::number(id) + ";");
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::scoreSameGenre(const int id, const uint score){
 query("UPDATE Track AS t1,Track AS t2 SET t2.score =  " + QString::number(score) 
       + " WHERE t2.genre=t1.genre AND t1.id = " + QString::number(id) + ";");
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::setMood(const int id, const QString mood) {
 query("UPDATE Track SET mood = \"" + mood + "\" WHERE id = " + QString::number(id) + ";");
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::renameAlbum(const QString oldName, const QString newName) {
 //TODO rely on referential integrity instead
 query("UPDATE Album SET title = \"" + cleanSQL(newName) + "\" WHERE title = \"" + oldName + "\";");
 query("UPDATE Track SET album = \"" + cleanSQL(newName) + "\" WHERE album = \"" + oldName + "\";");
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::moodSameAlbum(const int id, const QString mood) {
 query("UPDATE Track AS t1,Track AS t2 SET t2.mood =  \"" + mood 
       + "\" WHERE t2.album=t1.album AND t1.id = " + QString::number(id) + ";");
}
//-----------------------------------------------------------------------------------------------------------
void Dbase::moodSameCD(const int id, const QString mood) {
 query("UPDATE Track AS t1,Track AS t2 SET t2.mood =  \"" + mood 
       + "\" WHERE t2.cdno=t1.cdno AND t1.id = " + QString::number(id) + ";");
}
 //-----------------------------------------------------------------------------------------------------------
