/* --------------------------------------------------------------------------

   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#ifndef INC_DBASE_H
#define INC_DBASE_H

#include <vector>
#include "cd.h"
#include "mp3cd.h"
#include "moviecd.h"
#include "datacd.h"
#include "audiocd.h"
#include "track.h"
#include "file.h"
#include "collection.h"

#include <qsqlcursor.h>
#include <qsqldatabase.h>
#include <qsqlquery.h>
#include <qstringlist.h>
#include <qprogressbar.h>

using std::vector;
using TrackSpace::Track;
using MediaSpace::Mp3CD;
using MediaSpace::MovieCD;
using MediaSpace::DataCD;
using MediaSpace::AudioCD;
using FileSpace::File;
using MediaSpace::Collection;

namespace DbaseSpace {
        
        class Dbase {
        private:
                QString fDbaseDriver;
                QSqlDatabase* fDefaultDB;
        protected: 
        public:
                Dbase();
                virtual ~Dbase();
                
                bool isOpen();
                bool open();
                void close();
                
                void query(const QString);
                void query(const QString, QStringList&);
                void query(const QString, QSqlQuery&);
                void queryTracks(const QString, Track::Tracklist&, const QString="Track");
                void queryTracks(const QString, File::Filelist&, const QString="Track");
                int queryNum(const QString s);
                
                void deleteFromTable(const QString filter, const QString table);
                //  bool queryFiles(const QString, File::Filelist&, const QString="File");
                
                void add(const File*);
                void add(const Track*,const QString table="Track");
                void add(const Track::Tracklist&,const QString table="Track");
                
                void add(Mp3CD* const cd,QProgressBar* pBar=0);
                void add(AudioCD* const cd,QProgressBar* pBar=0);
                void add(MovieCD* const cd);
                void add(DataCD* const cd);
                void addCD(const CD*, QString type);
                
                void add(const Collection*);
                void addToCollection(CD*, const Collection*);
                void addAlbum(const int cdno, const QString title, const QString format, const int tracks);
                void addAlbum(const AudioCD* cd);
                
                QSqlRecord* trackToRec(const int);
                
                void deleteCD(const int id, const QString type,const QString col);
                void deleteCollection(const QString title);
                
                void updateAlbums(const int cdID,const int occurence,const QString format);
                void clearTable(const QString table, const QString filter="");
                void dropTable(const QString table);
                
                void getIDsFromTable(const QString table, QStringList& list);
                Track* getTrack(QSqlCursor& cur);
                Track* getTrack(const int id);
                QString getCDTitle(const int id);
                void scoreTrack(const int id, const uint score);
                void scoreSameArtist(const int id, const uint score);
                void scoreSameCD(const int id, const uint score);
                void scoreSameAlbum(const int id, const uint score);
                void scoreSameGenre(const int id, const uint score);
                void setMood(const int id, const QString mood);
                void moodSameCD(const int id, const QString mood);
                void moodSameAlbum(const int id, const QString mood);
                
                CD* getCD(const int id, const bool getTracks=true);
                void getMp3CD(Mp3CD*,const int id, const bool getTracks=true);
                void getAudioCD(AudioCD*,const int id, const bool getTracks=true);
                void getDataCD(DataCD*,const int id, const bool getTracks=true);
                void getMovieCD(MovieCD*,const int id, const bool getTracks=true);
                CD* getAlbumAsCD(const int id, const bool getTracks=true);
                
                bool cdTitleExists(const QString title);
                void getCollection(const QString title, Collection* col);
                void getCollectionTitles(QStringList& list);
                void getGenres(QStringList& slist, QString where="");
                void getArtists(QStringList& slist, QString where="");
                void getAlbums(QStringList& slist, QString where="");
                void renameAlbum(const QString oldName, const QString newName);
                
                QString getName() const;
                void setName(const QString);
                
                QString getUser() const;
                void setUser(const QString);
                
                QString getPwd() const;
                void setPwd(const QString);
                
                QString getHost() const;
                void setHost(const QString);
                
                void setDriver(const QString driver);
                QString getDriver() const;
                
                static QString cleanSQL(const QString s);
                void getIdTitles(QStringList& idList, QStringList& titleList, const QString table);
                
                QString lastErrorDriver() const;
                QString lastErrorDbase() const;
                QString lastError() const;
                
                int getResultSetSize(const QString s);
                virtual void createConnections(const QString n, const QString u, const QString p, const QString h);
                
                //Database dependent methods, need to be implemented by subclasses
                //See mysqldbase.cpp for an example
                virtual void createNewAccount(const QString rootUser, 
                                              const QString rootPass, 
                                              const QString user, 
                                              const QString pass, 
                                              const QString host, 
                                              const QString dbase,
                                              const QString connectDb="")=0;
                virtual int lastInsertID(const QString table)=0;
                virtual QString getExistingDbase() const=0;
                virtual QString getVendor() const=0;
                virtual void createTables()=0;
                virtual void createTmpTrackTable(const QString name)=0;
                virtual void createDbase(const QString  name)=0;    
                virtual QString buildRegexpWhereClause(const QString field, QStringList& slist)=0;
                virtual QStringList backupCommand(const QString user, const QString pwd,
                                                  const QString dbase, const QStringList extraSwitches) const=0;
        };
        
}

#endif
