/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_DBASEBACKUP_H
#include "dbasebackup.h"
#define INC_DBASEBACKUP_H
#endif

#ifndef INC_DOMOGLOBALS_H
#include "domoglobals.h"
#define INC_DOMOGLOBALS_H
#endif

#include <iostream>
#include "qdir.h"
#include "qobject.h"

using DbaseSpace::Dbase;

using std::cout;
//-----------------------------------------------------------------------------------------------------------
DbaseBackup::DbaseBackup(const QString user, 
			 const QString pwd, 
			 const QString dbase, 
			 const QString file,
			 const QString dir,
			 const QString switches,
			 const Dbase* db)
{

    fFile = file;
    fWriter = new WriterTxt(fFile);
    DomoGlobals::logger->debug("DbaseBackup::DbaseBackup","Opening output stream to '" + fFile + "'");        

    QString s = "-- Created by DOMO v" + DomoGlobals::DOMO_VERSION + "\n"
		+ "-- To use this file to restore you database:\n"
		+ "--       Create a new database [database]\n"
		+ "--       Execute \"" + db->getVendor() + " [database] < [this file]\"\n"
		+ "--\n"
		+ "-- (C) GoriTech Domo is available for free under the GNU GPL at http://domo.sf.net/\n"
		+ "-- Please send all bugs/comments/requests to Dirk Gorissen <" + DomoGlobals::AUTHOR_MAIL+">\n"
		+ "--\n\n";

    fWriter->writeHeader(s);

    fProc = new QProcess( db->backupCommand(user,pwd,dbase,switches) );
    
    if(!dir.isEmpty()) fProc->setWorkingDirectory( QDir(dir) );
    DomoGlobals::logger->debug("DbaseBackup::DbaseBackup","Process working directory set to '" + dir + "'");        
    
    connect( fProc, SIGNAL(processExited()),this,SLOT(backupFinished() ));
    connect( fProc, SIGNAL(readyReadStdout()),this, SLOT(readFromStdout()) );
    connect( fProc, SIGNAL(readyReadStderr()),this, SLOT(readFromStdout()) );
}

//-----------------------------------------------------------------------------------------------------------
DbaseBackup::~DbaseBackup() {
    delete fProc;
}
//-----------------------------------------------------------------------------------------------------------
bool DbaseBackup::start() {
    if ( !fProc->launch("") ) {
	DomoGlobals::logger->error("DbaseBackup::start","Backup process '" + fProc->arguments().join(" ") + "' failed to start");
	return false;
    } else {
	DomoGlobals::logger->info("DbaseBackup::start",
				  "Backup process '" + fProc->arguments().join(" ") + "' started..."); //with PID " + long(fProc->processIdentifier()));
	return true;
    }
}

//-----------------------------------------------------------------------------------------------------------
void DbaseBackup::readFromStdout() {
    QString s(fProc->readStdout());
    
    //DomoGlobals::logger->debug("DbaseBackup::readFromStdout","Read: '" + s + "'");
    
    fWriter->writeLn(s);
    fWriter->flush();
}
//-----------------------------------------------------------------------------------------------------------
void DbaseBackup::backupFinished() {
    DomoGlobals::logger->info("DbaseBackup::backupFinished","Backup process finished");
}
//-----------------------------------------------------------------------------------------------------------
