/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#include "dbase.h"
#include "pgsqldbase.h"
#include "mysqldbase.h"
#include "odbcdbase.h"
#include "dbasefactory.h"
#include "domoglobals.h"
#include "assert.h"

using  std::cout;
using namespace DbaseSpace;
//-----------------------------------------------------------------------------------------------------------
DbaseFactory::DbaseFactory() {
    
}
//-----------------------------------------------------------------------------------------------------------
DbaseFactory::~DbaseFactory() {
}
//-----------------------------------------------------------------------------------------------------------
Dbase* DbaseFactory::createDbase(const QString drv) {
    QString driver = drv.upper();
    DomoGlobals::logger->debug("DbaseFactory::createDbase","Driver passed to Database factory is '" + driver + "'");
    
    Dbase* dbase = 0;
    
     if  (driver == "QMYSQL3"){
	dbase = new MySqlDbase();
    } else if (driver == "QPSQL7") {
	dbase = new PgSqlDbase();
    } else if (driver == "QODBC3") {
	dbase = new OdbcDbase();
    } else {
	DomoGlobals::logger->error("DbaseFactory::createDbase","Database driver '" + driver 
				     + "' unknown, assuming MySQL");
	dbase = new MySqlDbase();
    }
    
    return dbase;
}
//-----------------------------------------------------------------------------------------------------------
QString DbaseFactory::existingDatabase(const QString drv) {
	Dbase* db = createDbase(drv);
	QString dbase = db->getExistingDbase();
	delete db;
	return dbase;
}
//-----------------------------------------------------------------------------------------------------------
