/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_DIRSCANNER_H
#include "dirscanner.h"
#define INC_DIRSCANNER_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#ifndef INC_METAINFO_H
#include "metainfo.h"
#define INC_METAINFO_H
#endif

#ifndef INC_FILEIO_H
#include "fileio.h"
#define INC_FILEIO_H
#endif

#include "musicexpression.h"
#include "domoglobals.h"
#include "readerfactory.h"
#include "qdatetime.h"
#include "qstringlist.h"
#include "assert.h"
#include "qregexp.h"
#include "cleandirdialog.h"
#include <iostream>

using  std::cout;
using FileSpace::File;
using TrackSpace::Track;
using DirScannerSpace::DirScanner;
using ResourceSpace::RcMan;
using ReaderSpace::ReaderFactory;
using ReaderSpace::IReader;

//-----------------------------------------------------------------------------------------------------------
DirScanner::DirScanner(RcMan* rc, Settings* set, const QString path) {
	assert(rc != 0);
	assert(set != 0);
	
	fPath = path;
	fRcMan = rc;
	fSettings = set;
}
//-----------------------------------------------------------------------------------------------------------
DirScanner::~DirScanner(){
    //pointer fRcMan not owned by dirscanner
}
//-----------------------------------------------------------------------------------------------------------
QString DirScanner::getPath() const {
    return fPath;
}
//-----------------------------------------------------------------------------------------------------------
void DirScanner::setPath(const QString s) {
    fPath = s;
}
//-----------------------------------------------------------------------------------------------------------
void DirScanner::scanDir(QString& path, QString& filter, QFileInfoList& flist, const bool recurse, const bool incDirs) {
    QDir d(path);
   
    if (!d.exists(path)) throw (new IoException("DirScanner::scanDir", "Path '" + path + "' does NOT exist"));
    
    d.setPath(path);
    DomoGlobals::logger->debug("DirScanner::scanDir","Starting scan of directory '" + path + "'");
    
    if (!d.isReadable()) throw (new IoException("DirScanner::scanDir", "Path '" + path + "' is NOT readable!"));

    const QFileInfoList* dirList;
    const QFileInfoList* tmpList;
    QString dirAbsPath="";
    
    if (incDirs) {
	tmpList = d.entryInfoList("*. " + filter, QDir::Files|QDir::Dirs, QDir::Name);	
    }else {
	tmpList = d.entryInfoList(filter, QDir::Files, QDir::Name);	//add files in directory to list
    }
    
    QFileInfoListIterator it( *tmpList );
    QFileInfo* fi2 = 0;
    QString fname = "";
    while ( (fi2 = it.current()) != 0 ) {
	if (fi2->isDir()) {
	    fname = fi2->dirPath();
	    // f << "dir found: " << fname << "\n";
	} else {
	    fname = fi2->fileName();
	}
	
	if ((fname != path) && (fname != "..") && (fname != ".")) {
		if(fi2->isReadable()) {
			QFileInfo* p = new QFileInfo(*fi2);
			p->setCaching(false);
			flist.append(p);
			//f << "     added file: " << fname << "\n";
		} else {
			DomoGlobals::logger->warning("DirScanner::scanDir",
						     "File '" + fi2->absFilePath() + "' NOT readable, not added to list");
		}
	}
	++it;
    }
    
    
    dirList = d.entryInfoList(QDir::Dirs, QDir::Name); 	//get the list of sub directories
    if (!recurse||(dirList == 0)||(dirList->isEmpty())) {//'path' contains sub dirs
	//f << "recursion stopped" << "\n";
	//recursion stops		
    } else {
	QFileInfoListIterator it3( *dirList );
	QFileInfo* fi = 0;
	
	while ( (fi = it3.current()) != 0 ) {
	    if ((fi->fileName() != ".") && (fi->fileName() != "..")  && (fi->isDir()) ) {
		dirAbsPath = fi->absFilePath();					//get the absolute path of the directory
		d.setPath(dirAbsPath);						//change to that directory
		
		if (d.isReadable()) {
			//f << "going to scan: " << dirAbsPath << "\n";
			this->scanDir(dirAbsPath, filter, flist, recurse, incDirs);	    //scan contents (recurse)	
		} else {
			DomoGlobals::logger->warning("DirScanner::scanDir", "Path '" + dirAbsPath + "' is NOT readable! Skipping directory.");
		}
	    }
	    ++it3;
	}
    }
	              
}
//-----------------------------------------------------------------------------------------------------------
File::Filelist DirScanner::indexDirectory(const QString mask, const bool recurse, const bool incDirs){
    QFileInfoList flist;
    File::Filelist fl;
    QString ext = "";
    
    QString pth = fPath;
    QString msk = mask;
    
    this->scanDir(pth, msk, flist, recurse, incDirs);
    
    
    QFileInfoListIterator it( flist );
    QFileInfo fi;
    
    while ( it.current() != 0 ) {
	fi = (*it.current());	
	ext = fi.extension();
	File* f = new File(fi.fileName(), fRcMan->getExtDesc(ext),fi.size());
	assert(f != 0);

 	f->setHidden(fi.isHidden());
	f->setCreated(fi.created().toString("yyyy.MM.dd  h:m:s"));
	f->setModified(fi.lastModified().toString("yyyy.MM.dd  h:m:s"));
	f->setAccessed(fi.lastRead().toString("yyyy.MM.dd  h:m:s"));
	    
	pth = fi.absFilePath();
	f->setPath(pth);
	f->setDirectory(fi.isDir());
	f->setReadOnly(fi.isReadable());
	    
	fl.push_back(f);
	++it;
    }
    flist.clear();
    return fl;
}
//-----------------------------------------------------------------------------------------------------------
QString DirScanner::makeAlbum(const QString& trackPath, const QString& scanPath) {
    QString s1 = (QDir::convertSeparators(trackPath));
    QString s2 = (QDir::convertSeparators(scanPath));
    
    /* may not happen in all cases
    QChar lastChar = s2[s2.length()-1];
    if (QString(lastChar) == DomoGlobals::DIR_SEP) {
	//pop '/' (or '\') if scanpath ends with a directory separator
	//to make sure subtraction always works
	s2 = s2.left(s2.length()-1);
    } */

    QString s3 = subtract(s1,s2);
   
    //DomoGlobals::logger->debug("DirscannerSpace::makeAlbum","Subtract: '" + s1 + "' - '" + s2 + "' = '" + s3 + "'");
    
    QString album = MusicExpression::fitToPattern(s3,
				  fSettings->get(Settings::fAlbumSrcPattern),
				  fSettings->get(Settings::fAlbumDestPattern));
    
    if(s3 != "") DomoGlobals::logger->debug("DirscannerSpace::makeAlbum",
					    "String in '" + s3 + "' album out '" + album + "'");
   
    return cleanFilename(album);
}
//-----------------------------------------------------------------------------------------------------------
Track::Tracklist DirScanner::indexMusicDirectory(const QString mask, const bool recurse, const bool incDirs, QProgressBar* pBar){
	QFileInfoList flist;
	Track::Tracklist fl;
	QString ext = "";
	QString tmp = "";
	QString filename = "";
	QString newFilename = "";
	QString pth = fPath;
	QString msk = mask;
	
	//Did the user specify a custom format that filenames are in
	bool useCustomTrackFormat = fSettings->getBool(Settings::fUseCustomTrackFormat);
	QString customTrackFormat = fSettings->get(Settings::fCustomTrackFormat);
	if(useCustomTrackFormat) 
		DomoGlobals::logger->debug("DirScanner::indexMusicDirectory","Using custom track format '" + customTrackFormat + "'");

	this->scanDir(pth, msk, flist, recurse, incDirs);
	
	QFileInfoListIterator it( flist );
	QFileInfo fi;
	
	//asks the user if tracks should be cleaned when scanning (remove underscores, make large case, etc)
	CleanDirDialog* dialog = new CleanDirDialog();
	dialog->setDirectory(pth);
	bool cleanDir = (dialog->exec() == QDialog::Accepted);
	if(!cleanDir) {
		DomoGlobals::logger->debug("DirScanner::indexMusicDirectory","Skipping filename clean...");
	} else {
		DomoGlobals::logger->debug("DirScanner::indexMusicDirectory","Enabling filename clean...");
	}
	
	pBar->reset();
	pBar->setTotalSteps(flist.count());
	int pi = 0;	
	int i = 1;
	while ( it.current() != 0 ) {
		fi = (*it.current());	
		
		DomoGlobals::logger->debug("DirScanner::indexMusicDirectory","Handling file '" + fi.absFilePath() + "'");
		
		ext = fi.extension();
		
		filename = fi.fileName();
		if(cleanDir) {
			//cleanup the filename
			newFilename = dialog->cleanFile(filename,fi.dirPath(false),fRcMan);
			if(newFilename != filename) {
				//only rename if the filename was actually changed
				if (FileIO::renameFile(fi.absFilePath(),fi.dirPath(true) + QDir::separator() + newFilename)) {
					DomoGlobals::logger->info("Dirscanner::indexMusicDirectory",
								  "File '" + filename + "' cleaned to '" + newFilename + "'");
					filename = newFilename;
					
				} else {
					DomoGlobals::logger->warning("Dirscanner::indexMusicDirectory",
								     "Failed to clean '" + filename + "' to '" + newFilename + "'");
				}
			}
			if(dialog->writeTags()) dialog->writeTags(fi.dirPath(true), filename);
		}
		
		Track* trk = 0;
		if(useCustomTrackFormat) {
			trk = MusicExpression::makeTrack(customTrackFormat,filename);
			trk->setName(filename);
		} else {
			//try to strip leading track numbers
			if ( ! fRcMan->startsWithSafeArtist(filename)) {
				QString goodFilename = filename;
				//strip away the portion of the filename indicating the track number		
				filename = stripTrackNo(filename); 
				//trk now (possibly) contains a non existing filename (if the track number was stripped)		
				trk = TrackSpace::makeTrack(filename);
				//reset the filename to its original value	
				trk->setName(goodFilename);
			} else {
				trk = TrackSpace::makeTrack(filename);
			}
		}
			
		trk->setSize(fi.size());
		trk->setCreated(fi.created().toString("yyyy.MM.dd  h:m:s"));
		trk->setModified(fi.lastModified().toString("yyyy.MM.dd  h:m:s"));
		trk->setAccessed(fi.lastRead().toString("yyyy.MM.dd  h:m:s"));
		trk->setPath(fi.dirPath(true));
		trk->setDirectory(fi.isDir());
		trk->setReadOnly(fi.isReadable());
			
		if (fi.isDir()) {
			trk->setName(fi.fileName());
		} else {
			trk->setDescription(fRcMan->getExtDesc(ext));
			tmp = makeAlbum(trk->getPath(),fPath);
			trk->setAlbum(tmp);
			
			trk->setTrackNo(i);
			++i;
			
			if (fi.size() > 0) {
				MetaInfo::getMetaInfo(trk,fSettings->getBool(Settings::fMetadataOverride));
			}	
			//trk->setMood();
			//trk->setScore();
			//trk->setArchive(fi.);
			//trk->setSystem(fia[i].IsSystem());
			//trk->setVolumeLabel(false); //still has to be fixed
			//trk->setCompressed(fia[i].IsCompressed());
			//trk->setTemporary(fia[i].IsTemporary());			
			//trk->setHidden(false);
		}	
		
		if (trk->getSize() >= 0) fl.push_back(trk);
		++it;
		++pi;
		pBar->setProgress(pi);
	}

	flist.clear();
	delete dialog;
	return fl;
}
//-----------------------------------------------------------------------------------------------------------
void DirScanner::getTracksFromSource(const QString fname,Track::Tracklist& tlist, QProgressBar* progressBar) {
    
    if (!FileIO::exists(fname)) {
	throw (new IoException("DirScanner::getTracksFromSource","Path '" + fname + "' does not exist"));
    }    
    
    bool directory = FileIO::isDirectory(fname);
    
    if (directory) {
	getTracksFromDir(fname, tlist, progressBar);
    } else {
	getTracksFromFile(fname, tlist);
    }
}
//-----------------------------------------------------------------------------------------------------------
void DirScanner::getTracksFromDir(const QString fname,Track::Tracklist& tlist, QProgressBar* progressBar) {
   setPath(fname);
   tlist = indexMusicDirectory(DomoGlobals::MUSIC_MASK,
			       fSettings->getBool(Settings::fRecurseDirs),
			       false,
			       progressBar);
}
//-----------------------------------------------------------------------------------------------------------
void DirScanner::getTracksFromFile(const QString fname,Track::Tracklist& tlist) {
    ReaderFactory* factory = new ReaderFactory();
    
    IReader* reader = factory->getReader(fname);
    assert(reader != 0);
    
    reader->read(tlist);
    delete reader;
    delete factory;
}
//-----------------------------------------------------------------------------------------------------------
