/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)
   
   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   
----------------------------------------------------------------------------*/
#ifndef CSVDIALOG_H
#include "csvdialog.h"
#define CSVDIALOG_H
#endif

#ifndef INC_WRITERCSV_H
#include "writercsv.h"
#define INC_WRITERCSV_H
#endif

#ifndef INC_DBASEEXCEPTION_H
#include "dbaseexception.h"
#define INC_DBASEEXCEPTION_H
#endif

#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_FILEIO_H
#include "fileio.h"
#define INC_FILEIO_H
#endif

#ifndef DOMOMAINWINDOW_H
#include "domomainwindow.h"
#define DOMOMAINWINDOW_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#ifndef CREATECOL_H
#include "createcol.h"
#define CREATECOL_H
#endif

#ifndef INC_WRITERM3U_H
#include "writerm3u.h"
#define INC_WRITERM3U_H
#endif

#ifndef INC_WRITERHTML_H
#include "writerhtml.h"
#define INC_WRITERHTML_H
#endif

#ifndef INC_WRITERTXT_H
#include "writertxt.h"
#define INC_WRITERTXT_H
#endif

#ifndef INC_WRITERB4S_H
#include "writerb4s.h"
#define INC_WRITERB4S_H
#endif

#ifndef INC_WRITERPLS_H
#include "writerpls.h"
#define INC_WRITERPLS_H
#endif

#ifndef INC_DOMOGLOBALS_H
#include "domoglobals.h"
#define INC_DOMOGLOBALS_H
#endif

#ifndef INC_SETTINGS_H
#include "settings.h"
#define INC_SETTINGS_H
#endif

#ifndef PREFERENCEDIAG_H
#include "preferencediag.h"
#define PREFERENCEDIAG_H
#endif

#ifndef INC_TRACK_H
#include "track.h"
#define INC_TRACK_H
#endif

#ifndef INC_FILE_H
#include "file.h"
#define INC_FILE_H
#endif

#ifndef WELCOMEDIAG_H
#include "welcomediag.h"
#define WELCOMEDIAG_H
#endif

#ifndef MUSICBRAINZDIALOG_H
#include "musicbrainzdialog.h"
#define MUSICBRAINZDIALOG_H
#endif

#ifndef FILECHOOSER_H
#include "filechooser.h"
#define FILECHOOSER_H
#endif

#ifndef INC_EXCEPTION_H
#include "exception.h"
#define INC_EXCEPTION_H
#endif

#ifndef INC_CDLOOKUP_H
#include "cdlookup.h"
#define INC_CDLOOKUP_H
#endif

#ifndef STATSDIALOG_H
#include "statsdialog.h"
#define STATSDIALOG_H
#endif

#ifndef BACKUPDBASEDIAG_H
#include "backupdbasediag.h"
#define BACKUPDBASEDIAG_H
#endif

#include "enterTracksDialog.h"
#include "renamealbumdialog.h"
#include "qradiobutton.h"
#include "qbuttongroup.h"
#include "qgroupbox.h"
#include "qdir.h"
#include "qfileinfo.h" 
#include "trackinfo.h"
#include "assert.h"
#include "qlistview.h"
#include "qlineedit.h"
#include "qdatatable.h"
#include "qdataview.h"
#include "qsqlrecord.h"
#include "qcombobox.h"
#include "qstring.h"
#include "qtextview.h"
#include "qcheckbox.h"
#include "qfile.h"
#include "qdatetime.h"
#include "qdatabrowser.h"
#include "qtabwidget.h"
#include "qspinbox.h"
#include "qlabel.h"
#include "qsqldatabase.h"
#include "qmessagebox.h"
#include "qpopupmenu.h"
#include "qcursor.h"
#include <iostream>
#include <list>

using std::cout;
using std::ostringstream;
using MediaSpace::Collection;
using CompareSpace::Compare;
using TrackSpace::Track;
using FileSpace::File;
using namespace WriterSpace;
using DbaseSpace::Dbase;
using DirScannerSpace::DirScanner;
//-----------------------------------------------------------------------------------------------------------
DOMOMainWindow::DOMOMainWindow( QWidget* parent, const char* name, WFlags f )
 : DOMOMainWindowBase( parent, name, f )
{
 setCaption("DOMO " + DomoGlobals::DOMO_VERSION + " - The Best Media Organizer out there - By GoriTech");
 
 bool b = false;
 fDB = 0;
 fLogger = new GuiLog(LogWindow);
 fCurCollection = new Collection();
 fSet = new Settings(DomoGlobals::DOMO_DIR + DomoGlobals::SETTINGS_FNAME);
 b = fSet->getSettings();
 
 if (!b) {
  fLogger->error("DOMOMainWindow::DOMOMainWindow",
          "The file 'Domo.conf' is erronous, please correct or delete this file and restart!");
 } else {
  fRcMan = new RcMan(fSet);
  try {
   fRcMan->parseAll();
  } catch(IoException* e) {
   fLogger->exception("DOMOMainWindow::DOMOMainWindow",e->msg());
   fLogger->error("DOMOMainWindow::DOMOMainWindow","Parsing of resources failed, check log");
   b = false;
  }
  
  if (b) { 
   fConsistencyChecker = new ConsistencyChecker(fLogger, fRcMan, fSet);
   fCompare = new Compare(fRcMan, fLogger);
   fDirScan = new DirScanner(fRcMan, fSet, "");
   initWidgets();
  }
 }
 
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::setDatabase(Dbase* db) {
 assert(db!=0);
 fDB = db;
 try {
  updateCollectionList(); 
  initBrowseTab();
 } catch(IoException* e) {
  fLogger->exception("DOMOMainWindow::setDatabase",e->msg());
 }
}
//-----------------------------------------------------------------------------------------------------------
DOMOMainWindow::~DOMOMainWindow() {
 fLogger->info("DOMOMainWindow::~DOMOMainWindow","Freeing resources...");
 bool b = fSet->setSettings();
 if (!b) fLogger->error("DOMOMainWindow::~DOMOMainWindow","Error during saving of settings");
 
 try {
  fRcMan->flushHyphenedArtists();
 } catch (IoException* e) {
  fLogger->exception("DOMOMainWindow::~DOMOMainWindow",e->msg());
  fLogger->error("DOMOMainWindow::~DOMOMainWindow","Flushing of hyphendartists failed...");
 }
 
 delete fSet;
 delete fCurCollection;
 delete fDB;
 delete fDirScan;
 delete fRcMan;
 fLogger->info("DOMOMainWindow::~DOMOMainWindow","Shutting down");
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::initWidgets() {
 compareResults->setSorting(2);
 
 queryResults->adjustColumn(0);
 queryResults->adjustColumn(1);
 queryResults->adjustColumn(2);
 queryResults->adjustColumn(3);
 queryResults->adjustColumn(4);
 queryResults->adjustColumn(5);
 queryResults->adjustColumn(6);
 
 cdList->adjustColumn(0);
 cdList->adjustColumn(1);
 cdList->adjustColumn(2);
 
 albumList->adjustColumn(0);
 albumList->adjustColumn(1);
 albumList->adjustColumn(2);
 albumList->adjustColumn(3);
 albumList->adjustColumn(4);
 
 QStringList* slist = fRcMan->getGenreListRef();
 assert(slist != 0);
 genreField->insertStringList(*slist,1);
 yearField->setValue(QDate::currentDate().year());
 commentField->setText("Added on " + QDateTime::currentDateTime().toString("dddd dd/MM/yyyy"));
 titleField->setText("My CD");
 
 destChooser->setFileName(fSet->get(Settings::fExportDir));
 addChooser->setFileName(fSet->get(Settings::fDefaultAddDir));
 cssChooser->setFileName(fSet->get(Settings::fCssFile));
 cssChooser->setMode(FileChooser::File);
 cssChooser->disableDirs();
 compareChooser->setFileName(fSet->get(Settings::fDefaultAddDir));
 compareChooser2->setFileName(fSet->get(Settings::fDefaultAddDir));
 exportChooser->setFileName(fSet->get(Settings::fDefaultAddDir));
 
 filenameField->setText( QString(fSet->get(Settings::fExportDir)) + "compare.txt" );
 
 Tabs->setTabIconSet(Tabs->page(0), QIconSet( QPixmap::fromMimeSource("compare.png") ) );
 Tabs->setTabIconSet(Tabs->page(1), QIconSet( QPixmap::fromMimeSource("export.png") ) );
 Tabs->setTabIconSet(Tabs->page(2), QIconSet( QPixmap::fromMimeSource("add.png") ) );
 Tabs->setTabIconSet(Tabs->page(3), QIconSet( QPixmap::fromMimeSource("browse.png") ) );
 Tabs->setTabIconSet(Tabs->page(4), QIconSet( QPixmap::fromMimeSource("help.png") ) );
 Tabs->setTabIconSet(Tabs->page(5), QIconSet( QPixmap::fromMimeSource("wish.png") ) );
 
 fLogger->info("DOMOMainWindow::initWidgets","Widgets initialised...");    
 fLogger->info("DOMOMainWindow::initWidgets","Current music mask: '" + DomoGlobals::MUSIC_MASK + "'");
 fLogger->info("DOMOMainWindow::initWidgets","Available database drivers: '" + DomoGlobals::DBDRIVERS.join(",") + "'");
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::initBrowseTab() {
 browseList->clear();
 QStringList ids;
 QStringList titles;
 browseList->setRootIsDecorated(true);
 
 fDB->getIdTitles(ids, titles, "CD");
 assert(ids.size() == titles.size());
 
 QStringList::Iterator it2 = titles.begin();
 
 for ( QStringList::Iterator it = ids.begin(); it != ids.end(); ++it ) {
  QListViewItem* item = new QListViewItem(browseList, *it, *it2);
  item->setPixmap(0,QPixmap::fromMimeSource("cd.png"));
  item->setExpandable(true);
  browseList->insertItem(item);
  ++it2;
 }
 
 //browse list 2
 browseList2->clear();
 QStringList genres;
 browseList2->setRootIsDecorated(true);
 
 fDB->getGenres(genres, "");
 
 for ( QStringList::Iterator it = genres.begin(); it != genres.end(); ++it ) {
  QListViewItem* item = new QListViewItem(browseList2, *it);
  item->setPixmap(0,QPixmap::fromMimeSource("musicnote.png"));
  item->setExpandable(true);
  browseList2->insertItem(item);
 }

}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::expandBrowseNode(QListViewItem* item) {
 QStringList list;
 QString whereClause = "";
 
 try{ 
  //CDs
  if (item->depth() == 0) {
   whereClause = "(cdno = " + item->text(0) + ")";
   browseTable->setFilter( whereClause );
   browseTable->refresh();
   
   //dont expand if already expanded
   if(item->childCount() <= 0) {
    fDB->getGenres(list, whereClause);
    for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
     QListViewItem* child = new QListViewItem(item, (*it));
     child->setPixmap(0,QPixmap::fromMimeSource("musicnote.png"));
     child->setExpandable(true);
    }
   }
   //genres
  } else if (item->depth() == 1) {
   whereClause = "((genre = '" + item->text(0) + "') AND (cdno = " + item->parent()->text(0) + "))";
   browseTable->setFilter( whereClause );
   browseTable->refresh();
   
   //dont expand if already expanded
   if(item->childCount() <= 0) {
    fDB->getArtists(list, whereClause);
    for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
     QListViewItem* child = new QListViewItem(item, (*it));
     child->setExpandable(true);
    }
   }
   //artists
  } else if (item->depth() == 2) {
   whereClause = "((artist = '" + item->text(0) + "')"
          +" AND (genre = '" + item->parent()->text(0) + "')"
          +" AND (cdno = " + item->parent()->parent()->text(0) + "))";
   
   browseTable->setFilter( whereClause);
   browseTable->refresh();
   
   //dont expand if already expanded
   if(item->childCount() <= 0) {
    fDB->getAlbums(list,whereClause);
    for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
     QListViewItem* child = new QListViewItem(item, (*it));   
     child->setExpandable(true);
    }
   }
   //albums
  } else if (item->depth() == 3) {
   whereClause = "((album = '" + item->text(0) + "')"
          + " AND (artist = '" + item->parent()->text(0) + "')"
          + " AND (genre = '" + item->parent()->parent()->text(0) + "')"
          + " AND (cdno = " + item->parent()->parent()->parent()->text(0) + "))";
   
   browseTable->setFilter(whereClause);
   browseTable->refresh();
  } else {
   assert(false);
  }
  
 } catch(IoException* e) {
  fLogger->exception("DOMOMainWindow::expandBrowseNode",e->msg());
 }
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::collapseBrowseNode(QListViewItem* item) {
 QStringList list;
 QString whereClause = "";
 
 try{ 
  if (item->depth() == 0) {
   if(browseList->hasMouse()) {
    whereClause = "(cdno = " + item->text(0) + ")";
   } else {
    whereClause = "(genre = '" + item->text(0) + "')";
   }
   
  } else if (item->depth() == 1) {
   if(browseList->hasMouse()) {
    whereClause = "((genre = '" + item->text(0) + "') AND (cdno = " + item->parent()->text(0) + "))";
   } else {
    whereClause = "((artist = '" + item->text(0) + "') AND (genre = '" + item->parent()->text(0) + "'))";
   }
   
  } else if (item->depth() == 2) {
   if(browseList->hasMouse()) {
    whereClause = "((artist = '" + item->text(0) + "')"
           +" AND (genre = '" + item->parent()->text(0) + "')"
           +" AND (cdno = " + item->parent()->parent()->text(0) + "))";
   } else {
    whereClause = "((album = '" + item->text(0) + "')"
           +" AND (artist = '" + item->parent()->text(0) + "')"
           +" AND (genre = '" + item->parent()->parent()->text(0) + "'))";
   }
   
   
   //albums
  } else if (item->depth() == 3) {
   //only browselist goes to depth 3
   assert(browseList->hasMouse());
   whereClause = "((album = '" + item->text(0) + "')"
          + " AND (artist = '" + item->parent()->text(0) + "')"
          + " AND (genre = '" + item->parent()->parent()->text(0) + "')"
          + " AND (cdno = " + item->parent()->parent()->parent()->text(0) + "))";
   
  } else {
   assert(false);
  }
  //cout<<"whereclause: " << whereClause << "\n";
  browseTable->setFilter(whereClause);
  browseTable->refresh();
  
 } catch(IoException* e) {
  fLogger->exception("DOMOMainWindow::collapseBrowseNode",e->msg());
 }
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::expandBrowseNode2(QListViewItem* item) {
 QStringList list;
 QString whereClause = "";
 
 try{ 
   //genres
  if (item->depth() == 0) {
   whereClause = "(genre = '" + item->text(0) + "')";
   browseTable->setFilter( whereClause );
   browseTable->refresh();
   
   //dont expand if already expanded
   if(item->childCount() <= 0) {
    fDB->getArtists(list, whereClause);
    for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
     QListViewItem* child = new QListViewItem(item, (*it));
     child->setExpandable(true);
    }
   }
   
   //artists
  } else if (item->depth() == 1) {
   whereClause = "((artist = '" + item->text(0) + "')"
          +" AND (genre = '" + item->parent()->text(0) + "'))";
   
   browseTable->setFilter( whereClause);
   browseTable->refresh();
   
   //dont expand if already expanded
   if(item->childCount() <= 0) {
    fDB->getAlbums(list,whereClause);
    for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
     QListViewItem* child = new QListViewItem(item, (*it));   
     child->setExpandable(true);
    }
   }
   
   //albums
  } else if (item->depth() == 2) {
   whereClause = "((album = '" + item->text(0) + "')"
          + " AND (artist = '" + item->parent()->text(0) + "')"
          + " AND (genre = '" + item->parent()->parent()->text(0) + "'))";
   
   browseTable->setFilter(whereClause);
   browseTable->refresh();
  } else {
   assert(false);
  }
  
 } catch(IoException* e) {
  fLogger->exception("DOMOMainWindow::expandBrowseNode2",e->msg());
 }
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::updateCollectionList(){
 QStringList slist(false);
 
 collectionList->clear();
 collectionList->insertItem("-= Active Collection =-");
 
 try {
  fDB->getCollectionTitles(slist);
 } catch (DbaseException* e) {
  fLogger->exception("DOMOMainWindow::updateCollectionList",e->msg());
  fLogger->error("DOMOMainWindow::updateCollectionList","Update collection list failed");
  return;
 }
 
 QStringList::const_iterator SI = slist.begin();
 
 int i = 0;
 while (SI != slist.end()) {
  if ((*SI) != "") {
   collectionList->insertItem(*SI);
   ++i;
  }
  ++SI;
 }
 
 QString t;
 if (i > 0) {
  fLogger->info("DOMOMainWindow::updateCollectionList",t.setNum(i) + " Collection(s) found...");
  collectionList->setCurrentItem(1);
  this->setActiveCollection(collectionList->currentText());
  sourceBox->setEnabled(true);
  mediaTypeBox->setEnabled(true);
  fieldsBox->setEnabled(true);
 } else {
  fLogger->info("DOMOMainWindow::updateCollectionList", 
         tr("Please create a collection first! (File->New Collection)") );
  sourceBox->setDisabled(true);
  mediaTypeBox->setDisabled(true);
  fieldsBox->setDisabled(true);
  WelcomeDiag* dialog = new WelcomeDiag(this);
  dialog->show();
  //delete dialog;
 }
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::addCD() {
 QTime t;
 QString s = fCurCollection->getTitle();
 QString tmp = "";
 Track::Tracklist tlist;
 QString filename = addChooser->fileName();
 QString cdTitle = titleField->text();
 
 if (empty(s)) {
  fLogger->error("DOMOMainWindow::addCD","Empty collection title, no active collection, add cd failed!!");
  return;
 }
 
 if (empty(filename)) {
  fLogger->error("DOMOMainWindow::addCD","No source specified!!!");
  return;
 }
 
 
 try {
  if(fDB->cdTitleExists(cdTitle)) {
   QMessageBox mb( tr("Existing CD title!"),
     tr("CD title '" + cdTitle + "' already exists!"),
     QMessageBox::Warning,
     QMessageBox::Ok | QMessageBox::Default,
     QMessageBox::NoButton, 
     QMessageBox::NoButton,this );
   mb.exec();
   return;
  }
 } catch (IoException* e) {
  fLogger->exception("DOMOMainWindow::addCD",e->msg());
  fLogger->error("DOMOMainWindow::addCD","Error during query, aborting add");
  return;
 }
 
 
 t.start(); 
 
 fLogger->info("DOMOMainWindow::addCD","Starting file scan...");
 
 if (mp3cdRadBut->isChecked() || (audiocdRadBut->isChecked() && fileEnter->isChecked())) {
  if (empty(cdTitle)) {
   QMessageBox mb( tr("Empty CD Title"),
     tr("Empty CD title!"),
     QMessageBox::Warning,
     QMessageBox::Ok | QMessageBox::Default,
     QMessageBox::NoButton, 
     QMessageBox::NoButton,this );
   mb.exec();
   return;
  }
  
  try {
   fDirScan->getTracksFromSource(filename, tlist, progressBar);
  } catch (IoException* e) {
   fLogger->exception("DOMOMainWindow::addCD",e->msg());
   fLogger->error("DOMOMainWindow::addCD","Error during scan of '" + filename + "' aborting add");
   return;
  }
  
  fLogger->info("DOMOMainWindow::addCD",tmp.setNum(tlist.size()) + " entries scanned...");
  
  if ( !fConsistencyChecker->makeConsistent(tlist, progressBar) ) {
   fLogger->warning("DOMOMainWindow::addCD","Consistency check + Add CD aborted...");
   Track::deleteTracks(tlist);
   fLogger->info("DOMOMainWindow::addCD","Deleted scanned tracks");
   progressBar->reset();
   return;
  }
  
  if (tlist.size() <= 0) {
   fLogger->warning("DOMOMainWindow::addCD",
      "Sorry, no tracks were found in '" + QString(filename) + "' CD not added");
   return;
  }
  
 } else {
  //Source to add is an audio CD, now how should we get the tracks?
  //through musicbrainz
  if(autoEnter->isChecked()) {
   this->lookupCD();
   return;
   //manually enter tracks
  } else if (manualEnter->isChecked()) {
   if (empty(cdTitle)) {
    QMessageBox mb( tr("Empty CD Title"),
      tr("Empty CD title!"),
      QMessageBox::Warning,
      QMessageBox::Ok | QMessageBox::Default,
      QMessageBox::NoButton, 
      QMessageBox::NoButton,this );
    mb.exec();
    return;
   }
   EnterTracksDialog* diag = new EnterTracksDialog(this);
   diag->setArtist(artistField->text());
   diag->setTitle(titleField->text());
   if (diag->exec() == QDialog::Accepted) {
    artistField->setText(diag->getArtist());
    tlist = *(diag->getTracks());
   } else {
    fLogger->warning("DOMOMainWindow::addCD","Add CD aborted...");
    delete diag;
    return;
   }
   fLogger->info("DOMOMainWindow::addCD",tmp.setNum(tlist.size()) + " entries entered...");
   delete diag;
  }
 }
 
 fLogger->info("DOMOMainWindow::addCD","Adding to database...");
 tmp = titleField->text();
 tmp = cleanFilename(tmp);
 titleField->setText(tmp);
 tmp = "";
 
 try {
  if (mp3cdRadBut->isChecked()) {
   Mp3CD* m = new Mp3CD();
   m->setComment(commentField->text());
   m->setGenre(genreField->currentText());
   m->setTitle(titleField->text());
   m->setTracklist(tlist); //track genres are set here
   
   fDB->add(m,progressBar); //correct CD id is set here
   fDB->addToCollection(m,fCurCollection); //add the cd to the active collection
   fDB->updateAlbums(m->getID(),fSet->getInt(Settings::fTracksPerAlbum),"DIG_AUDIO");  //extract all albums present on the CD
   delete m;
   
  } else if (audiocdRadBut->isChecked()){
   AudioCD* m = new AudioCD();
   m->setComment(commentField->text());
   m->setGenre(genreField->currentText());
   m->setTitle(titleField->text());
   m->setYear(yearField->value());
   m->setArtist(artistField->text());
   m->setTracklist(tlist); //album & year & track genres are set here
   
   fDB->add(m,progressBar); //correct CD id is set here
   fDB->addToCollection(m,fCurCollection);
   fDB->addAlbum(m);
   delete m;
  } else {
   assert(false);
  } 
  
  fLogger->info("DOMOMainWindow::addCD","CD added succesfully...");
  
  titleField->setText(titleField->text() + " (1)");
  
  cdList->refresh();
  albumList->refresh();
  initBrowseTab();
  fLogger->info("DOMOMainWindow::addCD","Add took: " + tmp.setNum(t.elapsed()/1000) + " seconds");
  
 } catch (DbaseException* e) {
  fLogger->exception("DOMOMainWindow::addCD",e->msg());
  fLogger->error("DOMOMainWindow::addCD","Add CD failed...");
 }
 
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::cleanDir() {
 Track::Tracklist tlist;
 QString tmp = "";
 FileChooser* dialog = new FileChooser(this,"Choose a directory");
 dialog->setOnlyDirs(true);
 dialog->setFileName( fSet->get(Settings::fDefaultAddDir) );
 
 if(!dialog->chooseFile()) {
  fLogger->info("DOMOMainWindow::cleanDir","Clean directory cancelled.");
  return;
 }
 QString source = dialog->fileName();
 
 bool consCheck = fSet->getBool(Settings::fSkipConsCheck);
 bool autoRename = fSet->getBool(Settings::fAutoRename);
 
 if (consCheck) {
  fLogger->warning("DOMOMainWindow::cleanDir","Temporarily enabling consistency checking");
  fSet->set(Settings::fSkipConsCheck, false);
 }
 if (!autoRename) {
  fLogger->warning("DOMOMainWindow::cleanDir","Temporarily enabling auto rename");
  fSet->set(Settings::fAutoRename, true);
 }
 
 
 fLogger->info("DOMOMainWindow::cleanDir","Starting file scan...");
 try {
  fDirScan->getTracksFromSource(source, tlist, progressBar);
  fLogger->info("DOMOMainWindow::cleanDir",tmp.setNum(tlist.size()) + " entries scanned...");
 } catch (IoException* e) {
  fLogger->exception("DOMOMainWindow::cleanDir",e->msg());
  fLogger->error("DOMOMainWindow::cleanDir","Error during scan of '" + source + "' aborting scan");
  fSet->set(Settings::fSkipConsCheck, consCheck);
  fSet->set(Settings::fAutoRename, autoRename);
  delete dialog;
  return;
 }
 
 if (tlist.size() <= 0) {
  fLogger->warning("DOMOMainWindow::cleanDir","Sorry, no tracks were found in '" + source + "' nothing done");
  fSet->set(Settings::fSkipConsCheck, consCheck);
  fSet->set(Settings::fAutoRename, autoRename);
  delete dialog;
  return;
 }
 
 fLogger->info("DOMOMainWindow::cleanDir","Starting consistency check...");
 if ( !fConsistencyChecker->makeConsistent(tlist, progressBar) ) {
  fLogger->warning("DOMOMainWindow::cleanDir","Consistency check aborted...");
  Track::deleteTracks(tlist);
  fLogger->info("DOMOMainWindow::cleanDir","Deleted scanned tracks");
  progressBar->reset();
  fSet->set(Settings::fSkipConsCheck, consCheck);
  fSet->set(Settings::fAutoRename, autoRename);
  delete dialog;
  return;
 }
 
 fLogger->info("DOMOMainWindow::cleanDir","Cleansing finished...");
 fSet->set(Settings::fSkipConsCheck, consCheck);
 fSet->set(Settings::fAutoRename, autoRename);
 delete dialog;
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::clearCompareResults() {
 fCompare->clear();
 compareResults->clear();
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::compare() {
 fLogger->warning("DOMOMainWindow::compare","Comparing large sources may take a while");
 
 QTime t;
 fCompare->setLenTolerance(fSet->getInt(Settings::fLengthTol));
 QString tmp;
 Track::Tracklist tlist;
 Track::Tracklist tlist2;
 QString filename = compareChooser->fileName();
 t.start();
 
 //get the first source
 fLogger->info("DOMOMainWindow::compare","Scanning source 1 for tracks...");
 try {
  fDirScan->getTracksFromSource(filename, tlist, progressBar);
  fLogger->info("DOMOMainWindow::compare",tmp.setNum(tlist.size()) + " entries scanned from source 1");
 } catch (IoException* e) {
  fLogger->exception("DOMOMainWindow::compare",e->msg());
  fLogger->error("DOMOMainWindow::compare","Error during scan of '" + filename + "' aborting compare");
  return;
 }
 
 if (tlist.size() <= 0) {
  fLogger->warning("DOMOMainWindow::compare","Sorry, no tracks were found in '"
     + filename + "', compare aborted");
  return;
 }
 
 //fLogger->info("DOMOMainWindow::",tmp.setNum(tlist.size()) + " before");
 if ( !fConsistencyChecker->makeConsistent(tlist, progressBar) ) {
  fLogger->warning("DOMOMainWindow::compare","Consistency check + Compare aborted...");
  Track::deleteTracks(tlist);
  fLogger->info("DOMOMainWindow::compare","Deleted scanned tracks");
  return;
 }
 //fLogger->info("DOMOMainWindow::",tmp.setNum(tlist.size()) + " after");
 tmp = "";
 
 fCompare->setRequireEqualCom(fSet->getBool(Settings::fCommentsMatch));
 
 try {
  if (compWith->isChecked()) {
   if(cmpSource2->currentText() == "Active Collection") {
    //second source is the database
    fLogger->info("DOMOMainWindow::compare","Comparing with Active Collection...");
    fCompare->compare(tlist,fDB,"Track",progressBar);
   } else if(cmpSource2->currentText() == "Wishlist") {
    //second source is the wishlist
    fLogger->info("DOMOMainWindow::compare","Comparing with Wishlist...");
    fLogger->error("DOMOMainWindow::compare","Function not implemented  yet...");
    /*fDB->createTmpTrackTable("WishlistTmpTrack");
      fDB->copyWishlistToTmpTrackTable("WishlistTmpTrack");
      fCompare->compare(tlist,fDB,"WishlistTmpTrack",progressBar);*/
   } else {
    assert(false);
   }
  } else if (compWithFile->isChecked()) {
   //second source is another file/dir
   filename = compareChooser2->fileName();
   
   try {
    fDirScan->getTracksFromSource(filename, tlist2, progressBar);
    fLogger->info("DOMOMainWindow::compare",tmp.setNum(tlist2.size()) +
           " files scanned from source 2");
   } catch (IoException* e) {
    fLogger->exception("DOMOMainWindow::",e->msg());
    fLogger->error("DOMOMainWindow::compare","Error during scan of '" +
            filename + "' aborting compare");
    return;
   }
   
   tmp = "";
   
   fDB->createTmpTrackTable("tmpTrack3");
   fDB->add(tlist2,"tmpTrack3");
   fLogger->info("DOMOMainWindow::compare","Starting compare with file/dir...");
   fCompare->compare(tlist,fDB,"tmpTrack3",progressBar);
   fDB->dropTable("tmpTrack3");
  } else {
   assert(false);
  }
 } catch (DbaseException* ex) {
  fLogger->exception("DOMOMainWindow::compare",ex->msg());
  fLogger->error("DOMOMainWindow::compare","Compare failed");
  return;
 } catch (Exception* e) {
  fLogger->exception("DOMOMainWindow::",e->msg());
  fLogger->error("DOMOMainWindow::compare","Compare failed");
  return;
 } 
 fLogger->info("DOMOMainWindow::compare","Comparing done after "
        + tmp.setNum(tlist.size()*11 +1) + " queries...\nDisplaying results...");
 
 fCompare->displayResults(compareResults);
 
 fLogger->info("DOMOMainWindow::compare","Done...total time: " + tmp.setNum(t.elapsed()/1000) + " seconds");
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::useCompareSelection() {
 
 if (exportSelection->isChecked()) {
  fCompare->exportCompareSelection(compareResults,
       filenameField->text(),
       noEntries->isChecked(),
       diffVerEntries->isChecked(),
       maybeEntries->isChecked(),
       yesEntries->isChecked(),
       selectedEntries->isChecked());
  
 }
 
 if (deleteSelection->isChecked()) {
  if (!confirm( tr("<font color=\"red\">WARNING</font>: This will physically remove all selected files!") )) {
   fLogger->info("DOMOMainWindow::useCompareSelection","Aborting delete...");
   return;
  }
  fCompare->deleteCompareSelection(compareResults,
       compareChooser->fileName(),
       noEntries->isChecked(),
       diffVerEntries->isChecked(),
       maybeEntries->isChecked(),
       yesEntries->isChecked(),
       selectedEntries->isChecked());
  
 }             
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::createCollection() {
 CreateCol* dialog = new CreateCol( this, "Create Collection", TRUE );
 dialog->dbaseName->setText(fDB->getName());
 dialog->colOwnerField->setText(fDB->getUser());
 
 if (dialog->exec() == QDialog::Rejected) {
  fLogger->info("DOMOMainWindow::createCollection","Create Collection cancelled...");
  return;
 }
 
 QString colTitle = dialog->colTitleField->text();
 QString colOwner = dialog->colOwnerField->text();
 QString colDesc = dialog->colDescField->text();
 
 if (colTitle.isEmpty()) {
  fCurCollection->setTitle("Default");
 }
 
 fCurCollection->setTitle(colTitle);
 fCurCollection->setOwner(colOwner);
 fCurCollection->setDesc(colDesc);
 
 try {
  fDB->add(fCurCollection);
 } catch(DbaseException* e) {
  fLogger->exception("DOMOMainWindow::createCollection",e->msg());
  fLogger->error("DOMOMainWindow::createCollection","Create collection failed");
  return;
 }
 
 collectionList->insertItem(colTitle);
 int i = collectionList->count();
 collectionList->setCurrentItem(i-1);
 sourceBox->setEnabled(true);
 mediaTypeBox->setEnabled(true);
 fieldsBox->setEnabled(true);
 
 fLogger->info("DOMOMainWindow::createCollection","Collection " + colTitle + " created...");
 delete dialog;
 
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::setActiveCollection(const QString& title) {
 fCurCollection->clear();  //erase old collection info
 try {
  fDB->getCollection(title,fCurCollection);
 } catch(DbaseException* e) {
  fLogger->exception("DOMOMainWindow::setActiveCollection",e->msg());
  fLogger->error("DOMOMainWindow::setActiveCollection","Set active collection failed");
 }
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::query() {
 QString colname = "";
 QString q = ""; 
 QString filter = queryString->text();
 
 filter = filter.replace("'","");
 
 if (qTitle->isChecked()) {
  colname = "title";
 } else if (qArtist->isChecked()) {
  colname = "artist";
 } else if (qAlbum->isChecked()) {
  colname = "album";
 } else if (qFilename->isChecked()) {
  colname = "name";
 } else if (qCDNo->isChecked()) {
  colname = "cdno";
 } else if (qGenre->isChecked()) {
  colname = "genre";
 } else if (qComment->isChecked()) {
  colname = "comment";
 } else if (qScore->isChecked()) {
  colname = "score";
 } else if (qMood->isChecked()) {
  colname = "mood";
 } else if (qLength->isChecked()) {
  colname = "length";
 } else {
  fLogger->error("DOMOMainWindow::query","Please select a query field!");
  return;
 }
 
 if (keyword->isChecked()) {
  //q = "t.cdno = m.id AND t." + colname + " LIKE '%" + "queryString->text() + "%';";
  q =  colname + " LIKE '%" + filter + "%'";
 } else {
  //q = "t.cdno = m.id AND t." + colname + " = '" + "queryString->text() + "';";
  q =  colname + " = '" + filter + "'";
 }
 
 if(queryTracks->isChecked()) {
  queryResults->setFilter(q);
  queryResults->refresh();
 } else {
  albumList->setFilter(q);
  albumList->refresh();
 }
 
 fLogger->info("DOMOMainWindow::query","Query on filter '" + filter + "' done");
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::updateAlbumTracks() {
 int row = albumList->currentRow();
 QString cdno = albumList->text(row,0);
 QString title = albumList->text(row,1);
 queryResults->setFilter("album = \"" + title + "\" AND cdno = " + cdno);
 queryResults->refresh();
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::exportToFile() {
 if (!confirm( tr("Warning: existing files will be overwritten! Do you want to continue?") )) {
  fLogger->info("DOMOMainWindow::","Export cancelled..");
  return;
 }
 
 fLogger->warning("DOMOMainWindow::","Exporting large collections, or to a large number of output formats may take a while");
 fLogger->info("DOMOMainWindow::","Starting export...");
 QTime timer;
 timer.start();
 QStringList slist;
 int id = -1;
 QString filename = exportFileName->text();
 QString baseName = destChooser->fileName() + DomoGlobals::DIR_SEP;
 QString tmp = "";
 WriterHtml* wh = 0;
 WriterTxt* wt = 0;
 WriterM3U* wm = 0;
 WriterB4s* wb = 0;
 WriterPls* wp = 0;
 WriterCsv* wc = 0;
 bool html = false;
 bool txt = false;
 bool m3u = false;
 bool b4s = false;
 bool pls = false;
 bool csv = false;
 QString table = "";
 int i = 0;
 progressBar->reset();
 QString stylesheet = "";
 QStringList* list = 0;
 QString cdType = "";
 
 //export our own collection
 if (exportDAudio->isChecked()) {
  table = "Mp3CD";
  cdType = "MP3_CD";
 }
 if (exportAudio->isChecked()) {
  table = "AudioCD";
  cdType = "AUDIO_CD";
 }
 if (exportAlbum->isChecked()) {
  table = "Album"; 
  cdType = "AUDIO_CD";
 }
 
 if (exportData->isChecked()) {
  table = "DataCD";
  cdType = "DATA_CD";
 }
 
 if (exportMovie->isChecked()) {
  table = "MovieCD";
  cdType = "MOVIE_CD";
 }
 
 try  {
  if (htmlChkBox->isChecked()) {
   wh = new WriterHtml(baseName + filename + ".html");
   wh->setShort(shortRadBut->isChecked());
   wh->setCdOnly(noTracks->isChecked());
   if (cssChkBox->isChecked()) stylesheet = cssChooser->fileName();
   wh->setStyleSheet( stylesheet );
   html = true;
  }
  
  if (txtChkBox->isChecked()) {
   wt = new WriterTxt(baseName + filename + ".txt");
   wt->setShort(shortRadBut->isChecked());
   txt = true;
  }
  
  if (m3uChkBox->isChecked()) {
   wm = new WriterM3U(baseName + filename + ".m3u");
   m3u = true;
  }
  
  if (plsChkBox->isChecked()) {
   wp = new WriterPls(baseName + filename + ".pls");
   pls = true;
  }
  
  if (b4sChkBox->isChecked()) {
   wb = new WriterB4s(baseName + filename + ".b4s");
   b4s = true;
  }
  
  if (csvChkBox->isChecked()) {
   CsvDialog* dialog = new CsvDialog(this);
   QString tmp = "";
   if (noTracks->isChecked()) tmp = cdType;
   dialog->setMode(tmp);
   if (dialog->exec() == QDialog::Accepted) { 
    list = dialog->getSelectedFields(tmp);
    wc = new WriterCsv(baseName + filename + ".csv",list);
    wc->setCdOnly(noTracks->isChecked());
    csv = true;
   } else {
    fLogger->info("DOMOMainWindow::","Export cancelled");
    return;
   }
  }
  
  if ( (html || txt || m3u || b4s || pls || csv) == false ) {
   fLogger->info("DOMOMainWindow::","Sorry, you must select an ouput format");
   return;
  }
  
  try {
   if (exportOther->isChecked()) {
    //export some external source, not our own collection
    QString tmp = "";
    Track::Tracklist tlist;
    QString filename = exportChooser->fileName();
    fLogger->info("DOMOMainWindow::","Scanning source for tracks...");
    try {
     fDirScan->getTracksFromSource(filename, tlist, progressBar);
     fLogger->info("DOMOMainWindow::",tmp.setNum(tlist.size()) + " files scanned from source");
    } catch (IoException* e) {
     fLogger->exception("DOMOMainWindow::",e->msg());
     fLogger->error("DOMOMainWindow::","Error during scan of '" + filename + "' aborting export");
     return;
    }
    
    if (tlist.size() <= 0) {
     fLogger->warning("DOMOMainWindow::","Sorry, no tracks were found in '"
        + QString(filename) + "', export aborted");
     delete wh;
     delete wt;
     delete wm;
     delete wb;
     delete wp;
     delete wc;
     return;
    }
    
    if ( !fConsistencyChecker->makeConsistent(tlist, progressBar) ) {
     fLogger->warning("DOMOMainWindow::","Consistency check + initiating action aborted...");
     delete wh;
     delete wt;
     delete wm;
     delete wb;
     delete wp;
     delete wc;
     return;
    }
    //treat the scanned tracks as an Mp3CD
    Mp3CD* cd = new Mp3CD();
    cd->setTitle("My Music");
    cd->setComment("Tracks scanned from: '" + filename + "'");
    cd->setTracklist(tlist);
    
    fLogger->info("DOMOMainWindow::","Exporting source...");
    if (html) wh->write(cd,true);
    if (txt) wt->write(cd,true);
    if (m3u) wm->write(cd,true);
    if (b4s) wb->write(cd,true);
    if (pls) wp->write(cd,true);
    if (csv) wc->write(cd,true);
    
    if (html) fLogger->info("DOMOMainWindow::","Index file creation disabled in this mode");
    
    delete cd;
    delete wh;
    delete wt;
    delete wm;
    delete wb;
    delete wp;
    delete wc;
    fLogger->info("DOMOMainWindow::","Done...total time: " 
           + tmp.setNum(timer.elapsed()/1000) + " seconds");
    return;
   }
   
   progressBar->reset();
   
   //b4s,pls playlists only apply to onePerCD
   if (onePerType->isChecked()) {
    fDB->getIDsFromTable(table,slist);
    QStringList::const_iterator SI = slist.begin();
    progressBar->setTotalSteps(slist.size());
    
    if (html) wh->writeHeader(cdType);
    if (txt) wt->writeHeader();
    if (m3u) wm->writeHeader();
    if (csv) wc->writeHeader();
    
    if (pls) {
     fLogger->warning("DOMOMainWindow::",
        "Sorry, exporting to pls only applies to the \"One per CD\" mode");
     wp->deleteCurrentFile(); 
    }
    if (b4s) {
     fLogger->warning("DOMOMainWindow::",
        "Sorry, exporting to b4s only applies to the \"One per CD\" mode");
     wb->deleteCurrentFile(); 
    }
    
    i = 1;
    while (SI != slist.end()) {
     if ((*SI) != "") {
      id = (*SI).toInt();
      
      CD* m = 0;
      
      if (exportAlbum->isChecked()) {
       m = fDB->getAlbumAsCD(id);
      } else {
       m = fDB->getCD(id);
      }
      if (html) wh->write(m,false);
      if (txt) wt->write(m,false);
      if (m3u) wm->write(m,false);
      if (csv) wc->write(m,false);
      delete m;
     }
     progressBar->setProgress(i);
     ++i;
     ++SI;
    }
    if (html) wh->writeFooter();
    if (txt) wt->writeFooter();
    if (m3u) wm->writeFooter();
    if (csv) wc->writeFooter();
    
    fLogger->info("DOMOMainWindow::","Selection exported..."); 
    
    if (html) {
     if (createIndex->isChecked()) {
      fLogger->info("DOMOMainWindow::","Creating index file....");
      QStringList idList;
      QStringList titleList;
      fDB->getIdTitles(idList,titleList,table);
      wh->changeFile(baseName + "index.html");
      wh->setCdOnly(false);
      wh->writeIndexFile(idList, titleList,filename + ".html",true);
     }
    } 
    
   } else if (onePerCD->isChecked()) {
    //writers are opened with the filename given by the user (see above)
    //if onePerCD is checked that filename will not be used and the created file must
    //be deleted again
    if (html) wh->deleteCurrentFile();
    if (txt) wt->deleteCurrentFile();
    if (m3u) wm->deleteCurrentFile();
    if (b4s) wb->deleteCurrentFile();
    if (pls) wp->deleteCurrentFile();
    if (csv) wc->deleteCurrentFile();
    
    fDB->getIDsFromTable(table,slist);
    QStringList::const_iterator SI = slist.begin();
    progressBar->setTotalSteps(slist.size());
    i = 1;
    while (SI != slist.end()) {
     if ((*SI) != "") {
      id = (*SI).toInt();
      
      CD* m = 0;
      if (exportAlbum->isChecked()) {
       m = fDB->getAlbumAsCD(id);
      } else {
       m = fDB->getCD(id);
      }
      
      if (html) wh->changeFile(baseName + m->getTitle() + ".html");
      if (txt) wt->changeFile(baseName + m->getTitle() + ".txt"); 
      if (m3u) wm->changeFile(baseName + m->getTitle() + ".m3u");
      if (b4s) wb->changeFile(baseName + m->getTitle() + ".b4s");
      if (pls) wp->changeFile(baseName + m->getTitle() + ".pls");
      if (csv) wc->changeFile(baseName + m->getTitle() + ".csv");
      
      if (html) wh->write(m,true);
      if (txt) wt->write(m,true);
      if (m3u) wm->write(m,true);
      if (b4s) wb->write(m,true);
      if (pls) wp->write(m,true);
      if (csv) wc->write(m,true);
      delete m;
     } 
     progressBar->setProgress(i);
     ++i;
     ++SI;
    }
    
    fLogger->info("DOMOMainWindow::","Selection exported..."); 
    
    if (html) {
     if (createIndex->isChecked()) {
      fLogger->info("DOMOMainWindow::","Creating index file....");
      QStringList idList;
      QStringList titleList;
      fDB->getIdTitles(idList,titleList,table);
      wh->changeFile(baseName + "index.html");
      wh->setCdOnly(false);
      wh->writeIndexFile(idList, titleList,filename + ".html",false);
     }
    } 
   } else {
    assert(false);
   }
  } catch(DbaseException* e) {
   fLogger->exception("DOMOMainWindow::",e->msg());
   fLogger->error("DOMOMainWindow::","Export failed");
   return;
  }
  
 } catch(IoException* e) {
  fLogger->exception("DOMOMainWindow::",e->msg());
  fLogger->error("DOMOMainWindow::","Export failed");
  return;
 }
 
 delete wh;
 delete wt;
 delete wm;
 delete wb;
 delete wp;
 delete wc;
 
 QString tmp2;
 fLogger->info("DOMOMainWindow::","Done...total time: " + tmp2.setNum(timer.elapsed()/1000) + " seconds");
}  
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::handleContextMenu(int menuId) {
        int id = -1;
        QString tmp = "";
        const QString curPage = Tabs->currentPage()->name();
        fLogger->debug("DOMOMainWindow::handleContextMenu","Context menu id is " + QString::number(menuId));
        
        try {    
                if (curPage == "queryTab") {
                        QSqlRecord* rec = queryResults->currentRecord();
                        if(rec != 0) {
                                id = rec->value("id").toInt();
                        } else {
                                fLogger->warning("DOMOMainWindow::handleContextMenu","No or invalid item selected");
                        }
                } else if (curPage == "browseTab") {
                        QSqlRecord* rec = browseTable->currentRecord();
                        if(rec != 0) {
                                id = rec->value("id").toInt();
                        } else {
                                fLogger->warning("DOMOMainWindow::handleContextMenu","No or invalid item selected");
                        }
                } else if (curPage == "compareTab") {
                        //first get the selected item
                        QListViewItem* it = 0;
                        if ((QListView::Extended == compareResults->selectionMode())||
                            (QListView::Multi == compareResults->selectionMode())) {
                                QListViewItemIterator iter(compareResults);
                                bool found = false;
                                while(iter.current() && !found){
                                        if (iter.current()->isSelected()){
                                                it = iter.current();
                                                found = true;
                                        }
                                        ++iter;
                                } 
                        } else {
                                it = compareResults->selectedItem();
                        } 
                        
                        if (it != 0) {
                                id = it->text(0).toInt();
                        } else {
                                fLogger->warning("DOMOMainWindow::handleContextMenu","No or invalid item selected");
                        }
                        
                } else {
                        fLogger->error("DOMOMainWindow::handleContextMenu","Wrong page: " + curPage);
                }
                
                int moodListSize = DomoGlobals::MoodList.size();
                //cout<<"moodlist size is : " << moodListSize << "\n";
                
                if (id > 0) {
                        if(menuId < 0)  {
                                fLogger->error("DOMOMainWindow::handleContextMenu","Invalid menu id!!!");
                                return;
                        } else if(menuId < 11)  {
                                fDB->scoreTrack(id,menuId);
                                fLogger->info("DOMOMainWindow::handleContextMenu","Track scored "
                                              + QString::number(menuId) + "/10");
                        } else if (menuId < 22) {
                                fDB->scoreSameArtist(id,menuId-11);
                                fLogger->info("DOMOMainWindow::handleContextMenu","Tracks  with same artist scored "
                                              + QString::number(menuId-11) + "/10");
                        } else if (menuId < 33) {
                                fDB->scoreSameCD(id,menuId-22);
                                fLogger->info("DOMOMainWindow::handleContextMenu","Tracks  with same CD scored"
                                              + QString::number(menuId-22) + "/10");
                        } else if (menuId < 44) {
                                fDB->scoreSameAlbum(id,menuId-33);
                                fLogger->info("DOMOMainWindow::handleContextMenu","Tracks with same album scored "
                                              + QString::number(menuId-33) + "/10");
                        } else if (menuId < 55) {
                                fDB->scoreSameGenre(id,menuId-44);
                                fLogger->info("DOMOMainWindow::handleContextMenu","Tracks with same genre scored "
                                              + QString::number(menuId-44) + "/10");
                        } else if (menuId < 55 + moodListSize) {
                                //set the mood, menu id = index into moodlist
                                QString mood = DomoGlobals::MoodList[menuId - 55];
                                fDB->setMood(id, mood);
                                fLogger->info("DOMOMainWindow::handleContextMenu","Mood set to " + mood + " for track with id "
                                              + QString::number(id) );
                        } else if (menuId < 55 + 2*moodListSize) {
                                QString mood = DomoGlobals::MoodList[menuId - 55 - 1*moodListSize];
                                fDB->moodSameCD(id, mood);
                                fLogger->info("DOMOMainWindow::handleContextMenu","Mood set to " + mood 
                                              + " for tracks from the same CD" );
                        } else if (menuId < 55 + 3*moodListSize) {
                                QString mood = DomoGlobals::MoodList[menuId - 55 - 2*moodListSize];
                                fDB->moodSameAlbum(id, mood);
                                fLogger->info("DOMOMainWindow::handleContextMenu","Mood set to " + mood 
                                              + " for tracks from the same album" );
                        } else {
                                fLogger->error("DOMOMainWindow::handleContextMenu","Invalid menu id!");
                        }
                } else {
                        fLogger->warning("DOMOMainWindow::handleContextMenu","Failed to get valid track ID");
                }
                
        } catch(DbaseException* e) {
                fLogger->exception("DOMOMainWindow::handleContextMenu",e->msg());
                fLogger->error("DOMOMainWindow::handleContextMenu","Failed to update track");
        }
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::trackInfoClicked() {
        TrackInfo* dialog = new TrackInfo( this );
        int id = -1;
        QString tmp = "";
        const QString curPage = Tabs->currentPage()->name();
        
        try {    
                if (curPage == "queryTab") {
                        QSqlRecord* rec = queryResults->currentRecord();
                        if (rec != 0) {
                                QSqlCursor cur( "Track" );
                                dialog->trackView->setSqlCursor( &cur );
                                dialog->trackView->setFilter( queryResults->filter() ); 
                                dialog->trackView->setSort(QSqlIndex::fromStringList( queryResults->sort(), &cur ) );
                                dialog->trackView->refresh();
                                int i = queryResults->currentRow();
                                if ( i < 0 ) i = 0; // Always use the first row
                                dialog->trackView->seek( i );
                                dialog->cdTitle->setText(fDB->getCDTitle(rec->value("cdno").toInt()));
                                dialog->exec();
                                queryResults->refresh();
                        } else {
                                fLogger->warning("DOMOMainWindow::trackInfoClicked","No item selected");
                        }
                        
                } else if (curPage == "browseTab") {
                        QSqlRecord* rec = browseTable->currentRecord();
                        if (rec != 0) {
                                QSqlCursor cur( "Track" );
                                dialog->trackView->setSqlCursor( &cur );
                                dialog->trackView->setFilter( browseTable->filter() ); 
                                dialog->trackView->setSort(QSqlIndex::fromStringList( browseTable->sort(), &cur ) );
                                dialog->trackView->refresh();
                                int i = browseTable->currentRow();
                                if ( i < 0 ) i = 0; // Always use the first row
                                dialog->trackView->seek( i );
                                dialog->cdTitle->setText(fDB->getCDTitle(rec->value("cdno").toInt()));
                                dialog->exec();
                                browseTable->refresh();
                        } else {
                                fLogger->warning("DOMOMainWindow::trackInfoClicked","No item selected");
                        }
                } else if (curPage == "compareTab") {
                        
                        //first get the selected item
                        QListViewItem* it = 0;
                        if ((QListView::Extended == compareResults->selectionMode())||(QListView::Multi == compareResults->selectionMode())) {
                                QListViewItemIterator iter(compareResults);
                                bool found = false;
                                while(iter.current() && !found){
                                        if (iter.current()->isSelected()){
                                                it = iter.current();
                                                found = true;
                                        }
                                        ++iter;
                                } 
                        } else {
                                it = compareResults->selectedItem();
                        }
                        
                        if (it != 0) {
                                id = it->text(0).toInt();
                                fLogger->info("DOMOMainWindow::trackInfoClicked","Selected ID is '" + tmp.setNum(id) + "'");
                                QSqlRecord* rec = fDB->trackToRec(id);
                                if (rec != 0) {
                                        dialog->trackView->setFilter( "id = " + QString::number(id) );
                                        dialog->trackView->refresh();
                                        dialog->trackView->seek( 0 );
                                        dialog->cdTitle->setText(fDB->getCDTitle(rec->value("cdno").toInt()));
                                        dialog->exec();
                                } else {
                                        fLogger->error("DOMOMainWindow::trackInfoClicked","Illegal id selected");
                                }
                        } else {
                                fLogger->warning("DOMOMainWindow::trackInfoClicked","No or invalid item selected");
                        }
                } else {
                        fLogger->error("DOMOMainWindow::trackInfoClicked","Wrong page: " + curPage);
                }
        } catch(DbaseException* e) {
                fLogger->exception("DOMOMainWindow::trackInfoClicked",e->msg());
                fLogger->error("DOMOMainWindow::trackInfoClicked","Get Track info failed");
        }
        delete dialog;
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::getStats() {
 try {    
  StatsDialog* dialog = new StatsDialog( this );
  dialog->getStats( fDB );
  dialog->exec();
 } catch(DbaseException* e) {
  fLogger->exception("DOMOMainWindow::getStats",e->msg());
  fLogger->error("DOMOMainWindow::getStats","Error during statistics collection...");
 }
}
//-----------------------------------------------------------------------------------------------------------
bool DOMOMainWindow::confirm(const QString msg) {
 QMessageBox mb("Are you sure?",
         msg,
         QMessageBox::Question,
         QMessageBox::Yes ,
         QMessageBox::No | QMessageBox::Default  | QMessageBox::Escape,
         QMessageBox::NoButton, this );
 mb.adjustSize();
 return ( mb.exec() == QMessageBox::Yes );
}
//-----------------------------------------------------------------------------------------------------------
bool DOMOMainWindow::deleteCD() {
 
 if(!confirm( tr("<font color=\"red\">WARNING:</font> All resources related to this CD will be removed!! Are you sure?") )) {
  fLogger->info("DOMOMainWindow::","Delete CD cancelled...");
  return true;
 }
 
 QString tmp = "";
 
 try {
  
  QSqlRecord* rec = cdList->currentRecord();
  if (rec != 0) {
   int id = rec->value("id").toInt();
   QString type = rec->value("type").toString(); 
   
   fDB->deleteCD(id,type,fCurCollection->getTitle());
  } else {
   fLogger->info("DOMOMainWindow::deleteCD","No item selected...");
   return true;
  }
  cdList->refresh();
  albumList->refresh();
  queryResults->refresh();
  initBrowseTab();
  fLogger->info("DOMOMainWindow::deleteCD","Delete CD successfull...");
  
 } catch(DbaseException* e) {
  fLogger->exception("DOMOMainWindow::deleteCD",e->msg());
  fLogger->error("DOMOMainWindow::deleteCD","Delete CD failed...");
  return false;
 }
 return true;
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::preferencesDialog() {
 PreferenceDiag* dialog = new PreferenceDiag( this );
 dialog->loadSettings( fSet );
 
 if (dialog->exec() == QDialog::Accepted) {
  dialog->retrieveSettings( fSet );
  bool b = fSet->setSettings();
  if (b) {
   DomoGlobals::logger->setDebugLevel(fSet->getInt(Settings::fDebugLevel));
   fLogger->info("DOMOMainWindow::preferencesDialog","Settings saved...");
  } else {
   fLogger->error("DOMOMainWindow::preferencesDialog","Save settings FAILED!");
  }
 } else {
  fLogger->info("DOMOMainWindow::preferencesDialog","Settings NOT saved...");
 }
 
 delete dialog;
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::lookupCD() {
 CDLookup lookup;
 lookup.setServer(fSet->get(Settings::fMbServer), fSet->getInt(Settings::fMbPort));
 lookup.setProxy(fSet->get(Settings::fMbProxyServer), fSet->getInt(Settings::fMbProxyPort));
 lookup.setDevice( fSet->get(Settings::fCdromDir) );
 vector<AudioCD*> cdList;
 vector<AudioCD*>::const_iterator it;
 try {
  fLogger->info("DOMOMainWindow::lookupCD","Please wait while contacting musicbrainz server...");
  lookup.lookupCD(cdList, true);
  fLogger->info("DOMOMainWindow::lookupCD","Server contacted...");
  MusicBrainzDialog* dialog = new MusicBrainzDialog( this );
  dialog->setGenreList( fRcMan->getGenreListRef() );
  
  it = cdList.begin();
  QString tmp;
  while(it != cdList.end()) {
   tmp.setNum( (*it)->getFileCount() );
   dialog->addAlbum( (*it)->getIDString() , (*it)->getTitle(), (*it)->getArtist() , tmp);
   ++it;
  }
  
  if (dialog->exec() == QDialog::Accepted) {
   int id = dialog->getSelectedId().toInt();
   
   if (id < 0) return;
   AudioCD* cd = cdList[ id ];
   
   fLogger->info("DOMOMainWindow::lookupCD","Adding CD to collection...)");
   cd->setYear(dialog->getYear());
   cd->setGenre(dialog->getGenre());
   if ( !fConsistencyChecker->makeConsistent( *cd->getTracklist(), progressBar ) ) {
    fLogger->warning("DOMOMainWindow::lookupCD","Consistency check + initiating action aborted...");
    delete dialog;
    return;
   } 
   cd->setTrackAttributes();  //set track year/genre attributes + set name to "Track xx.cda"
   
   fDB->add(cd,progressBar); //correct CD id is set here
   fDB->addToCollection(cd,fCurCollection);
   fDB->addAlbum(cd);
   fLogger->info("DOMOMainWindow::lookupCD","CD added succesfully...");
   delete cd;
   
   this->cdList->refresh();
   this->albumList->refresh();
  } else {
   fLogger->warning("DOMOMainWindow::lookupCD","CD not added...");
  }
  
  delete dialog;
 } catch (DbaseException* ex) {
  fLogger->exception("DOMOMainWindow::lookupCD",ex->msg());
  fLogger->error("DOMOMainWindow::lookupCD","Add CD failed...");
 } catch (Exception* e) {
  fLogger->exception("DOMOMainWindow::",e->msg());
  fLogger->error("DOMOMainWindow::lookupCD","CD Lookup failed");
 }   
 
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::deleteCurCollection() {
 QString title = collectionList->currentText();
 
 QString msg = tr("<font color=\"red\">WARNING: This will erase all Tracks, CDs, ... in the CURRENT collection"
    " are you SURE you want to continue?</font>");
 if (confirm(msg)) {
  fLogger->info("DOMOMainWindow::deleteCurCollection","Deleting collection '" + title + "' ...");
  try {
   fDB->deleteCollection(title);
   
   updateCollectionList();
  } catch (DbaseException* ex) {
   fLogger->exception("DOMOMainWindow::deleteCurCollection",ex->msg());
   fLogger->error("DOMOMainWindow::deleteCurCollection","Delete collection '" + title + "' failed");
  }
 } else {
  fLogger->warning("DOMOMainWindow::deleteCurCollection","Delete collection '" + title + "' cancelled");
 }
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::backupDatabase() {
 BackupDbaseDiag* dialog = new BackupDbaseDiag(this);
 
 QString filename = fSet->get(Settings::fExportDir) + DomoGlobals::DIR_SEP + fDB->getName() + ".dump.sql";
 dialog->setFields( fDB->getUser(), fDB->getPwd(), fDB->getName(), filename);
 
 try {
  if ( dialog->exec() == QDialog::Accepted) {
   if (dialog->backup(fDB)) {
    fLogger->info("DOMOMainWindow::backupDatabase","Backup process started in background");
   } else {
    fLogger->error("DOMOMainWindow::backupDatabase",
            "Failed to start backup process, please check your parameters");
   }
  } else {
   fLogger->info("DOMOMainWindow::backupDatabase","Backup cancelled");
  }
 } catch (IoException* e) {
  fLogger->exception("DOMOMainWindow::backupDatabase",e->msg());
  fLogger->error("DOMOMainWindow::backupDatabase","Backup of database failed");
 }
 delete dialog;
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::renameAlbum() {
 int row = albumList->currentRow();
 QString oldTitle = albumList->text(row,1);
 QString newTitle = "";
 
 RenameAlbumDialog* dialog = new RenameAlbumDialog();
 dialog->oldTitle->setText(oldTitle); 
 dialog->newTitle->setText(oldTitle); 
 
 if(dialog->exec() == QDialog::Accepted) {
  newTitle = dialog->newTitle->text();
  
  try {
   fDB->renameAlbum(oldTitle,newTitle);
   albumList->refresh();
   queryResults->refresh();
   initBrowseTab();
  } catch (DbaseException* ex) {
   fLogger->exception("DOMOMainWindow::renameAlbum",ex->msg());
   fLogger->error("DOMOMainWindow::renameAlbum","Failed to rename album '" + oldTitle + "'");
  }
 }
 delete dialog;
}
//-----------------------------------------------------------------------------------------------------------
void DOMOMainWindow::contextMenuEvent( QContextMenuEvent * event ) {
 //int x = event->x();
 //int y = event->y();
 //cout<<"Event x=" << x << " y=" << y <<"\n";
 
 //what follows is a rather ugly hack.  Since I'm working with designer I cant just override contextMenuEvent()
 //for every widget inside the main window.  A workaround is to let the mainwindow receive the event and then ignore
 //it if it doesn't occur above one of the relevant widgets.
 
 if( compareResults->hasMouse() ||
     queryResults->hasMouse() ||
     cdList->hasMouse() ||
     albumList->hasMouse() ||
     browseTable->hasMouse()  ) {
 } else {
  //cout<<"Event ignored\n";
  event->ignore();
  return;
 }
 
 //top evel menu, level 1
 QPopupMenu* contextMenu = new QPopupMenu( this );
 
 if(cdList->hasMouse()) {
  contextMenu->insertItem(  QIconSet( QPixmap::fromMimeSource("stop.png")),
       "&Delete CD",  this, SLOT(deleteCD()));
 } else if(albumList->hasMouse()) {
  contextMenu->insertItem(  QIconSet( QPixmap::fromMimeSource("refresh.png")),
       "&Rename Album",  this, SLOT(renameAlbum()));
 } else {
  
  QLabel *caption = new QLabel( "<font color=darkblue><u><b>"
           "Track Operations</b></u></font>", this );
  caption->setAlignment( Qt::AlignCenter );
  contextMenu->insertItem( caption );
  contextMenu->insertItem( "&View/Edit Track",  this, SLOT(trackInfoClicked()));
  
  //level 2
  QPopupMenu *scoreSubmenu = new QPopupMenu( this );
  
  //level 3
  QPopupMenu *scoreSubmenu1 = new QPopupMenu( this );
  for (int i = 0; i < 11; ++i) scoreSubmenu1->insertItem( QString::number(i),i);
  scoreSubmenu->insertItem( "This &Track", scoreSubmenu1);
  
  QPopupMenu *scoreSubmenu2 = new QPopupMenu( this );
  for (int i = 11; i < 22; ++i) scoreSubmenu2->insertItem( QString::number(i-11),i);
  scoreSubmenu->insertItem( "This &Artist", scoreSubmenu2);
  
  QPopupMenu *scoreSubmenu3 = new QPopupMenu( this );
  for (int i = 22; i < 33; ++i) scoreSubmenu3->insertItem( QString::number(i-22),i);
  scoreSubmenu->insertItem( "This &CD", scoreSubmenu3);
  
  QPopupMenu *scoreSubmenu4 = new QPopupMenu( this );
  for (int i = 33; i < 44; ++i) scoreSubmenu4->insertItem( QString::number(i-33),i);
  scoreSubmenu->insertItem( "This A&lbum", scoreSubmenu4);
  
  QPopupMenu *scoreSubmenu5 = new QPopupMenu( this );
  for (int i = 44; i < 55; ++i) scoreSubmenu5->insertItem( QString::number(i-44),i);
  scoreSubmenu->insertItem( "This &Genre", scoreSubmenu5 );
  
  contextMenu->insertItem( "&Score", scoreSubmenu );
  
  //Mood
  QPopupMenu* moodSubmenu = new QPopupMenu( this );
  
  QPopupMenu* moodSubmenu6 = new QPopupMenu( this );
  int i = 55;
  for ( QStringList::ConstIterator it = DomoGlobals::MoodList.begin(); it != DomoGlobals::MoodList.end(); ++it ) {
   moodSubmenu6->insertItem( *it, i );
   ++i;
  }
  moodSubmenu->insertItem( "This &Track", moodSubmenu6 );
  
  QPopupMenu* moodSubmenu7 = new QPopupMenu( this );
  for ( QStringList::ConstIterator it = DomoGlobals::MoodList.begin(); it != DomoGlobals::MoodList.end(); ++it ) {
   moodSubmenu7->insertItem( *it, i );
   ++i;
  }
  moodSubmenu->insertItem( "This &CD", moodSubmenu7 );
  
  QPopupMenu* moodSubmenu8 = new QPopupMenu( this );
  for ( QStringList::ConstIterator it = DomoGlobals::MoodList.begin(); it != DomoGlobals::MoodList.end(); ++it ) {
   moodSubmenu8->insertItem( *it, i );
   ++i;
  }
  moodSubmenu->insertItem( "This &Album", moodSubmenu8 );
  
  contextMenu->insertItem( "Set &Mood for", moodSubmenu );
  
  connect( scoreSubmenu1, SIGNAL( activated(int) ), this, SLOT( handleContextMenu(int) ) );
  connect( scoreSubmenu2, SIGNAL( activated(int) ), this, SLOT( handleContextMenu(int) ) );
  connect( scoreSubmenu3, SIGNAL( activated(int) ), this, SLOT( handleContextMenu(int) ) );
  connect( scoreSubmenu4, SIGNAL( activated(int) ), this, SLOT( handleContextMenu(int) ) );
  connect( scoreSubmenu5, SIGNAL( activated(int) ), this, SLOT( handleContextMenu(int) ) );
  connect( moodSubmenu6, SIGNAL( activated(int) ), this, SLOT( handleContextMenu(int) ) );
  connect( moodSubmenu7, SIGNAL( activated(int) ), this, SLOT( handleContextMenu(int) ) );
  connect( moodSubmenu8, SIGNAL( activated(int) ), this, SLOT( handleContextMenu(int) ) );
 }
 contextMenu->exec( QCursor::pos() );
 delete contextMenu;
}
//-----------------------------------------------------------------------------------------------------------


