/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#ifndef INC_FILE_H
#define INC_FILE_H

#include <vector>
#include "textostream.h"
#include "qstringlist.h"

using std::vector;
using std::ostream;
using WriterSpace::TextOStream;

namespace FileSpace {

    class File {
	private:
	int fID;
	int fCDno;
	QString fName;
	QString fPath;
	QString fDesc;
	long fSize;
	QString fCreated;
	QString fModified;
	QString fAccessed;
	QStringList fAttributes;
	
	/* Attributes:
	   READ_ONLY
	   DIRECTORY
	   HIDDEN
	   
	   others may be defined later on such as:
                  VOLUME_LABEL, ARCHIVE, COMPRESSED, TEMPORARY
	*/
	
	public:
	
	typedef vector<File*> Filelist;
	typedef Filelist::iterator FileIterator;
	typedef Filelist::const_iterator ConstFileIterator;
	typedef Filelist::reverse_iterator ReverseFileIterator;
	typedef Filelist::const_reverse_iterator ConstReverseFileIterator;
	
	File();
	File(const QString, const QString, const long);
	File(const File&);
	virtual ~File();
	
	QString getDescription() const;
	void setDescription(const QString);
	
	QString getName() const;
	void setName(const QString);
	
	QString getPath() const;
	void setPath(const QString);
	
	long getSize() const;
	void setSize(const long);
	void setSize(const QString);
	
	void setHidden(const bool);
	void setDirectory(const bool);
	void setReadOnly(const bool);
	
	QString getCreated() const;
	void setCreated(const QString);
	
	QString getModified() const;
	void setModified(const QString);
	
	QString getAccessed() const;
	void setAccessed(const QString);
	
	void setAttributes(const QString); 
	QString getAttributes() const;
	
	QString getExtension() const;
	
	int getCDno() const;
	void setCDno(const int);
	void setCDno(const QString);
	
	int getID() const;
	void setID(const int);
	void setID(const QString);
	
	QString absPath() const;
	
	virtual void writeTxt(TextOStream* ostr) const;
	virtual void writeHtml(TextOStream* ostr) const;
	virtual void writeM3u(TextOStream* ostr) const;
	virtual void writeB4s(TextOStream* ostr) const;
	virtual void writePls(TextOStream* ostr,const uint num = -1) const;
	virtual void writeCsv(TextOStream* ostr,const QStringList& fields) const;
};

} //Close namespace FileSpace

ostream& operator<<(ostream &, FileSpace::File&);

#endif

