/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#ifndef INC_DOMOGLOBALS_H
#include "domoglobals.h"
#define INC_DOMOGLOBALS_H
#endif

#ifndef FILECHOOSER_H
#include "filechooser.h"
#define FILECHOOSER_H
#endif

#include "assert.h"
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qfiledialog.h>
#include <qlayout.h>

FileChooser::FileChooser( QWidget *parent, const char *name )
    : QWidget( parent, name ), md( Directory )
{
    QGridLayout *layout = new QGridLayout( this,2,2 );
    layout->setMargin( 0 );

    lineEdit = new QLineEdit( this, "filechooser_lineedit" );
    layout->addWidget( lineEdit,0,0 );

    connect( lineEdit, SIGNAL( textChanged( const QString & ) ),
	     this, SIGNAL( fileNameChanged( const QString & ) ) );

  //  button = new QPushButton( "...", this, "filechooser_button" );
  //  button->setFixedWidth( button->fontMetrics().width( " ... " ) );
  
    QPixmap icon = QPixmap::fromMimeSource("folder.png");
    button = new QPushButton(icon,"", this, "filechooser_button" );
    button->setFixedWidth( icon.width() );
    button->setFixedHeight( lineEdit->height() );
  

    layout->addWidget( button,0,1 );

    connect( button, SIGNAL( clicked() ), this, SLOT( chooseFile() ) );

    checkBox = new QCheckBox( tr("Directories Only") , this, "onlyDirs_chkbox" );
    checkBox->setChecked(true);
   
    layout->addWidget( checkBox,1,0 );
		
    connect( checkBox, SIGNAL( toggled(bool) ), this, SLOT( setOnlyDirs(bool) ) );


    setFocusProxy( lineEdit );
}

void FileChooser::setOnlyDirs( bool b )
{
	if (b) {
		this->setMode(Directory);
	} else {
		this->setMode(File);
	}
}


bool FileChooser::onlyDirs()
{
    return (md == Directory);
}


void FileChooser::setMode( Mode m )
{
    md = m;
    switch(m) {
    case Directory:
	checkBox->setChecked(true);
	break;
    case File:
	checkBox->setChecked(false);
	break;
    default:
	assert(false);
	break;
    }	
}

FileChooser::Mode FileChooser::mode() const
{
    return md;
}

void FileChooser::setFileName( const QString &fn )
{
    lineEdit->setText( fn );
}

QString FileChooser::fileName() const
{
    return lineEdit->text();
}

bool FileChooser::chooseFile()
{
    QString fn;
    if ( mode() == File ) {
	fn = QFileDialog::getOpenFileName( lineEdit->text(), "DOMO supported Files (" 
					   + DomoGlobals::FILE_TYPES + ");;All files (*)", this );
    }    else {
	fn = QFileDialog::getExistingDirectory( lineEdit->text(),this);
    }
    
    //user pressed cancel
    if (fn.isNull()) return false;
    
    if ( !fn.isEmpty() ) {
		lineEdit->setText( fn );
	emit fileNameChanged( fn );
    }
    
    return true;
}

void FileChooser::disableDirs() {
    checkBox->setEnabled(false);
}
