/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#include "fileio.h"
#include "stringmanip.h"
#include "domoglobals.h"
#include <string>
#include "qdir.h"
#include "qfile.h"
#include "qfileinfo.h"
#include <fstream>
#include <iostream>
using std::cout;
//-----------------------------------------------------------------------------------------------------------
FileIO::FileIO() {
}
//-----------------------------------------------------------------------------------------------------------
FileIO::~FileIO() {
}
//-----------------------------------------------------------------------------------------------------------
/**
* Thanks to: http://www.planet-source-code.com/vb/scripts/ShowCode.asp?txtCodeId=6147&lngWId=3
*/
bool FileIO::copyFile(const QString source, const QString destination) {
    if (!exists(source)) return false;
    bool ok = false;
    std::ifstream fin(source.latin1());
    std::ofstream fout(destination.latin1(), std::ios_base::out | std::ios_base::trunc);
    if (fin.is_open() && fout.is_open()) {
	fout << fin.rdbuf();
	ok = true;
    } else {
	ok = false;
    }
    fin.close();
    fout.close();
    return ok;
}
//-----------------------------------------------------------------------------------------------------------
bool FileIO::exists(const QString name) {
    return QFile::exists(name);
}
//-----------------------------------------------------------------------------------------------------------
bool FileIO::deleteFile(const QString name) {
    return QFile::remove(name);
}
//-----------------------------------------------------------------------------------------------------------
bool FileIO::renameFile(const QString oldname, const QString newname) {
	QString name = newname.stripWhiteSpace();
	QString tmp = "";
	QString oname = "";
	
	uint i = 1;
	while(exists(name)) {
		oname = name;
		name = stripExtension(newname) + " (" + tmp.setNum(i) + ")." + getExtension(newname);
		DomoGlobals::logger->warning("FileIO::renameFile","File '" + oname + "' exists, trying '" + name + "' instead");
		tmp = "";
		++i;
	}
	
	QDir dir;
	return dir.rename(oldname, name.stripWhiteSpace());
}
//-----------------------------------------------------------------------------------------------------------
bool FileIO::isDirectory(const QString name) {
    if (!exists(name)) return false;
    QFileInfo file(name);
    return file.isDir();
}
//-----------------------------------------------------------------------------------------------------------
bool FileIO::deleteDirectory(const QString dir){
	QDir d(dir);
	
	QStringList entries = d.entryList();
	for(QStringList::Iterator it = entries.begin(); it != entries.end(); ++it) {
		if(*it == "." || *it == "..") continue;
		
		QFileInfo info(d, *it);
		
		//cout<<"Deleting '" << info.filePath() << "\n";
		if(info.isDir()) {
			if(!FileIO::deleteDirectory( info.filePath()) ) {
				DomoGlobals::logger->error("FileIO::deleteDirectory","Failed to delete dir '" + info.filePath() + "'");
				return false;
			} else {
				DomoGlobals::logger->warning("FileIO::deleteDirectory","Deleted directory '" + info.filePath() + "'");
			}
		} else {
			if(!d.remove(info.filePath())){
				DomoGlobals::logger->error("FileIO::deleteDirectory","Failed to delete file '" + info.filePath() + "'");
			} else {
				DomoGlobals::logger->warning("FileIO::deleteDirectory","Deleted file '" + info.filePath() + "'");
			}
			
		}
	}
	QString name = d.dirName();
	//cout<<"dirname is '" << name << "\n";

	if(!d.cdUp()) {
		DomoGlobals::logger->error("FileIO::deleteDirectory","Failed to cdUp() from directory '" + d.path() + "'");
		return false;
	}
	//cout<<"after cdup, deleting '" << name << "\n";
	
	if(!d.rmdir(name)) {
		DomoGlobals::logger->error("FileIO::deleteDirectory","Failed to delete directory '" + name + "'");
	} else {
		DomoGlobals::logger->warning("FileIO::deleteDirectory","Deleted directory '" + name + "'");
	}
	
	return true;
}
//-----------------------------------------------------------------------------------------------------------
bool FileIO::isReadable(const QString name) {
	QFile f(name);
	bool ok = f.open(IO_ReadOnly);
	f.close();
	return ok;
}
//-----------------------------------------------------------------------------------------------------------
bool FileIO::isWriteable(const QString name) {
	QFile f(name);
	//read only flag to prevent file from being truncated
	bool ok = f.open(IO_WriteOnly | IO_ReadOnly);
	f.close();
	return ok;
}
//-----------------------------------------------------------------------------------------------------------
void FileIO::splitAbsPath(const QString absPath, QString& dir, QString& fname) {
	QFileInfo f(absPath);
	dir = f.dirPath(true);
	if(dir == ".") dir = QDir::currentDirPath();
	fname = f.fileName();
}
//-----------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------
