/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_IWRITER_H
#include "iwriter.h"
#define INC_IWRITER_H
#endif

#include "qfile.h"
#include "assert.h"
#include <iostream>
using std::cout;

using WriterSpace::IWriter;
using WriterSpace::TextOStream;
using std::ios;
using std::cout;

//-----------------------------------------------------------------------------------------------------------
IWriter::IWriter(){
  fFilename = "";
  fFile = 0;
}
//-----------------------------------------------------------------------------------------------------------
IWriter::IWriter(const QString s){
  if (s.isEmpty()) throw (new IoException("IWriter::IWriter(const QString)","Empty filename"));
  fFilename = s;
  fFile = 0;
  openFile();   
}
//-----------------------------------------------------------------------------------------------------------
IWriter::IWriter(const QString s, const bool append){
  if (s.isEmpty()) throw (new IoException("IWriter::IWriter(const QString)","Empty filename"));
  fFilename = s;
  fFile = 0;
    
    if (!append) {
	openFile();
    } else {
	openFileAppend();
    }
    
}
//-----------------------------------------------------------------------------------------------------------
IWriter::~IWriter(){
    fFile->flush();
}
//-----------------------------------------------------------------------------------------------------------
void IWriter::setFilename(const QString s){
	fFilename = s;
}
//-----------------------------------------------------------------------------------------------------------
QString IWriter::getFilename() const{
	return fFilename;
}
//-----------------------------------------------------------------------------------------------------------
void IWriter::openFileAppend(){
    
    if (fFile != 0) {
	delete fFile;
    }
    fFile = new TextOStream(fFilename, IO_WriteOnly | IO_Append);
    
    if (!fFile->isGood()) throw (new IoException("IWriter::openFileAppend","Failed to open file '" + fFilename + "'"));
}
//-----------------------------------------------------------------------------------------------------------
void IWriter::openFile(){
   
    if (fFile != 0) {
	delete fFile;
    }
    
    fFile = new TextOStream(fFilename, IO_WriteOnly | IO_Truncate );
    
    if (!fFile->isGood()) throw (new IoException("IWriter::openFile","Failed to open file '" + fFilename + "'"));
}
//-----------------------------------------------------------------------------------------------------------
void IWriter::closeFile(){
    if (fFile == 0) throw (new IoException("IWriter::closeFile","fFile is 0, current file is '" + fFilename + "'"));
    fFile->flush();
    (fFile->device())->close();
    fFile->unsetDevice();
 }
//-----------------------------------------------------------------------------------------------------------
void IWriter::changeFile(const QString s, const bool append) {
    if (fFile == 0) throw (new IoException("IWriter::changeFile","fFile is 0, current file is '" + fFilename + "' attempting to change to '" + s + "'"));
    
    TextOStream::Mode m = fFile->getMode();
    bool b = fFile->isCdOnly();
    
    if (fFile->isOpen()) {
	closeFile();
    }
    
    setFilename(s);
    if (append) {
	openFileAppend();
    } else {
	openFile();
    }
    
    fFile->setMode(m);
    fFile->setCdOnly(b);
}
//-----------------------------------------------------------------------------------------------------------
void IWriter::setCdOnly(const bool b) {
   fFile->setCdOnly(b);
}
//-----------------------------------------------------------------------------------------------------------
void IWriter::setShort(const bool b) {
    
    if (fFile == 0) throw (new IoException("IWriter::setShort","fFile is 0, current file is '" + fFilename + "'"));
    
    TextOStream::Mode m;

    if (b) {
	m = TextOStream::SHORT;
    } else {
    	m = TextOStream::LONG;
    }
    
    fFile->setMode(m);
}
//-----------------------------------------------------------------------------------------------------------
void IWriter::setLong(const bool b) {
    if (fFile == 0) throw (new IoException("IWriter::setLong","fFile is 0, current file is '" + fFilename + "'"));
    
    if (b) {
	setShort(false);
    } else {
	setShort(true);
    }
}
//-----------------------------------------------------------------------------------------------------------
void IWriter::flush() {
    if (fFile == 0) throw (new IoException("IWriter::flush","fFile is 0, current file is '" + fFilename + "'"));
    fFile->flush();
}
//-----------------------------------------------------------------------------------------------------------
void IWriter::deleteCurrentFile(){
    if (fFile == 0) throw (new IoException("IWriter::deleteCurrentFile","fFile is 0, current file is '" + fFilename + "'"));
    closeFile();
    bool ok = QFile::remove(fFilename);
    
    if (!ok) throw (new IoException("IWriter::deleteCurrentFile","Failed to remove file '" + fFilename + "'"));
 }
//-----------------------------------------------------------------------------------------------------------
TextOStream* IWriter::getStream() {
    if ((fFile == 0) ||(!fFile->isGood())) throw (new IoException("IWriter::getStream","fFile is null or not good"));    
    return fFile;
}
//-----------------------------------------------------------------------------------------------------------
