/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/


#ifndef INC_DOMOGLOBALS_H
#include "domoglobals.h"
#define INC_DOMOGLOBALS_H
#endif

#ifndef INC_LOG_H
#include "log.h"
#define INC_LOG_H
#endif

#include "qdatetime.h"
#include <iostream>

using std::cout;
using std::cerr;

TextOStream* Log::fOstream = 0;
QString Log::fTarget = DomoGlobals::LOG_FILE;
uint Log::fDebugLevel = 3;
//-----------------------------------------------------------------------------------------------------------
Log::Log() {
    //cout<<"Created new Log object\n";
   //fOstream is static, if this is the first Log obj
   //fOstream is 0 else the stream is already open
}
//-----------------------------------------------------------------------------------------------------------
Log::Log(const QString target) {
   fTarget = target;
   //fOstream is static, if this is the first Log obj
   //fOstream is 0 else the stream is already open
}
//-----------------------------------------------------------------------------------------------------------
void Log::open() {
    
    if (fOstream != 0) {
	cerr<<"Warning: Log::open, Open() method called on already open stream.  Deleting existing stream handler\n";
	fOstream->close();
	delete fOstream;
    }
    
    if(fTarget.stripWhiteSpace().isEmpty()) {
	cerr<<"Error: Log::open, empty target, using default '" << DomoGlobals::LOG_FILE << "'\n";
	fTarget = DomoGlobals::LOG_FILE;
    }
    
    fOstream = new TextOStream(fTarget, IO_WriteOnly | IO_Append);
    
    append("# Start of new log session created by Domo v" 
	  + DomoGlobals::DOMO_VERSION + " @ " +  QDateTime::currentDateTime().toString("ddd MMMM d hh:mm:ss"));
}
//-----------------------------------------------------------------------------------------------------------
Log::~Log() {
}
//-----------------------------------------------------------------------------------------------------------
void Log::setTarget(const QString name) {
    fTarget = name;
}
//-----------------------------------------------------------------------------------------------------------
bool Log::append(const QString s, const uint debug) {
    if (debug <= fDebugLevel) {
	cout  << "[" << debug << "] " << s << "\n";
	
	if ((fOstream == 0) || !fOstream->isGood()) {
	    cerr<<"[0] [Log::append(QString, debug)] ERROR: Problem using output stream!\n";
	    return false;
	} else {
	    *fOstream << '[' << QTime::currentTime().toString("hh:mm:ss") << "] [" << debug << "] " << s << "\n";
	    fOstream->flush();
	    return ((fOstream != 0) && fOstream->isGood());
	}
    }
    return true;
}
//-----------------------------------------------------------------------------------------------------------
bool Log::append(const QString method, const QString s, const uint debug) {
    if (debug <= fDebugLevel) {
	cout << "[" << debug << "] [" << method << "] " << s << "\n";
	
	if ((fOstream == 0) || !fOstream->isGood()) {
	    cerr<<"[0] [Log::append(QString, QString, debug)] ERROR: Problem using output stream!\n";
	    return false;
	} else {
	    *fOstream << '[' << QTime::currentTime().toString("hh:mm:ss") << "] [" << debug << "] [" << method << "] " << s << "\n";
	    fOstream->flush();
	    return ((fOstream != 0) && fOstream->isGood());
	}
    }
    
    return true;
}
//-----------------------------------------------------------------------------------------------------------
void Log::setDebugLevel(const uint i) {
    uint j = i;
    if (j > 3) j = 3;
    fDebugLevel = j;
}
//-----------------------------------------------------------------------------------------------------------
uint Log::getDebugLevel() const {
    return fDebugLevel;
}
//-----------------------------------------------------------------------------------------------------------
bool Log::error(const QString method, const QString msg) {
    return append(method,"Error: " + msg,0);
}
//-----------------------------------------------------------------------------------------------------------
bool Log::exception(const QString method, const QString msg) {
    return append(method,"Exception: " + msg,0);
}
//-----------------------------------------------------------------------------------------------------------
bool Log::warning(const QString method, const QString msg) {
    return append(method,"Warning: " + msg,1);
}
//-----------------------------------------------------------------------------------------------------------
bool Log::info(const QString method, const QString msg) {
    return append(method, msg,2);
}
//-----------------------------------------------------------------------------------------------------------
bool Log::debug(const QString method, const QString msg) {
    return append(method,msg,3);
}
//-----------------------------------------------------------------------------------------------------------
