/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_MP3CD_H
#include "mp3cd.h"
#define INC_MP3CD_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

using FileSpace::File;
using TrackSpace::Track;
using MediaSpace::Mp3CD;
//-----------------------------------------------------------------------------------------------------------
Mp3CD::Mp3CD() : CD(){
    fGenre = "";
}	
//-----------------------------------------------------------------------------------------------------------
Mp3CD::Mp3CD(const int i) : CD(i){
    fGenre = "";
}
//-----------------------------------------------------------------------------------------------------------
Mp3CD::~Mp3CD(){
}
//-----------------------------------------------------------------------------------------------------------
QString Mp3CD::getGenre() const{
    return fGenre;
}
//-----------------------------------------------------------------------------------------------------------
void Mp3CD::setGenre(const QString g){
    fGenre = g;
}
//-----------------------------------------------------------------------------------------------------------
void Mp3CD::setTracklist(Track::Tracklist& f) {
    clear();
    Track::TrackIterator FI = f.begin();
    bool setGenre = ((fGenre != "")&&(fGenre != "Various"));
    
    while (FI != f.end()) {
	if (setGenre) (*FI)->setGenre( fGenre );
	add(*FI);
	++FI;
    }
	
}
//-----------------------------------------------------------------------------------------------------------
void Mp3CD::writeTxt(TextOStream* ostr) const {
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Mp3CD::writeTxt","Pre: Ostream is 0 or not good"));
    
    CD::writeTxt(ostr);
    
    switch (ostr->getMode()) {
    case TextOStream::SHORT:
	break;
    case TextOStream::LONG:
	if (!empty(fGenre)) *ostr << "Genre: "  << fGenre << "\n";
	break;
    default:
	break;
    }
    *ostr << "Tracks:\n\n";
    writeTxtFileList(ostr);	
    
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Mp3CD::writeTxt","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void Mp3CD::writeHtml(TextOStream* ostr) const{
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Mp3CD::writeHtml","Pre: Ostream is 0 or not good"));
    
    switch (ostr->getMode()) {
    case TextOStream::SHORT:
	if (ostr->isCdOnly()) {
	    *ostr << "<tr>\n";
	    CD::writeHtml(ostr);
	    *ostr << "</tr>\n";
	} else {
	    CD::writeHtml(ostr);
	}
	break;
    case TextOStream::LONG:
	if (ostr->isCdOnly()) {
	    *ostr << "<tr>\n";
	    CD::writeHtml(ostr);
	    *ostr << "   <td>" << fGenre << "</td>\n";
	    *ostr << "</tr>\n";
	} else {
	    CD::writeHtml(ostr);
	    if (!empty(fGenre))         *ostr << "<b>Genre: </b>"          << fGenre          << "<br>\n";
	}
	break;
    default:
	break;
    }
    
    if (!ostr->isCdOnly()) {
	*ostr << "<b>Tracks:</b><br><br>\n";
	writeHtmlFileList(ostr);	
    }
    
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Mp3CD::writeHtml","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
//The M3U format only deals with Tracks, not CDs
void Mp3CD::writeM3u(TextOStream* ostr) const {
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Mp3CD::writeM3u","Pre: Ostream is 0 or not good"));
    writeM3uFileList(ostr);
}
//-----------------------------------------------------------------------------------------------------------
//The Pls format only deals with Tracks, not CDs
void Mp3CD::writePls(TextOStream* ostr) const {
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Mp3CD::writePls","Pre: Ostream is 0 or not good"));
    writePlsFileList(ostr);
}
//-----------------------------------------------------------------------------------------------------------
void Mp3CD::writeB4s(TextOStream* ostr) const{
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Mp3CD::writeB4s","Pre: Ostream is 0 or not good"));
    writeB4sFileList(ostr);
}
//-----------------------------------------------------------------------------------------------------------
void Mp3CD::writeCsv(TextOStream* ostr, const QStringList& fields) const{
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("CD::writeCsv","Ostream is 0 or not good"));
    
    CD::writeCsv(ostr,fields);
    
    if (ostr->isCdOnly()) {
	if (fields.contains("Genre")) *ostr << fGenre              << ";";
    }
}
//-----------------------------------------------------------------------------------------------------------
QString Mp3CD::getTypeString() const {
    return  "MP3_CD";    
}
//-----------------------------------------------------------------------------------------------------------
