/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#include "musicexpression.h"
#include "stringmanip.h"
#include "domoglobals.h"
#include "assert.h"
#include "qregexp.h"
#include <iostream>

using  std::cout;
//-----------------------------------------------------------------------------------------------------------
MusicExpression::MusicExpression() {
}
//-----------------------------------------------------------------------------------------------------------
MusicExpression::~MusicExpression(){
}
//-----------------------------------------------------------------------------------------------------------
QString MusicExpression::translateToRegex(const QString pat) {
    QMap<QString, uint> map;
    return MusicExpression::translateToRegex(pat,map);
}
//-----------------------------------------------------------------------------------------------------------
QString MusicExpression::translateToRegex(const QString pat, QMap<QString, uint>& indexMap) {
    indexMap.clear();
    QString pattern = pat;
    //replace multiple spaces with one space
    pattern = pattern.simplifyWhiteSpace();
    
    if (pattern.isEmpty()) return "";
    
    uint i = 0;
    uint index = 1;
    uint patLen = pattern.length();
    QChar field;
    QString regEx = "";
    
    while(i <  patLen) {
	if(pattern[i] == '%') {
	    //cout<<"% found\n";
	    ++i;
	    if (i < patLen) {
		field = pattern[i];
		//cout<<"field is '" << field << "'\n";
		
		switch(field) {
		case 'a':
		    indexMap["%a"] = index;
		    regEx += "(.*)";
		    break;
		case 't':
		    indexMap["%t"] = index;
		    regEx += "(.*)";
		    break;
		case 'y':
		    indexMap["%y"] = index;
		    //a year should have 2 or 4 digits
		    regEx += "((?:\\d{2})|(?:\\d{4}))";
		    break;
		case 'g':
		    indexMap["%g"] = index;
		    //a genre may contain only letters
		    regEx += "((?:[a-z]|[A-Z])*)";
		    break;
		case 'n':
		    indexMap["%n"] = index;
		    //regEx += "(\\d*)";
		    //a track number contains: 1,2 or 3 digits
		    regEx +="(\\d{1,3})";
		    break;
		case 'c':
		    indexMap["%c"] = index;
		    regEx += "(.*)";
		    break;
		case 'i':
		    //ignore entry
		    regEx += "(.*)";
		    break;
		default:
		    DomoGlobals::logger->warning("MusicExpression::translateToRegex",
						 "Ignoring '%' placeholder with undefined type '" + QString(field) 
						 + "' in pattern '" + pattern + "'");
		    //ignore entry
		    regEx += "(.*)";
		    break;
		}
	    } else {
		DomoGlobals::logger->warning("MusicExpression::translateToRegex",
					     "'%' placeholder with no type in pattern '" + pattern + "'");
		//ignore entry
		regEx += "(.*)";
	    }
	} else {
	    // cout<<"pat[i] is '" << pattern[i] << "'\n";
	    switch(pattern[i].latin1()) {
	    case ' ':
		regEx += "(\\s+)";
		break;
	    case '.':
		regEx += "(\\.)";
		break;	
	    case '-':
		regEx += "(\\-)";
		break;	
	    case '$':
		regEx += "(\\$)";
		break;	
	    case '(':
		regEx += "(\\()";
		break;	
	    case ')':
		regEx += "(\\))";
		break;	
	    case '[':
		regEx += "(\\[)";
		break;	
	    case ']':
		regEx += "(\\[)";
		break;	
	    case '{':
		regEx += "(\\{)";
		break;	
	    case '}':
		regEx += "(\\})";
		break;	
	    default:
		regEx += "(" + QString(pattern[i]) + ")";
		break;	
	    }
	    
	}
	++index;
	++i;
    }
    return regEx;
}
 //-----------------------------------------------------------------------------------------------------------
QString MusicExpression::fitToPattern(const QString src, const QString pattern, const QString fitPattern) {
    QString source = trimDirSeparators(src);
    
    if (pattern.isEmpty()) return source;
    if (source.isEmpty()) return source;
    if (fitPattern.isEmpty()) return source;
    
    DomoGlobals::logger->debug("MusicExpression::fitToPattern",
			       "String in '" + source + "' source pattern '" 
			       + pattern + "' destination pattern '" + fitPattern + "'");

    QString artist = "";
    QString title = "";
    QString year = "";
    QString genre = "";
    QString num = "";
    QString key = "";
    QString comment =  "";
    QMap<QString, uint> indexMap;
    
    QString regEx = translateToRegex(pattern, indexMap);
    
    DomoGlobals::logger->debug("MusicExpression::fitToPattern",
		       "Input pattern '" + pattern + "' translated to regex '" + regEx + "'");
    
    QRegExp rx( regEx,false );
    int pos = rx.search( source );
    if ( pos < 0 ) {
	 DomoGlobals::logger->warning("MusicExpression::fitToPattern",
			       "Input string '" + source + "' does not match the given pattern '" 
			       + pattern + "' (regex: '" + regEx + "')");
	 return source;
    }
    
   // for(int i = 1;i <= 7;++i) cout <<"i="<<i<<" cap='"<<rx.cap(i)<<"'\n";
    
    QMap<QString, uint>::ConstIterator it;
    for ( it = indexMap.begin(); it != indexMap.end(); ++it ) {
	//cout<<"key : " << it.key() << " data: " << it.data() <<"\n";
	key = it.key();
	if (key == "%a") {
	    artist = rx.cap(it.data());
	} else if (key == "%t") {
	    title = rx.cap(it.data());
	} else if (key == "%g") {
	    genre = rx.cap(it.data());
	} else if (key == "%y") {
	    year = rx.cap(it.data());
	} else if (key == "%n") {
	    num = rx.cap(it.data());
	} else if (key == "%c") {
	    comment = rx.cap(it.data());
	} else {
	    DomoGlobals::logger->warning("MusicExpression::fitToPattern",
					 "IndexMap contains unknown key '" + key + "'"); 
	}	    
    }
    
    QString output = fitPattern;
    output.replace("%a",artist);
    output.replace("%t",title);
    output.replace("%y",year);
    output.replace("%g",genre);
    output.replace("%n",num);
    output.replace("%c",comment);
    
    DomoGlobals::logger->debug("MusicExpression::fitToPattern", "Input '" + src + "' fitted to '" + output + "'"); 
    return output;
}
//-----------------------------------------------------------------------------------------------------------
//Note that the track name field is NOT set.  This is because source could be a filename or
//a complete file path and we can't assume which will be passed.
Track* MusicExpression::makeTrack(const QString pattern, const QString source) {
    QString artist = "";
    QString title = "";
    QString year = "";
    QString genre = "";
    QString num = "";
    QString key = "";
    QString comment = "";
    QMap<QString, uint> indexMap;
    
    QString regEx = MusicExpression::translateToRegex(pattern,indexMap);
     
    DomoGlobals::logger->debug("MusicExpression::makeTrack",
			       "Input pattern '" + pattern + "' translated to regex '" + regEx + "'");
    
    QRegExp rx( regEx,false );
    int pos = rx.search( source );
    if ( pos < 0 ) {
	 DomoGlobals::logger->warning("MusicExpression::makeTrack",
			       "Input string '" + source + "' does not match the given pattern '" 
			       + pattern + "' (regex: '" + regEx + "').  Resorting back to default behaviour.");
	 
	 return TrackSpace::makeTrack(source);
    }
    
    QMap<QString, uint>::ConstIterator it;
    for ( it = indexMap.begin(); it != indexMap.end(); ++it ) {
	//cout<<"key : " << it.key() << " data: " << it.data() <<"\n";
	key = it.key();
	if (key == "%a") {
	    artist = rx.cap(it.data());
	} else if (key == "%t") {
	    title = rx.cap(it.data());
	} else if (key == "%g") {
	    genre = rx.cap(it.data());
	} else if (key == "%y") {
	    year = rx.cap(it.data());
	} else if (key == "%n") {
	    num = rx.cap(it.data());
	} else if (key == "%c") {
	    comment = rx.cap(it.data());
	} else {
	    DomoGlobals::logger->warning("MusicExpression::makeTrack",
					 "IndexMap contains unknown key '" + key + "'"); 
	}	    
    }
    
    DomoGlobals::logger->debug("MusicExpression::makeTrack",
			       "Created track: artist: '" + artist + "' title: '"+ title + "' comment: '" 
			       + comment + "' no: '" + num + "' genre: '" + genre + "' year: '" + year + "'");
    
    Track* t = new Track(artist, title, comment, num.toInt());    
    t->setGenre(genre);
    t->setYear(year.toInt());
    
    return t;
     
}
//-----------------------------------------------------------------------------------------------------------
bool MusicExpression::matchesPattern(const QString pattern, const QString source) {
    QMap<QString, uint> indexMap;
    
    QString regEx = MusicExpression::translateToRegex(pattern,indexMap);
     
    DomoGlobals::logger->debug("MusicExpression::matchesPattern",
			       "Input pattern '" + pattern + "' translated to regex '" + regEx + "'");
    
    QRegExp rx( regEx,false );
    int pos = rx.search( source );
    if ( pos < 0 ) {
	 DomoGlobals::logger->info("MusicExpression::matchesPattern",
			       "Input string '" + source + "' does not match the given pattern '" 
			       + pattern + "' (regex: '" + regEx + "').");
	 return false;
    } else {
	 return true;
    }
}
//-----------------------------------------------------------------------------------------------------------
QString MusicExpression::makeFilename(const Track* t, const QString expression) {
	QString output = expression;
	output.replace("%a",t->getArtist());
	output.replace("%t",t->getTitle());
	output.replace("%y",QString::number(t->getYear()));
	output.replace("%g",t->getGenre());
	output.replace("%n",QString::number(t->getTrackNo()));
	output.replace("%c",t->getComment());
	
	DomoGlobals::logger->debug("MusicExpression::makeFilename",
			       "Input expression '" + expression + "' combined with track '" + t->absPath() +"' to form filename '" + output + "'");
 	return output;
}
//-----------------------------------------------------------------------------------------------------------
    
