/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#include "assert.h"
#include "mysqldbase.h"
#include "stringmanip.h"
#include "dbaseexception.h"
#include "domoglobals.h"
#include <iostream>
using std::cout;
using DbaseSpace::MySqlDbase;

//-----------------------------------------------------------------------------------------------------------
MySqlDbase::MySqlDbase() {
    setDriver("QMYSQL3");
}
//-----------------------------------------------------------------------------------------------------------
MySqlDbase::~MySqlDbase() {
}
//-----------------------------------------------------------------------------------------------------------
//Create a temporary table of exactly the same structure as the Track table
void MySqlDbase::createTmpTrackTable(const QString name) {
    QString q = ("CREATE TEMPORARY TABLE " + name + " ("
   "id INT,"   
   "cdno INT,"
   "name VARCHAR(255) NOT NULL,"
   "path VARCHAR(255) NOT NULL,"
   "descr VARCHAR(255),"
   "size INT,"// CHECK (size >= 0),"
   "created VARCHAR(25),"
   "modified VARCHAR(25),"
   "accessed VARCHAR(25),"
   "attributes CHAR(10),"
   //track attributes
   "trackno INT,"// CHECK (trackno >= 0),"
   "artist VARCHAR(255) NOT NULL,"
   "title VARCHAR(255) NOT NULL,"
   "album VARCHAR(100),"
   "length INT,"// CHECK (length >= 0),"
   "bitrate INT,"// CHECK (bitrate >= 0),"
   "samplingrate INT,"// CHECK (samplingrate >= 0),"
   "year INT,"// CHECK ((year >= 1900)AND(year <= 2010)),"
   "mood VARCHAR(50),"
   "genre VARCHAR(50),"
   "comment VARCHAR(255),"
   "score INT"// CHECK ((score >= 0)AND(score <= 10)),"
   ");");
    
    //z << "create tmp track table: " << q << "\n";
    
    try {
 Dbase::query(q);
    } catch(DbaseException* e) {
 throw (new DbaseException("MySqlDbase::createTmpTrackTable", "Create temp track table failed, query: '" + q + "'"
                           + "\n - Driver error: '" + lastError() + "'"));
    }
} 
//-----------------------------------------------------------------------------------------------------------
//an existing database that is always there, for example, with mysql you
//can always rely on the "mysql" and "test" databases to be there
//return empty string if no such thing
QString MySqlDbase::getExistingDbase() const {
    return "mysql";
}
//-----------------------------------------------------------------------------------------------------------
//Vendor string
QString MySqlDbase::getVendor() const {
    return "mysql";
}
//-----------------------------------------------------------------------------------------------------------
//Connect to 'connectDb' with the account 'rootUser/rootPass' and from that connection create a new account
//"user/pass" with all privileges on database "database"
void MySqlDbase::createNewAccount(const QString rootUser,
                                  const QString rootPass,
                                  const QString user,
                                  const QString pass,
                                  const QString host,
                                  const QString database,
                                  const QString connectDb) {
        
        DomoGlobals::logger->info("MySqlDbase::createNewAccount",
                                  QString("Attempting to create a new database account with the following parameters: ")
                                  + "\n   Existing User: " + rootUser
                                  + "\n   New User: " + user
                                  + "\n   Host: " + host
                                  + "\n   Database: " + database + " (to grant permissions to)"
                                  + "\n   Database to connect to: " + connectDb + "");
        
        QSqlDatabase* dbase = QSqlDatabase::addDatabase( getDriver() , "rootConnection");
        if (!dbase) throw (new DbaseException("MySqlDbase::createNewAccount","Failed to connect to database driver '" 
                                              + getDriver() + "'"));      
        
        QString conDb = "";
        if (connectDb.isEmpty()) {
                conDb = this->getExistingDbase();
        } else {
                conDb = connectDb;
        }
        
        dbase->setDatabaseName( conDb );
        dbase->setUserName( rootUser );
        dbase->setPassword( rootPass );
        dbase->setHostName( host );
        
        if ( ! dbase->open() ) {
                QSqlDatabase::removeDatabase("rootConnection");
                throw (new DbaseException("MySqlDbase::createNewAccount",
                                          "Failed to open database '" + conDb + "@" + host + "' as user '" + rootUser + "'"));
        } else {
                DomoGlobals::logger->info("MySqlDbase::createNewAccount",
                                          "Connected to database '" + conDb + "@" + host + "' as user '" + rootUser + "'");
                QString tmp = database;
                
                //Create user 'user' with NO privileges but who can connect from anywhere (%)
                QString q = "GRANT USAGE ON * TO " + user + "@\'%\' IDENTIFIED BY '" + pass + "' ;";
                QSqlQuery query1(q, dbase);
                
                if ( !query1.isActive() ) {
                        QSqlDatabase::removeDatabase("rootConnection");
                        throw (new DbaseException("MySqlDbase::createNewAccount",
                                                  "Failed to create user '" + user + "' while connected to '" + conDb + "@" + host
                                                  + "' as user '" + rootUser + "'"));
                } else {
                        DomoGlobals::logger->info("MySqlDbase::createNewAccount","Created new user '" + user + "' with NO privileges");
                }
                
                //Give the users some privileges
                q = "GRANT ALTER, CREATE, DROP, CREATE TEMPORARY TABLES, DELETE, INDEX, INSERT, SELECT, UPDATE ON " 
                    + database + ".* TO " + user + "@\'%\';";
                
                QSqlQuery query2(q, dbase);
                
                if ( !query2.isActive() ) {
                        //cout<<"last db error: " << fDefaultDB->lastError().databaseText() << "\n";
                        //cout<<"last driver error: " << fDefaultDB->lastError().driverText() << "\n";
                        
                        cout<<"something went wrong\n";
                        
                        //QSqlDatabase::removeDatabase("rootConnection");
                        throw (new DbaseException("MySqlDbase::createNewAccount",
                                                  "Failed to grant privileges to '" + user + "' while connected to '" + conDb + "@" + host
                                                  + "' as user '" + rootUser + "'"));
                }  else {
                        DomoGlobals::logger->info("MySqlDbase::createNewAccount","Granted privileges to user '" + user + "'");
                }
                
                //Flush the privileges
                q = "FLUSH PRIVILEGES;";
                QSqlQuery query3(q, dbase);
                
                if ( !query3.isActive() ) {
                        QSqlDatabase::removeDatabase("rootConnection");
                        throw (new DbaseException("MySqlDbase::createNewAccount",
                                                  "Failed to flush privileges while connected to '" + conDb + "@" + host
                                                  + "' as user '" + rootUser + "'"));
                } else {
                        DomoGlobals::logger->info("MySqlDbase::createNewAccount","Flushed privileges");
                }
                QSqlDatabase::removeDatabase("rootConnection");
        }
        
}
//-----------------------------------------------------------------------------------------------------------
//Get the last inserted id from 'table"
int MySqlDbase::lastInsertID(const QString table) {
    return queryNum("SELECT LAST_INSERT_ID() FROM " + table + ";");
}
//-----------------------------------------------------------------------------------------------------------
//create a new database 'name'
void MySqlDbase::createDbase(const QString  name){
    query("CREATE DATABASE " + name + ";");
}
//-----------------------------------------------------------------------------------------------------------
//Create the domo tables
void MySqlDbase::createTables() {
    query("CREATE TABLE Wishlist ("
   "song VARCHAR(255) NOT NULL,"
   "PRIMARY KEY (song)"
   ");");
    
    query("CREATE TABLE CD ("
   "id INT AUTO_INCREMENT NOT NULL,"
   "title VARCHAR(100) NOT NULL UNIQUE,"
   "type ENUM('MP3_CD','AUDIO_CD','DATA_CD','MOVIE_CD') NOT NULL,"
   "PRIMARY KEY (id,title));");
    
    
    query("CREATE TABLE Album ("
   "title VARCHAR(100) NOT NULL,"
   "cdno INT REFERENCES CD(id),"
   "id INT AUTO_INCREMENT,"
   "format ENUM('DIG_AUDIO','CDA') NOT NULL,"
   "tracks INT NOT NULL,"
   "PRIMARY KEY (id,title));");
    
    query("CREATE TABLE File ("
   "id INT AUTO_INCREMENT,"   
   "cdno INT REFERENCES CD(id),"
   "name VARCHAR(255) NOT NULL,"
   "path VARCHAR(255) NOT NULL,"
   "descr VARCHAR(255),"
   "size INT,"//CHECK (size >= 0),"
   "created VARCHAR(25),"
   "modified VARCHAR(25),"
   "accessed VARCHAR(25),"
   "attributes VARCHAR(25),"
   "PRIMARY KEY (id));");
    
    query("CREATE TABLE Track ("
   "id INT AUTO_INCREMENT,"   
   "cdno INT REFERENCES CD(id),"
   "name VARCHAR(255) NOT NULL,"
   "path VARCHAR(255) NOT NULL,"
   "descr VARCHAR(255),"
   "size INT,"// CHECK (size >= 0),"
   "created VARCHAR(25),"
   "modified VARCHAR(25),"
   "accessed VARCHAR(25),"
   "attributes VARCHAR(25),"
   //track attributes
   "trackno INT,"// CHECK (trackno >= 0),"
   "artist VARCHAR(255) NOT NULL,"
   "title VARCHAR(255) NOT NULL,"
   "album VARCHAR(100),"
   "length INT,"// CHECK (length >= 0),"
   "bitrate INT,"// CHECK (bitrate >= 0),"
   "samplingrate INT,"// CHECK (samplingrate >= 0),"
   "year INT,"// CHECK ((year >= 1900)AND(year <= 2010)),"
   "mood VARCHAR(50),"
   "genre VARCHAR(50),"
   "comment VARCHAR(255),"
   "score INT,"// CHECK ((score >= 0)AND(score <= 10)),"
   "PRIMARY KEY (id));");
    
    query("CREATE TABLE Mp3CD ("
   "id INT NOT NULL REFERENCES CD(id),"
   "trackcount INT,"// CHECK (trackcount >= 0),"
   "title VARCHAR(100) REFERENCES CD(title),"
   "genre VARCHAR(50) DEFAULT 'Various',"
   "comment VARCHAR(255),"
   "PRIMARY KEY (id));");
    
    query("CREATE TABLE AudioCD ("
   "id INT NOT NULL REFERENCES CD(id),"
   "trackcount INT,"// CHECK (trackcount >= 0),"
   "title VARCHAR(100) REFERENCES CD(title),"
   "artist VARCHAR(100) DEFAULT 'Various',"
   "year INT,"// CHECK ((year >= 1900)AND(year <= 2010)),"
   "genre VARCHAR(50) DEFAULT 'Various',"
   "comment VARCHAR(255),"
   "PRIMARY KEY (id));");    
    
    
    query("CREATE TABLE MovieCD ("
   "id INT NOT NULL REFERENCES CD(id),"
   "filecount INT,"// CHECK (filecount >= 0),"
   "title VARCHAR(100) REFERENCES CD(title),"
   "genre VARCHAR(50) DEFAULT 'Various',"
   "comment VARCHAR(255),"
   "PRIMARY KEY (id));");
    
    query("CREATE TABLE DataCD ("
   "id INT NOT NULL REFERENCES CD(id),"
   "filecount INT,"// CHECK (filecount >= 0),"
   "title VARCHAR(100) REFERENCES CD(title),"
   "volumelabel VARCHAR(50),"
   "comment VARCHAR(255),"
   "PRIMARY KEY (id));");
    
    query("CREATE TABLE Collection ("
   "owner VARCHAR(50) NOT NULL,"
   "descr VARCHAR(50),"
   "title VARCHAR(50) NOT NULL DEFAULT 'myCollection',"
   "mp3CDcount INT,"// CHECK (mp3CDcount >= 0),"
   "audioCDcount INT,"// CHECK (audioCDcount >= 0),"
   "movieCDcount INT,"// CHECK (movieCDcount >= 0),"
   "dataCDcount INT,"// CHECK (dataCDcount >= 0),"
   "comment VARCHAR(255),"
   "PRIMARY KEY (title));");
    
    query("CREATE TABLE Contains ("
   "title VARCHAR(50) NOT NULL REFERENCES Collection(title),"
   "id INT NOT NULL REFERENCES CD(id),"
 "PRIMARY KEY (id,title));");
}
//-----------------------------------------------------------------------------------------------------------
/* Create a where clase that matches all the words (words, NOT substrings!!!) in the stringlist
   For example:
      field =  myfield
      slist = This is a list of strings
      
      returned: '((myfield REGEXP "[[:<:]]This[[:>:]]") 
        OR (myfield REGEXP "[[:<:]]is[[:>:]]") 
  OR (myfield REGEXP "[[:<:]]a[[:>:]]") 
  OR (myfield REGEXP "[[:<:]]list[[:>:]]") 
  OR (myfield REGEXP "[[:<:]]of[[:>:]]") 
  OR (myfield REGEXP "[[:<:]]strings[[:>:]]"))'

     Strings which contain any of these words will match the where clase
*/
QString MySqlDbase::buildRegexpWhereClause(const QString field, QStringList& slist) {
    if (slist.empty()) return "(1<2)"; //return a where clause that is always true;
    
    QString query= "((" + field + " REGEXP ";
    QString tmp = "";
    uint size = slist.size();
    uint i = 1;
    for ( QStringList::Iterator it = slist.begin(); it != slist.end(); ++it ) {
 tmp = *it;
 if ( i < size ) {
     query += "\"[[:<:]]" + MySqlDbase::cleanSQL(tmp) + "[[:>:]]\") OR (" + field + " REGEXP ";
 } else {
      query += "\"[[:<:]]" + MySqlDbase::cleanSQL(tmp) + "[[:>:]]\"))";
 }
 ++i;
    }
    //cout<<"where clause is: " << query << "\n";
    return query;
}
//-----------------------------------------------------------------------------------------------------------
//Return the command necessary to backup a database
QStringList MySqlDbase::backupCommand(const QString user, const QString pwd,
          const QString dbase, const QStringList extraSwitches) const {
 
 QStringList cmd;
 cmd.append("mysqldump");
 cmd.append("-p" + pwd);
 cmd.append("-u" + user);
 cmd.append("--add-drop-table");
 
 for ( QStringList::ConstIterator it = extraSwitches.begin(); it != extraSwitches.end(); ++it ) {
  if(!(*it).isEmpty()) cmd.append(*it);
 }
 
 cmd.append(dbase);
 
 DomoGlobals::logger->info("MySqlDbase::backupCommand","Returning backup command '" + cmd.join(" ") + "'");
 return cmd;
}
//-----------------------------------------------------------------------------------------------------------
