/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#include "assert.h"
#include "odbcdbase.h"
#include "stringmanip.h"
#include "dbaseexception.h"
#include "domoglobals.h"
#include <iostream>
using std::cout;
using DbaseSpace::OdbcDbase;

/*
* TODO: this is just a clone of the mysql class
* try to update to something more universal
*/

//-----------------------------------------------------------------------------------------------------------
OdbcDbase::OdbcDbase() {
    setDriver("QODBC3");
}
//-----------------------------------------------------------------------------------------------------------
OdbcDbase::~OdbcDbase() {
}
//-----------------------------------------------------------------------------------------------------------
//Create a temporary table of exactly the same structure as the Track table
void OdbcDbase::createTmpTrackTable(const QString name) {
    QString q = ("CREATE TEMPORARY TABLE " + name + " ("
		 "id INT,"			
		 "cdno INT,"
		 "name VARCHAR(255) NOT NULL,"
		 "path VARCHAR(255) NOT NULL,"
		 "descr VARCHAR(255),"
		 "size INT,"// CHECK (size >= 0),"
		 "created VARCHAR(25),"
		 "modified VARCHAR(25),"
		 "accessed VARCHAR(25),"
		 "attributes CHAR(10),"
		 //track attributes
		 "trackno INT,"// CHECK (trackno >= 0),"
		 "artist VARCHAR(255) NOT NULL,"
		 "title VARCHAR(255) NOT NULL,"
		 "album VARCHAR(100),"
		 "length INT,"// CHECK (length >= 0),"
		 "bitrate INT,"// CHECK (bitrate >= 0),"
		 "samplingrate INT,"// CHECK (samplingrate >= 0),"
		 "year INT,"// CHECK ((year >= 1900)AND(year <= 2010)),"
		 "mood VARCHAR(50),"
		 "genre VARCHAR(50),"
		 "comment VARCHAR(255),"
		 "score INT"// CHECK ((score >= 0)AND(score <= 10)),"
		 ");");
    
    //z << "create tmp track table: " << q << "\n";
    
    try {
	Dbase::query(q);
    } catch(DbaseException* e) {
	throw (new DbaseException("OdbcDbase::createTmpTrackTable",
				  "Create temp track table failed, query: '" + q + "'"));
    }
} 
//-----------------------------------------------------------------------------------------------------------
//an existing database that is always there, for example, with Mysql you
//can always rely on the "mysql" and "test" databases to be there
//return empty string if no such thing
QString OdbcDbase::getExistingDbase() const {
    return "mysql";
}
//-----------------------------------------------------------------------------------------------------------
//Vendor string
QString OdbcDbase::getVendor() const {
    return "mysql";
}
//-----------------------------------------------------------------------------------------------------------
//Connect to 'connectDb' with the account 'rootUser/rootPass' and from that connection create a new account
//"user/pass" with all privileges on database "database"
void OdbcDbase::createNewAccount(const QString rootUser,
				  const QString rootPass,
				  const QString user,
				  const QString pass,
				  const QString host,
				  const QString database,
				  const QString connectDb) {
				
    DomoGlobals::logger->debug("OdbcDbase::createNewAccount",
			       QString("Attempting to create a new database account with the following parameters: ")
			       + "\n   Root User: " + rootUser
			       + "\n   New User: " + user
			       + "\n   Host: " + host
			       + "\n   Database: " + database + " (to grant permissions to (not used yet))"
			       + "\n   Database to connect to: " + connectDb + " (usually empty)");
    
      QSqlDatabase* dbase = QSqlDatabase::addDatabase( getDriver() , "rootConnection");
      if (!dbase) throw (new DbaseException("OdbcDbase::createNewAccount","Failed to connect to database driver '" 
					    + getDriver() + "'"));      
      
      QString conDb = "";
      if (connectDb.isEmpty()) {
        conDb = this->getExistingDbase();
      } else {
	conDb = connectDb;
      }

      dbase->setDatabaseName( conDb );
      dbase->setUserName( rootUser );
      dbase->setPassword( rootPass );
      dbase->setHostName( host );
        
    if ( ! dbase->open() ) {
	QSqlDatabase::removeDatabase("rootConnection");
	throw (new DbaseException("OdbcDbase::createNewAccount",
			      "Failed to open database '" + host + "@" + conDb + "' as user '" + rootUser + "'"));
    } else {
	QString tmp = database;
	//QString s = "GRANT ALL PRIVILEGES ON " + database + ".* TO " + user + "@localhost"
	//	    " IDENTIFIED BY '" + pass + "' WITH GRANT OPTION;";
	QString s = "GRANT ALL PRIVILEGES ON *.* TO " + user + "@localhost"
		    " IDENTIFIED BY '" + pass + "' WITH GRANT OPTION;";
	QSqlQuery query(s, dbase);
	
	if ( !query.isActive() ) {
	    QSqlDatabase::removeDatabase("rootConnection");
	    throw (new DbaseException("OdbcDbase::createNewAccount",
				  "Failed to issue grant statement 1 on '" + host + "@" + conDb 
				  + "' as user '" + rootUser + "' to user '" + user + "'"));            
	}
	
//	s = "GRANT ALL PRIVILEGES ON " + database + ".* TO " + user + "@\"%\""
	//    " IDENTIFIED BY '" + pass + "' WITH GRANT OPTION;";
	
	s = "GRANT ALL PRIVILEGES ON *.* TO " + user + "@\"%\""
	    " IDENTIFIED BY '" + pass + "' WITH GRANT OPTION;";
	QSqlQuery query2(s, dbase);
	
	if ( !query2.isActive() ) {
	    QSqlDatabase::removeDatabase("rootConnection");
	    throw (new DbaseException("OdbcDbase::createNewAccount",
				  "Failed to issue grant statement 1 on '" + host + "@" + conDb
				  + "' as user '" + rootUser + "' to user '" + user + "'"));            
	}
		    
	QSqlDatabase::removeDatabase("rootConnection");
    }
    
}
//-----------------------------------------------------------------------------------------------------------
//Get the last inserted id from 'table"
int OdbcDbase::lastInsertID(const QString table) {
    return queryNum("SELECT LAST_INSERT_ID() FROM " + table + ";");
}
//-----------------------------------------------------------------------------------------------------------
//create a new database 'name'
void OdbcDbase::createDbase(const QString  name){
    query("CREATE DATABASE " + name + ";");
}
//-----------------------------------------------------------------------------------------------------------
//Create the domo tables
void OdbcDbase::createTables() {
    query("CREATE TABLE Wishlist ("
	  "song VARCHAR(255) NOT NULL,"
	  "PRIMARY KEY (song)"
	  ");");
    
    query("CREATE TABLE CD ("
	  "id INT AUTO_INCREMENT NOT NULL,"
	  "title VARCHAR(100) NOT NULL UNIQUE,"
	  "type ENUM('MP3_CD','AUDIO_CD','DATA_CD','MOVIE_CD') NOT NULL,"
	  "PRIMARY KEY (id,title));");
    
    
    query("CREATE TABLE Album ("
	  "title VARCHAR(100) NOT NULL,"
	  "cdno INT REFERENCES CD(id),"
	  "id INT AUTO_INCREMENT,"
	  "format ENUM('DIG_AUDIO','CDA') NOT NULL,"
	  "tracks INT NOT NULL,"
	  "PRIMARY KEY (id,title));");
    
    query("CREATE TABLE File ("
	  "id INT AUTO_INCREMENT,"			
	  "cdno INT REFERENCES CD(id),"
	  "name VARCHAR(255) NOT NULL,"
	  "path VARCHAR(255) NOT NULL,"
	  "descr VARCHAR(255),"
	  "size INT,"//CHECK (size >= 0),"
	  "created VARCHAR(25),"
	  "modified VARCHAR(25),"
	  "accessed VARCHAR(25),"
	  "attributes VARCHAR(25),"
	  "PRIMARY KEY (id));");
    
    query("CREATE TABLE Track ("
	  "id INT AUTO_INCREMENT,"			
	  "cdno INT REFERENCES CD(id),"
	  "name VARCHAR(255) NOT NULL,"
	  "path VARCHAR(255) NOT NULL,"
	  "descr VARCHAR(255),"
	  "size INT,"// CHECK (size >= 0),"
	  "created VARCHAR(25),"
	  "modified VARCHAR(25),"
	  "accessed VARCHAR(25),"
	  "attributes VARCHAR(25),"
	  //track attributes
	  "trackno INT,"// CHECK (trackno >= 0),"
	  "artist VARCHAR(255) NOT NULL,"
	  "title VARCHAR(255) NOT NULL,"
	  "album VARCHAR(100),"
	  "length INT,"// CHECK (length >= 0),"
	  "bitrate INT,"// CHECK (bitrate >= 0),"
	  "samplingrate INT,"// CHECK (samplingrate >= 0),"
	  "year INT,"// CHECK ((year >= 1900)AND(year <= 2010)),"
	  "mood VARCHAR(50),"
	  "genre VARCHAR(50),"
	  "comment VARCHAR(255),"
	  "score INT,"// CHECK ((score >= 0)AND(score <= 10)),"
	  "PRIMARY KEY (id));");
    
    query("CREATE TABLE Mp3CD ("
	  "id INT NOT NULL REFERENCES CD(id),"
	  "trackcount INT,"// CHECK (trackcount >= 0),"
	  "title VARCHAR(100) REFERENCES CD(title),"
	  "genre VARCHAR(50) DEFAULT 'Various',"
	  "comment VARCHAR(255),"
	  "PRIMARY KEY (id));");
    
    query("CREATE TABLE AudioCD ("
	  "id INT NOT NULL REFERENCES CD(id),"
	  "trackcount INT,"// CHECK (trackcount >= 0),"
	  "title VARCHAR(100) REFERENCES CD(title),"
	  "artist VARCHAR(100) DEFAULT 'Various',"
	  "year INT,"// CHECK ((year >= 1900)AND(year <= 2010)),"
	  "genre VARCHAR(50) DEFAULT 'Various',"
	  "comment VARCHAR(255),"
	  "PRIMARY KEY (id));");				
    
    
    query("CREATE TABLE MovieCD ("
	  "id INT NOT NULL REFERENCES CD(id),"
	  "filecount INT,"// CHECK (filecount >= 0),"
	  "title VARCHAR(100) REFERENCES CD(title),"
	  "genre VARCHAR(50) DEFAULT 'Various',"
	  "comment VARCHAR(255),"
	  "PRIMARY KEY (id));");
    
    query("CREATE TABLE DataCD ("
	  "id INT NOT NULL REFERENCES CD(id),"
	  "filecount INT,"// CHECK (filecount >= 0),"
	  "title VARCHAR(100) REFERENCES CD(title),"
	  "volumelabel VARCHAR(50),"
	  "comment VARCHAR(255),"
	  "PRIMARY KEY (id));");
    
    query("CREATE TABLE Collection ("
	  "owner VARCHAR(50) NOT NULL,"
	  "descr VARCHAR(50),"
	  "title VARCHAR(50) NOT NULL DEFAULT 'myCollection',"
	  "mp3CDcount INT,"// CHECK (mp3CDcount >= 0),"
	  "audioCDcount INT,"// CHECK (audioCDcount >= 0),"
	  "movieCDcount INT,"// CHECK (movieCDcount >= 0),"
	  "dataCDcount INT,"// CHECK (dataCDcount >= 0),"
	  "comment VARCHAR(255),"
	  "PRIMARY KEY (title));");
    
    query("CREATE TABLE Contains ("
	  "title VARCHAR(50) NOT NULL REFERENCES Collection(title),"
	  "id INT NOT NULL REFERENCES CD(id),"
	"PRIMARY KEY (id,title));");
}
//-----------------------------------------------------------------------------------------------------------
/* Create a where clase that matches all the words (words, NOT substrings!!!) in the stringlist
   For example:
      field =  myfield
      slist = This is a list of strings
      
      returned: '((myfield REGEXP "[[:<:]]This[[:>:]]") 
      		OR (myfield REGEXP "[[:<:]]is[[:>:]]") 
		OR (myfield REGEXP "[[:<:]]a[[:>:]]") 
		OR (myfield REGEXP "[[:<:]]list[[:>:]]") 
		OR (myfield REGEXP "[[:<:]]of[[:>:]]") 
		OR (myfield REGEXP "[[:<:]]strings[[:>:]]"))'

     Strings which contain any of these words will match the where clase
*/
QString OdbcDbase::buildRegexpWhereClause(const QString field, QStringList& slist) {
    if (slist.empty()) return "(1<2)"; //return a where clause that is always true;
    
    QString query= "((" + field + " REGEXP ";
    QString tmp = "";
    uint size = slist.size();
    uint i = 1;
    for ( QStringList::Iterator it = slist.begin(); it != slist.end(); ++it ) {
	tmp = *it;
	if ( i < size ) {
	    query += "\"[[:<:]]" + OdbcDbase::cleanSQL(tmp) + "[[:>:]]\") OR (" + field + " REGEXP ";
	} else {
	     query += "\"[[:<:]]" + OdbcDbase::cleanSQL(tmp) + "[[:>:]]\"))";
	}
	++i;
    }
    //cout<<"where clause is: " << query << "\n";
    return query;
}
//-----------------------------------------------------------------------------------------------------------
//Return the command necessary to backup a database
QStringList OdbcDbase::backupCommand(const QString user, const QString pwd,
				      const QString dbase, const QStringList extraSwitches) const {
	
	QStringList cmd;
	cmd.append("mysqldump");
	cmd.append("-p" + pwd);
	cmd.append("-u" + user);
	cmd.append("--add-drop-table");
	
	for ( QStringList::ConstIterator it = extraSwitches.begin(); it != extraSwitches.end(); ++it ) {
		if(!(*it).isEmpty()) cmd.append(*it);
	}
	
	cmd.append(dbase);
	
	DomoGlobals::logger->info("OdbcDbase::backupCommand","Returning backup command '" + cmd.join(" ") + "'");
	return cmd;
}
//-----------------------------------------------------------------------------------------------------------
