/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#ifndef INC_SETTINGS_H
#include "settings.h"
#define INC_SETTINGS_H
#endif

#ifndef INC_DOMOGLOBALS_H
#include "domoglobals.h"
#define INC_DOMOGLOBALS_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#include "assert.h"

void PreferenceDiag::init() {
   cssChooser->setOnlyDirs(false);
   cssChooser->disableDirs();
   addDir->disableDirs();
   exportDir->disableDirs();
   cdromDir->disableDirs();
   langList->insertStringList(DomoGlobals::LANGUAGES);
   dbDriver->insertStringList(DomoGlobals::DBDRIVERS,0);
      
  tabWidget->setTabIconSet(tabWidget->page(0), QIconSet( QPixmap::fromMimeSource("compare.png") ) );
  tabWidget->setTabIconSet(tabWidget->page(1), QIconSet( QPixmap::fromMimeSource("add.png") ) );
  tabWidget->setTabIconSet(tabWidget->page(2), QIconSet( QPixmap::fromMimeSource("folder_sound.png") ) );
  tabWidget->setTabIconSet(tabWidget->page(3), QIconSet( QPixmap::fromMimeSource("database.png") ) );
  tabWidget->setTabIconSet(tabWidget->page(4), QIconSet( QPixmap::fromMimeSource("misc.png") ) );
  tabWidget->setTabIconSet(tabWidget->page(5), QIconSet( QPixmap::fromMimeSource("musicbrainz.png") ) );
}

void PreferenceDiag::checkPwd( const QString & s ) {
    if (!isAlnum(s)) {
 DomoGlobals::logger->error("PreferenceDiag:checkPwd","Invalid password, password may only contain letters and/or digits");
 DomoGlobals::logger->warning("PreferenceDiag:checkPwd","Password set to 'test'");
 dbPassword->setText("test");
    }
}

void PreferenceDiag::loadSettings( Settings * s )
{
    assert(s != 0);
    
   recurseDirs->setChecked( s->getBool(Settings::fRecurseDirs) );
   requireEqualCom->setChecked( s->getBool(Settings::fCommentsMatch) );
   cssChooser->setFileName( s->get(Settings::fCssFile) );
   tracksPerAlbum->setValue( s->getInt(Settings::fTracksPerAlbum) );
   addDir->setFileName( s->get(Settings::fDefaultAddDir) );
   exportDir->setFileName( s->get(Settings::fExportDir) );
   dbDriver->setCurrentText( s->get(Settings::fDbaseDriver) );
   dbHost->setText( s->get(Settings::fDbaseHost) );
   dbUsername->setText( s->get(Settings::fDbaseUser) );
   dbName->setText( s->get(Settings::fDbaseName) );
   dbPassword->setText( decrypt(s->get(Settings::fDbasePwd)) );
   
   mbServer->setText( s->get(Settings::fMbServer) );
   mbPort->setValue( s->getInt(Settings::fMbPort) );
   mbProxyServer->setText( s->get(Settings::fMbProxyServer) );
   mbProxyPort->setValue( s->getInt(Settings::fMbProxyPort) );
   mbUsername->setText( s->get(Settings::fMbLogin) );
   mbPassword->setText( decrypt(s->get(Settings::fMbPassword)) );
   
   lengthTolerance->setValue( s->getInt(Settings::fLengthTol) );
   debugLevel->setValue( s->getInt(Settings::fDebugLevel) );
   cdromDir->setFileName( s->get(Settings::fCdromDir) );
   skipConsCheck->setChecked( s->getBool(Settings::fSkipConsCheck) );
   skipQualityCheck->setChecked( s->getBool(Settings::fSkipQualityCheck) );
   autoRename->setChecked( s->getBool(Settings::fAutoRename) );
   forcePrompt->setChecked( s->getBool(Settings::fForcePrompt) );
   
   albumSrcPattern->setText( s->get(Settings::fAlbumSrcPattern) );
   albumDestPattern->setText( s->get(Settings::fAlbumDestPattern) );
   useCustomTrackFormat->setChecked( s->getBool(Settings::fUseCustomTrackFormat) );
   customTrackFormat->setText( s->get(Settings::fCustomTrackFormat) );
   metadataOverride->setChecked( s->getBool(Settings::fMetadataOverride) );
   
   QString tmp = "";
   QString cur = s->get(Settings::fLanguageFile);
   uint i = 0;
   for(QStringList::const_iterator it = DomoGlobals::LANGUAGES.begin(); it != DomoGlobals::LANGUAGES.end(); ++it) {
	++i;
	if ( (*it).contains(cur) ) break;
   }
   if (i > DomoGlobals::LANGUAGES.size()) i = 0;
   langList->setCurrentItem(i-1);
}


void PreferenceDiag::retrieveSettings( Settings * s )
{
   assert(s != 0);
   
   s->set(Settings::fRecurseDirs, recurseDirs->isChecked() );
   s->set(Settings::fCommentsMatch, requireEqualCom->isChecked() );
   s->set(Settings::fCssFile, cssChooser->fileName() );
   s->set(Settings::fTracksPerAlbum, tracksPerAlbum->value() );
   s->set(Settings::fDefaultAddDir, addDir->fileName() );
   s->set(Settings::fExportDir, exportDir->fileName() );
   s->set(Settings::fDbaseDriver, dbDriver->currentText() );
   s->set(Settings::fDbaseHost, dbHost->text() );
   s->set(Settings::fDbaseUser, dbUsername->text() );
   s->set(Settings::fDbaseName, dbName->text() );
   s->set(Settings::fDbasePwd, encrypt(dbPassword->text()) );
   
   s->set(Settings::fLengthTol, lengthTolerance->value() );
   s->set(Settings::fDebugLevel, debugLevel->value() );
   s->set(Settings::fCdromDir, cdromDir->fileName() );
   s->set(Settings::fSkipConsCheck, skipConsCheck->isChecked());
   s->set(Settings::fSkipQualityCheck, skipQualityCheck->isChecked());
   s->set(Settings::fAutoRename, autoRename->isChecked());
   
   s->set( Settings::fMbServer, mbServer->text() );
   s->set( Settings::fMbPort, mbPort->value() );
   s->set( Settings::fMbProxyServer, mbProxyServer->text() );
   s->set( Settings::fMbProxyPort, mbProxyPort->value() );
   s->set( Settings::fMbLogin, mbUsername->text() );
   s->set( Settings::fMbPassword, encrypt(mbPassword->text()) );
   
    s->set(Settings::fAlbumSrcPattern,albumSrcPattern->text());
    s->set(Settings::fAlbumDestPattern,albumDestPattern->text());
    s->set(Settings::fForcePrompt,forcePrompt->isChecked() );    
    
    s->set(Settings::fUseCustomTrackFormat, useCustomTrackFormat->isChecked() );
    s->set(Settings::fCustomTrackFormat, customTrackFormat->text() );
    s->set(Settings::fMetadataOverride, metadataOverride->isChecked() );
    
    QString str = langList->currentText();
   s->set(Settings::fLanguageFile,  extractComment(str) );
   
}



