/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_READERCSV_H
#include "readercsv.h"
#define INC_READERCSV_H
#endif

#include "qstringlist.h"

using std::cout;
using ReaderSpace::IReader;
using ReaderSpace::ReaderCsv;
using TrackSpace::Track;

//-----------------------------------------------------------------------------------------------------------
ReaderCsv::ReaderCsv(const QString s ) : IReader(s){
}
//-----------------------------------------------------------------------------------------------------------
ReaderCsv::~ReaderCsv(){
}
//-----------------------------------------------------------------------------------------------------------
void ReaderCsv::read(Track::Tracklist& tlist) {
    TextIStream* istream = getStream();
    
   if ((istream == 0) || (!istream->isGood())) throw (new IoException("ReaderCsv::read(Tracklist&)","Pre: Istream is 0 or not good"));
   QString field = "";
   QString entry = "";
   QString tmp = "";
   QStringList line;
   Track* trk = 0;
   tlist.clear();
   
    istream->getLine(tmp);
    QStringList fields = QStringList::split(";",tmp, true);
    if (tmp.endsWith(";") && (fields.size() > 0) ) fields.pop_back();
    
    //cout<<"fields: " << fields.join("|") << " size: " << fields.size() << "\n";
    
    int i = 1;
    while (istream->isGood()){
	++i;

	istream->getLine(tmp);
	if (tmp.isEmpty()) break;
	
	line = QStringList::split(";",tmp, true);
	if (tmp.endsWith(";") && (line.size() > 0) ) line.pop_back();
	//cout<<"line: " << line.join("|") << " size: " << line.size() << "\n";
	
	if (line.size() != fields.size()) {
	    throw (new IoException("ReaderCsv::read(Tracklist&)","The field count of line " 
				   + QString(tmp.setNum(istream->line())) + " does not match the declared number of fields"));
	}
	
	trk = new Track();
	
	QStringList::ConstIterator it2 = line.begin();
	for ( QStringList::ConstIterator it = fields.begin(); it != fields.end(); ++it ) {
	    field = (*it);
	    tmp = (*it2);
	    if (tmp.isNull()) tmp = "";
	    
	    entry = tmp;
	    
	    if (field =="Name") {
		trk->setName(entry);
	    } else if (field == "Id") {
		trk->setID(entry);
	    } else if (field == "Path") {
		trk->setPath(entry);
	    } else if (field == "Size") {
		trk->setSize(entry);
	    } else if (field == "Created") {
		trk->setCreated(entry);
	    } else if (field == "Modified") {
		trk->setModified(entry);
	    } else if (field == "Accessed") {
		trk->setAccessed(entry);
	    } else if (field == "Description") {
		trk->setDescription(entry);
	    } else if (field == "Attributes") {
		trk->setAttributes(entry);
	    } else if (field == "CD id") {
		trk->setCDno(entry);
	    } else if (field == "TrackNo") {
		trk->setTrackNo(entry);
	    } else if (field == "Artist") {
		trk->setArtist(entry);
	    } else if (field == "Title") {
		trk->setTitle(entry);
	    } else if (field == "Album") {
		trk->setAlbum(entry);
	    } else if (field == "Length") {
		trk->setLength(entry);
	    } else if (field == "Bitrate") {
		trk->setBitrate(entry);
	    } else if (field == "Samplingrate") {
		trk->setSamplingRate(entry);
	    } else if (field == "Year") {
		trk->setYear(entry);
	    } else if (field == "Mood") {
		trk->setMood(entry);
	    } else if (field == "Genre") {
		trk->setGenre(entry);
	    } else if (field == "Comment") {
		trk->setComment(entry);
	    } else if (field == "Score") {
		trk->setScore(entry);
	    } else {
		throw (new IoException("ReaderCsv::read(Tracklist&)","Unknown field '" + field + "'"));
	    }
	    ++it2;

	}
	tlist.push_back(trk);
    
    }
	    
   if ((istream == 0) || (!istream->isOpen())) throw (new IoException("ReaderCsv::read(Tracklist&)","Post: Istream is 0 or not open"));
}
//-----------------------------------------------------------------------------------------------------------

