/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#include "readerm3u.h"
#include "readerb4s.h"
#include "readerpls.h"
#include "readercsv.h"
#include "readertxt.h"
#include "readerfactory.h"
#include "assert.h"
#include "domoglobals.h"
#include "stringmanip.h"

using  std::cout;
using namespace ReaderSpace;
//-----------------------------------------------------------------------------------------------------------
ReaderFactory::ReaderFactory() {
    
}
//-----------------------------------------------------------------------------------------------------------
ReaderFactory::~ReaderFactory() {
}
//-----------------------------------------------------------------------------------------------------------
IReader* ReaderFactory::getReader(const QString filename) {
    QString ext = getExtension(filename).upper();
    
    DomoGlobals::logger->debug("ReaderFactory::getReader","Extension found is '" + ext + "'");
    
    IReader* reader = 0;
    
     if  (ext == "TXT"){
	reader = new ReaderTxt(filename);
    } else if (ext.isEmpty()) {
	//no extension specified, txt assumed
	DomoGlobals::logger->warning("ReaderFactory::getReader","Filename '" + filename 
				     + "' has no extension, assuming ASCII text");
	reader = new ReaderTxt(filename);
    } else if (ext == "M3U") {
	reader = new ReaderM3U(filename);
    } else if (ext == "B4S") {
	reader = new ReaderB4s(filename);
    } else if (ext == "PLS") {
	reader = new ReaderPls(filename);
    } else if (ext == "CSV") {
	reader = new ReaderCsv (filename);
    } else {
	DomoGlobals::logger->error("ReaderFactory::getReader","Filename '" + filename 
				     + "' contains unknown extension '" + ext + "', assuming ASCII text");
	reader = new ReaderTxt(filename);
    }
    
    return reader;
}
//-----------------------------------------------------------------------------------------------------------
