/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#ifndef INC_READERPLS_H
#include "readerpls.h"
#define INC_READERPLS_H
#endif

using ReaderSpace::IReader;
using ReaderSpace::ReaderPls;
using TrackSpace::Track;

//-----------------------------------------------------------------------------------------------------------
ReaderPls::ReaderPls(const QString s) : IReader(s){
}
//-----------------------------------------------------------------------------------------------------------
ReaderPls::~ReaderPls(){
}
//-----------------------------------------------------------------------------------------------------------
void ReaderPls::read(Track::Tracklist& tlist){
        TextIStream* istream = getStream();
    if ((istream == 0) || (!istream->isGood())) throw (new IoException("ReaderPls::read(Tracklist&)","Pre: Istream is 0 or not good"));
    
    QString tmp = "";
    QString tmp2 = "";
    QString trackInfo = "";
    QString fname = "";
    QString path = "";
    QString len = "";
    QString playlist = "";
    QString field = "";
    QString value =  "";
    int count = 0;
    
    istream->getLine(tmp);
    
    if (tmp != "[playlist]") {
	throw (new IoException("ReaderPls::read(Tracklist&)","'[playlist]' header not found in file '" + getFilename() + "'"));   
    }
    
    tmp = "";
    istream->getLine(tmp);
    splitOnFirstSep(tmp, '=', playlist);
    if (tmp != "PlaylistName") {
	throw (new IoException("ReaderPls::read(Tracklist&)","'PlaylistName' field not found in file '" + getFilename() + "'"));   
    }
    
    /* typical entry:
        File1=F:\tmp\track-84.wav
        Title1=track-84.wav
        Length1=64 */
    
    bool footer = false;
    tmp = "";
    while (istream->isGood()) {
	
	for(int i = 1; (i <= 3) && (istream->isGood()); ++i) {
	    field = "";
	    value = "";
	    istream->getLine(field);
	    
	    splitOnFirstSep(field, '=', value);
	    tmp2 = field;
	    
	    if (tmp2.startsWith("File")) {
		fname = getNameFromPath(value);
		path = getPathFromName(value);
	    } else if (tmp2.startsWith("Title")) {
		trackInfo = value;
	    } else if (tmp2.startsWith("Length")) {
		len = value;
	    } else if (tmp2.startsWith("NumberOfEntries")) {
		footer = true;
	    } else if (tmp2.startsWith("Version")) {
		footer = true;
	    } else {
		throw (new IoException("ReaderPls::read(Tracklist&)","Invalid field '" 
				       + tmp2 + "' found in file '" + getFilename() + "'")); 
	    }
	}
	if ( !footer ) {
	    if (empty(fname) || empty(trackInfo) /*|| empty(len)*/ ) {
		throw (new IoException("ReaderPls::read(Tracklist&)","Filename and/or Title and/or Length are empty in file '" 
				       + getFilename() + "'")); 
	    }
	    
	    ++count;
	    Track* t = TrackSpace::makeTrack(trackInfo); 
	    t->setName(fname);
	    t->setLength(len.toInt());
	    t->setPath(path);
	    t->setComment(trackInfo);
	    t->setTrackNo(count);
	    
	    if (fname != "") {
		tlist.push_back(t);
	    }
	    
	    trackInfo = "";
	    tmp = "";
	    fname = "";
	    path = "";
	    len = "";
	}
    }
    
   if ((istream == 0) || (!istream->isOpen())) throw (new IoException("ReaderPls::read(Tracklist&)","Post: Istream is 0 or not open"));
}
//-----------------------------------------------------------------------------------------------------------

