/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_DOMOGLOBALS_H
#include "domoglobals.h"
#define INC_DOMOGLOBALS_H
#endif

#ifndef INC_READERTXT_H
#include "readertxt.h"
#define INC_READERTXT_H
#endif

#ifndef INC_SETTINGS_H
#include "settings.h"
#define INC_SETTINGS_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#ifndef INC_WRITERTXT_H
#include "writertxt.h"
#define INC_WRITERTXT_H
#endif

#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#include "assert.h"
#include <iostream>
#include <vector>
#include "qdatetime.h"

using std::cout;
using std::map;
using std::vector;
using std::ios;
using ReaderSpace::ReaderTxt;
using WriterSpace::WriterTxt;

const QString Settings::fDbaseDriver	= "dbase_driver";
const QString Settings::fDbaseName 	= "dbase_name";
const QString Settings::fDbaseUser 	= "dbase_user";
const QString Settings::fDbasePwd 	= "dbase_pwd";
const QString Settings::fDbaseHost 	= "dbase_host";
const QString Settings::fExportDir 	= "export_dir";
const QString Settings::fDefaultAddDir = "default_add_dir";
const QString Settings::fExtInfoFile 	= "extension_info_file";
const QString Settings::fGenreListFile	= "genrelist_file";
const QString Settings::fArtistSafeListFile = "artist_safe_list_file";
const QString Settings::fCdromDir 	= "cdrom_dir";
const QString Settings::fLengthTol 	= "compare_length_tol";
const QString Settings::fTracksPerAlbum = "tracks_per_album";
const QString Settings::fCommentsMatch = "comments_must_match";
const QString Settings::fMaxLength 	= "max_length";
const QString Settings::fDebugLevel 	= "debug_level";
const QString Settings::fCssFile 	= "default_css_file";
const QString Settings::fSkipConsCheck = "skip_consistency_checks";
const QString Settings::fSkipQualityCheck = "skip_quality_checks";
const QString Settings::fKeywordsFile 	= "keywords_file";
const QString Settings::fRecurseDirs 	= "recurse_directories";
const QString Settings::fHyphenedArtistsFile 	= "hyphened_artists_file";
const QString Settings::fLanguageFile	= "language_file";
const QString Settings::fLanguageDir	= "language_dir";
const QString Settings::fMbServer 	= "mb_server";
const QString Settings::fMbPort 	= "mb_port";
const QString Settings::fMbProxyServer  = "mb_proxy_server";
const QString Settings::fMbProxyPort 	= "mb_proxy_port";
const QString Settings::fMbLogin 	= "mb_login";
const QString Settings::fMbPassword 	= "mb_password";
const QString Settings::fAutoRename = "auto_rename";
const QString Settings::fAlbumSrcPattern = "album_source_pattern";
const QString Settings::fAlbumDestPattern = "album_destination_pattern";
const QString Settings::fForcePrompt = "prompt_on_every_track";
const QString Settings::fUseCustomTrackFormat = "use_custom_track_format";
const QString Settings::fCustomTrackFormat = "custom_track_format";
const QString Settings::fMetadataOverride = "metadata_overrides_filename_data";
//-----------------------------------------------------------------------------------------------------------
void Settings::initMap() {
    //Set default values
    QString extInfoFile = DomoGlobals::EXTINFO_FILE;
    QString genreListFile = DomoGlobals::GENRELIST_FILE;
    QString artistSafeListFile = DomoGlobals::ARTISTSAFELIST_FILE;
    QString keywordsFile = DomoGlobals::KEYWORDS_FILE;
    QString hyphenedartistsFile = DomoGlobals::HYPHENEDARTISTS_FILE;    
    
    fFields[fDbaseDriver]		= "";
    fFields[fDbaseName]		= "";
    fFields[fDbaseUser]		= "";
    fFields[fDbasePwd]		= "";
    fFields[fDbaseHost]		= "";
    fFields[fExportDir]		= "";
    fFields[fDefaultAddDir]		= "";
    fFields[fExtInfoFile]		= extInfoFile;
    fFields[fGenreListFile]		= genreListFile;
    fFields[fArtistSafeListFile]	= artistSafeListFile;
    fFields[fCdromDir]		= "/dev/cdrom";
    fFields[fLengthTol]		= "10";
    fFields[fTracksPerAlbum]	= "8";
    fFields[fCommentsMatch]	= "0";
    fFields[fMaxLength]		= "7200";
    fFields[fDebugLevel]		= "3";
    fFields[fCssFile]		= "";
    fFields[fSkipConsCheck]	= "0";
    fFields[fSkipQualityCheck]	= "0";
    fFields[fKeywordsFile]		= keywordsFile;
    fFields[fRecurseDirs]		= "1";
    fFields[fHyphenedArtistsFile]	= hyphenedartistsFile;
    fFields[fLanguageFile]		= "";
    fFields[fLanguageDir]		= "";
    fFields[fMbServer] 		= "mm.musicbrainz.org";
    fFields[fMbPort] 		= "80";
    fFields[fMbProxyServer] 		 = "";
    fFields[fMbProxyPort] 		= "";
    fFields[fMbLogin] 		= "";
    fFields[fMbPassword] 		= "";
    fFields[fAutoRename]		= "0";
    fFields[fAlbumSrcPattern] 	= "";
    fFields[fAlbumDestPattern]	= "%a - %t";
    fFields[fForcePrompt]		= "0";
    fFields[fCustomTrackFormat]     	= "%a - %t";
    fFields[fUseCustomTrackFormat] 	= "0";
    fFields[fMetadataOverride]	= "0"; 
}
//-----------------------------------------------------------------------------------------------------------
Settings::Settings(const QString s) {
    this->initMap();
    fFilename = s;
}
//-----------------------------------------------------------------------------------------------------------
Settings::~Settings() {
}
//-----------------------------------------------------------------------------------------------------------
bool Settings::contains(const QString key) const {
    return fFields.contains(key);
}
//-----------------------------------------------------------------------------------------------------------
int Settings::getInt( const QString key ) const {
    return get(key).toInt();
}
//-----------------------------------------------------------------------------------------------------------
bool Settings::getBool( const QString key ) const {
    return ( get(key) == "1" || get(key) == "true" || get(key) == "TRUE" );
}
//-----------------------------------------------------------------------------------------------------------
QString Settings::get( const QString key ) const {
    if (contains( key ) ) {
	return fFields[key];
    } else {
	return "";
    }
}
//-----------------------------------------------------------------------------------------------------------
bool Settings::set( const QString key, const QString value ) {
    if (contains( key )){
	fFields[key] = value;
    } else {
	return false;
    }
    return true;
}
//-----------------------------------------------------------------------------------------------------------
bool Settings::set( const QString key, const int value ) {
    if (contains( key )){
	QString tmp = "";
	fFields[key] = tmp.setNum(value);
    } else {
	return false;
    }
    return true;
}
//-----------------------------------------------------------------------------------------------------------
bool Settings::set( const QString key, const bool value ) {
    if (contains( key )){
	fFields[key] = toString(value,true);
    } else {
	return false;
    }
    return true;
}
//-----------------------------------------------------------------------------------------------------------
bool Settings::getSettings() {
    try {    
	ReaderTxt reader(fFilename);
	QStringList v;
	
	reader.read(v);
	QString name = "";
	QString value = "";
	QStringList::ConstIterator it = v.begin();
	
	while (it != v.end()) {
	    name = (*it);
	    value = "";
	    splitOnFirstSep(name,'=',value);
	    //cout<<"name,value:" << name << ":" << value << "\n";
	    //if (value.isEmpty()) return false;
	    if (!set(name,value)) return false;
	    ++it;
	}
	
	return true;
    } catch (IoException* e) {
	DomoGlobals::logger->error("Settings::getSettings","Exception during reading of settings: " + e->msg());
	return false;
    }
}
//-----------------------------------------------------------------------------------------------------------
bool Settings::setSettings() const{
    try {
	WriterTxt out(fFilename);
	
	out.writeLn("#############################################################");
	out.writeLn("#       DOMO Configuration file, Generated by DOMO v" + DomoGlobals::DOMO_VERSION + "    #");
	out.writeLn("#							    #");
	out.writeLn("#       Domo is a GNU GPL licensed product by GoriTech 	    # ");
	out.writeLn("#							    #");
	out.writeLn("# 	      Created @ " + QDateTime::currentDateTime().toString("hh:mm:ss ddd MMMM d yy") + "             #");
	out.writeLn("#							    #");
	out.writeLn("#           (Mess with this file at your own risk)          #");
	out.writeLn("#############################################################\n");
	
	QMap<QString, QString>::const_iterator it;
	for ( it = fFields.begin(); it != fFields.end(); ++it ) {
	    out.writeLn( it.key() + "=" + it.data() );
	    //cout<< it.key() + "=" + it.data() +"\n";
	}
	out.closeFile();    
    } catch (IoException* e) {
	DomoGlobals::logger->error("Settings::setSettings","Exception during saving of settings: " + e->msg());
	return false;
    }
    return true;
}
//-----------------------------------------------------------------------------------------------------------
