/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef NEWACCOUNTDIAG_H
#include "newaccountdiag.h"
#define NEWACCOUNTDIAG_H
#endif

#ifndef INC_DBASEEXCEPTION_H
#include "dbaseexception.h"
#define INC_DBASEEXCEPTION_H
#endif

#ifndef SETUPDIALOG_H
#include "setupdialog.h"
#define SETUPDIALOG_H
#endif

#ifndef FILECHOOSER_H
#include "filechooser.h"
#define FILECHOOSER_H
#endif

#ifndef INC_DOMOGLOBALS_H
#include "domoglobals.h"
#define INC_DOMOGLOBALS_H
#endif

#ifndef INC_SETTINGS_H
#include "settings.h"
#define INC_SETTINGS_H
#endif

#ifndef INC_FILEIO_H
#include "fileio.h"
#define INC_FILEIO_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#include "dbasefactory.h"
#include "qmessagebox.h"
#include "assert.h"
#include "qlineedit.h"
#include "qcheckbox.h"
#include "qstringlist.h"
#include "qcombobox.h"
#include <qdir.h>

#include <iostream>
using std::cerr;
using DbaseSpace::DbaseFactory;
//-----------------------------------------------------------------------------------------------------------
SetupDialog::SetupDialog( QWidget* parent, const char* name, bool modal, WFlags f )
 : SetupDialogBase( parent, name, modal, f )
{
    exportDir->setFileName(DomoGlobals::DOMO_DIR + "export");
    cdromDir->setFileName("/dev/cdrom");
    addDir->setFileName(DomoGlobals::DOMO_DIR);  
    fDefaultDB = 0;
    fLangFile = "default";
    fResCode = QDialog::Rejected;
}
//-----------------------------------------------------------------------------------------------------------
void SetupDialog::setLangFile(const QString s) {
    fLangFile = s;
}
//-----------------------------------------------------------------------------------------------------------
void SetupDialog::error(QString s) {
    QMessageBox mb( tr("Error"),
      s,
  QMessageBox::Critical,
  QMessageBox::Ok | QMessageBox::Default,
  QMessageBox::NoButton, 
  QMessageBox::NoButton,this );
  mb.exec();
}
//-----------------------------------------------------------------------------------------------------------
void SetupDialog::createDirs(Settings* s) {
   assert(s != 0);
   bool b = false;
   QDir d;
   QString tmp = "";
   QStringList slist;
   
    slist.append( DomoGlobals::DOMO_DIR );
    slist.append( DomoGlobals::DOMO_DIR + DomoGlobals::IMAGE_DIR);
    slist.append( DomoGlobals::DOMO_DIR + DomoGlobals::RESOURCE_DIR );
    slist.append( DomoGlobals::DOMO_DIR + DomoGlobals::LANGUAGE_DIR );
    slist.append(  exportDir->fileName() );
    
    for(QStringList::const_iterator it = slist.begin(); it != slist.end(); ++it) {
 tmp = (*it);
 
 if(!QFile::exists(tmp)) {
     b = d.mkdir(tmp);
     if (!b) DomoGlobals::logger->warning("SetupDialog::createDirs","Creation of dir '" + tmp + "' failed...");
 }
    }
    
    tmp = DomoGlobals::DOMO_DIR + DomoGlobals::LANGUAGE_DIR;
    s->set(Settings::fLanguageDir, tmp);
    
    tmp = exportDir->fileName();
    s->set(Settings::fExportDir, tmp);
}
//-----------------------------------------------------------------------------------------------------------
void SetupDialog::copyResources(const Settings* s) {
   assert(s != 0);
    QStringList slist;
    QStringList::ConstIterator it;

    slist.append( DomoGlobals::KEYWORDS_FILE );
    slist.append( DomoGlobals::EXTINFO_FILE );
    slist.append( DomoGlobals::GENRELIST_FILE );
    slist.append( DomoGlobals::ARTISTSAFELIST_FILE );
    slist.append( DomoGlobals::HYPHENEDARTISTS_FILE );
    slist.append( DomoGlobals::SPLASHSCREEN );
    
    //copy all translation files
    QString tmp = "";
    for (it = DomoGlobals::LANGUAGES.begin(); it != DomoGlobals::LANGUAGES.end(); ++it ) {
 if ( !(*it).contains("default") ) {
     tmp = (*it);
     tmp = extractComment( tmp );
     slist.append( DomoGlobals::LANGUAGE_DIR + DomoGlobals::DIR_SEP + QString(tmp) );
 }
    }
    
    
    bool ok = true;
    
    //copy resource files
    for(it = slist.begin(); it != slist.end(); ++it) {
 QString resFile = (*it);
 
 if (!FileIO::exists( DomoGlobals::DOMO_DIR + resFile ) ) {
     //try the current directory first
     ok = FileIO::copyFile( resFile,  DomoGlobals::DOMO_DIR + resFile );
     if (!ok) {
  //else try the CONFIG directory
  ok = FileIO::copyFile( DomoGlobals::CONFIG_DIR + resFile,  DomoGlobals::DOMO_DIR + resFile );
  if (!ok) DomoGlobals::logger->warning("SetupDialog::copyResources","Copying of resource '" + resFile 
            + "' from '" + DomoGlobals::CONFIG_DIR + "' to '" 
            + DomoGlobals::DOMO_DIR + "' failed...");
     } 
 } else {
     DomoGlobals::logger->warning("SetupDialog::copyResources","Resource file '" + resFile + "' already exists in '" 
      + DomoGlobals::DOMO_DIR + "', not copied...");
 }
    }
       
}
//-----------------------------------------------------------------------------------------------------------
void SetupDialog::checkPwd( const QString & s ) {
    if (!isAlnum(s)) {
 DomoGlobals::logger->error("SetupDialog:checkPwd","Invalid password, password may only contain letters and/or digits");
 DomoGlobals::logger->warning("SetupDialog:checkPwd","Password set to 'test'");
 PasswordLine->setText("test");
    }
}
//-----------------------------------------------------------------------------------------------------------
bool SetupDialog::install() {
        Settings* s = new Settings(DomoGlobals::DOMO_DIR + DomoGlobals::SETTINGS_FNAME);
        QString defaultAddDir = this->addDir->fileName();
        QString cdromDir = this->cdromDir->fileName();
        QString host = this->DBServerLine->text();
        QString driver = this->DBdriverLine->currentText();
        QString name = this->DBNameLine->text();
        QString user = this->UsernameLine->text();
        QString pwd = this->PasswordLine->text();
        
        createDirs(s);
        copyResources(s);
        
        s->set(Settings::fLanguageFile, fLangFile);
        s->set(Settings::fCdromDir, cdromDir);
        s->set(Settings::fDbaseName, name);
        s->set(Settings::fDbaseDriver, driver);
        s->set(Settings::fDbaseHost, host);
        s->set(Settings::fDbaseUser, user);
        s->set(Settings::fDbasePwd, encrypt(pwd) );
        s->set(Settings::fDefaultAddDir, defaultAddDir);
        
        if (fDefaultDB == 0) {
                DbaseFactory* factory = new DbaseFactory();
                fDefaultDB = factory->createDbase(driver);
                delete factory;
        }
        
        bool b = s->setSettings();
        if (!b) {
                DomoGlobals::logger->error("SetupDialog::install","Generation of config file failed!!\n "
                                           "Please assure you have adequate space and permissions");
                return false;
        }
        
        if (nocreate->isChecked()) {
                DomoGlobals::logger->info("SetupDialog::install","Skipping database creation...");
        } else {
                b = createDbase(s->get(Settings::fDbaseName),
                                s->get(Settings::fDbaseUser),
                                decrypt(s->get(Settings::fDbasePwd)),
                                s->get(Settings::fDbaseHost),
                                existingDbase->text());
                if (!b) {
                        DomoGlobals::logger->error("SetupDialog::install","Creation of database failed!!! Please check:\n"
                                                   " - Your user/pass combination\n"
                                                   " - You have permissions to create/drop databases and your database name is unique"
                                                   " Delete ~/.domo (or edit config file) and restart");
                        return false;
                } else {
                        b = setupTables();
                        if (!b) {
                                DomoGlobals::logger->error("SetupDialog::install",
                                                             "Setting up of tables failed on database '" 
                                                             + s->get(Settings::fDbaseName) + "' Please check:\n"
                                                             " - Your database permissions\n"); 
                        } else {
                                fDefaultDB->close();
                        }
                        
                }
        }
        delete s;
        return b;
}
//-----------------------------------------------------------------------------------------------------------
bool SetupDialog::setupTables() {
    try {
 fDefaultDB->createTables();
    } catch (DbaseException* e) {
 DomoGlobals::logger->error("SetupDialog::setupTables","Failed to setup tables\n Exception: " + e->msg());
 return false;
    }
    return true;
}
//-----------------------------------------------------------------------------------------------------------
bool SetupDialog::createDbase(const QString DBName, const QString username, const QString password,
                              const QString host, const QString existingDb) {
        try {
                if(fExistingUser.isEmpty()) {
                        //The user did not use domo to create a new account, he already had a mysql account
                        //First connect to create the database and then reconnect using the same info.
                        //Why twice: not necessary here but is if we end up in the 'else' clause.
                        fDefaultDB->createConnections( existingDb, username, password, host);
                        DomoGlobals::logger->info("SetupDialog::createDbase","Created connection to '"
                                                  + existingDb + "@" + host + "' as '" + username + "'");
                } else {
                        //If the user used Domo to create a new account, that account only has privileges to access the domo database.
                        //This means we cant use that account to create a new database since that account doesnt have the privileges.
                        //Instead we use the existing account information (specified by the user when creating the domo account)
                        //to connect to an exising database and create the domo database.  When that is done we can disconnect and forget
                        //about the existing account.  In the future only the domo account will be used.
                        DomoGlobals::logger->info("SetupDialog::createDbase",
                                                  "Using existing account to create an initial connection (only used to create the database)");
                        fDefaultDB->createConnections( existingDb, fExistingUser, fExistingPass, host);
                        DomoGlobals::logger->info("SetupDialog::createDbase","Created connection to '"
                                                  + existingDb + "@" + host + "' as '" + fExistingUser + "'");
                }
        } catch (DbaseException* e) {
                DomoGlobals::logger->error("SetupDialog::createDbase",QString("Failed to connect to database\n")
                                           + "- Root cause: '" + e->msg() + "'\n"
                                           + "- Last driver error: '" + fDefaultDB->lastError() + "'");
                return false;
        }
        
        if ( !fDefaultDB->isOpen() ) {
                DomoGlobals::logger->error("SetupDialog::createDbase","Failed to open database '" 
                                           + DBName + "' through driver '" + fDefaultDB->getDriver() 
                                           + "', error: " + fDefaultDB->lastError());
                return false;
        } else {
                try { 
                        fDefaultDB->createDbase( DBName );
                        DomoGlobals::logger->info("SetupDialog::createDbase","Created database '" + DBName + "'");
                } catch (DbaseException* e) {
                        DomoGlobals::logger->error("SetupDialog::createDbase",
                                                   "Failed to create database '"+ DBName + "'\n"
                                                   + "- Root cause: " + e->msg() + "\n"
                                                   + "- Last driver error: '" + fDefaultDB->lastError() + "'");
                        return false;
                }
                
                try {
                        fDefaultDB->close();
                        DomoGlobals::logger->info("SetupDialog::createDbase","Closed connection to '" + fDefaultDB->getName() + "'");
                        fDefaultDB->createConnections(DBName, username, password, host);
                        DomoGlobals::logger->info("SetupDialog::createDbase","Created connection to '"
                                                  + DBName + "@" + host + "' as '" + username + "'");
                } catch (DbaseException* e) {
                        DomoGlobals::logger->error("SetupDialog::createDbase","Failed to create connection to '" + DBName + "'\n"
                                                   + "- Root cause: " + e->msg() + "\n"
                                                   + "- Last driver error: '" + fDefaultDB->lastError() + "'");
                        return false;
                }
                
                if ( ! fDefaultDB->isOpen() ) {
                        DomoGlobals::logger->error("SetupDialog::createDbase",
                                                   "Failed to open newly created database, error: " + fDefaultDB->lastError());
                        return false;
                } else {
                        DomoGlobals::logger->info("SetupDialog::createDbase","Successfully opened connection to '"
                                                  + fDefaultDB->getName() + "'");
                        return true;
                }
        }
        return true;
} 
//-----------------------------------------------------------------------------------------------------------
void SetupDialog::createNewAccount() {
    NewAccountDiag* dialog = new NewAccountDiag();
    QString driver = DBdriverLine->currentText();
    dialog->driver->setCurrentText( driver );
    dialog->host->setText( DBServerLine->text() );
    dialog->dbase->setText( DBNameLine->text() );
    dialog->user->setText( UsernameLine->text() );
    dialog->pass1->setText( PasswordLine->text() );
    
    if (dialog->exec() == QDialog::Accepted) {
 if (dialog->pass1->text() != dialog->pass2->text()) {
     DomoGlobals::logger->error("SetupDialog::createNewAccount","Sorry, passwords do not match!");
     error("Sorry, passwords do not match!");
 } else if(dialog->user->text().isEmpty()) {
     DomoGlobals::logger->error("SetupDialog::createNewAccount","Sorry, empty username");
     error("Sorry, empty username");
 } else if(dialog->dbase->text().isEmpty()) {
     DomoGlobals::logger->error("SetupDialog::createNewAccount","Sorry, empty database name");
     error("Sorry, empty database name");
 } else if((!dialog->driver->currentText().startsWith("QMYSQL")) && (empty(dialog->existingDbase->text())) ) {
     DomoGlobals::logger->error("SetupDialog::createNewAccount",
           "Sorry, Existing database field still empty for non-MySQL database");     
     error("Sorry, existing database field empty"); 
 } else {
     try {
  DbaseFactory* factory = new DbaseFactory();
  fDefaultDB = factory->createDbase(driver);
  delete factory; factory = 0;
  
  fDefaultDB->createNewAccount( 
      dialog->rootUser->text(), 
      dialog->rootPass->text(), 
      dialog->user->text(), 
      dialog->pass1->text(), 
      dialog->host->text(),
      dialog->dbase->text(),
      dialog->existingDbase->text());
  DomoGlobals::logger->info("SetupDialog::createNewAccount","New account created....");
                              
  fExistingUser =  dialog->rootUser->text();
  fExistingPass = dialog->rootPass->text();

  QMessageBox mb( tr("Create new Database Account"),tr("New account created!"),
  QMessageBox::Information,
  QMessageBox::Ok | QMessageBox::Default,
  QMessageBox::NoButton, 
  QMessageBox::NoButton,this );
  mb.exec();
  
  DBNameLine->setText(dialog->dbase->text());
  UsernameLine->setText(dialog->user->text());
  PasswordLine->setText(dialog->pass1->text());
  existingDbase->setText(dialog->existingDbase->text());
  
     } catch (DbaseException* e) {
  DomoGlobals::logger->error("SetupDialog::createNewAccount", QString("Creation of new account failed\n")
        + "- Root cause: '" + e->msg() + "'\n"
        + "- Last driver error: '" + fDefaultDB->lastError() + "'"); 
        
  error("Creation of new account failed, see log for details");
     }
 }
    }
    delete dialog;
}
//-----------------------------------------------------------------------------------------------------------
void SetupDialog::accept() {
    fResCode = QDialog::Accepted;
    done(0);
}
//-----------------------------------------------------------------------------------------------------------
int SetupDialog::res() const {
    return fResCode;
}
//-----------------------------------------------------------------------------------------------------------
