/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
----------------------------------------------------------------------------*/

#include "stringmanip.h"
#include "domoglobals.h"

void SplitTrackDiag::init() {
 activateTab();
 mood->insertStringList(DomoGlobals::MoodList,0);
 mood->setCurrentText( "Any" );
}

void SplitTrackDiag::setGenreList(const QStringList* list) {
 metaGenre->insertStringList(*list);
 customGenre->insertStringList(*list);
 trmGenre->insertStringList(*list);
}

void SplitTrackDiag::enableTrm() {
    trm->setEnabled(QFile::exists(trackLine->text()));
}

void SplitTrackDiag::initMetatagFields(const TrackSpace::Track* t) {
 metaArtist->setText(t->getArtist());
 metaTitle->setText(t->getTitle());
 metaComment->setText(t->getComment());
 metaAlbum->setText(t->getAlbum());
 if(!t->getGenre().isEmpty()) metaGenre->setCurrentText(t->getGenre());
 metaYear->setValue(t->getYear());
 length->setText(QString::number(t->getLength()));
 bitrate->setText(QString::number(t->getBitrate()));
 frequency->setText(QString::number(t->getSamplingRate()));
}

void SplitTrackDiag::initCustomFields(const TrackSpace::Track* t) {
 customArtist->setText(t->getArtist());
 customTitle->setText(t->getTitle());
 customComment->setText(t->getComment());
 customAlbum->setText(t->getAlbum());
 if(!t->getGenre().isEmpty()) customGenre->setCurrentText(t->getGenre());
 customYear->setValue(t->getYear());
}

void SplitTrackDiag::enableSplitMode(const bool b) {
   onFirst->setHidden(!b);
   onSecond->setHidden(!b);
   onThird->setHidden(!b);
   dashNum->setHidden(!b);
   remember->setHidden(!b); 

   if(!b) {
       //make sure that what is disabled is not checked
       if(onFirst->isChecked()||onSecond->isChecked()||onThird->isChecked()) {
    meta->setChecked(true);
       }
   }
   activateTab();
}

void SplitTrackDiag::setProperties(const ConsistencyProp& properties) {    
    meta->setChecked( properties.fMetaChecked );
    trm->setChecked( properties.fTrmChecked );
    custom->setChecked( properties.fCustomChecked );
    onExpression->setChecked( properties.fExpressionChecked );
    expression->setText( properties.fExpression );
    onFirst->setChecked( properties.fSplitOnDash == 1);
    onSecond->setChecked( properties.fSplitOnDash == 2);
    onThird->setChecked( properties.fSplitOnDash >= 3);
    if(properties.fSplitOnDash >= 3) dashNum->setValue( properties.fSplitOnDash );
    
    remember->setChecked( properties.fRememberArtist);
    skipTrackBox->setChecked( properties.fSkipFile );
    skipDeleteTrackBox->setChecked( properties.fDeleteFile );
    renameFileBox->setChecked( properties.fRenameFile );
    doNothing->setChecked( properties.fDoNothing );
    
    toAllTracks->setChecked( properties.fApplyToAll ) ;
    toAllTracksInAlbum->setChecked( properties.fApplyToAllInAlbum ) ;
    toNone->setChecked( properties.fApplyToNone ) ;
    
    writeMetatag->setChecked( properties.fWriteMetatag );
    mood->setCurrentText( properties.fMood );
    score->setValue( properties.fScore );

    //must happen at the end
     enableSplitMode(properties.fEnableSplitMode);

}

void SplitTrackDiag::getProperties(ConsistencyProp& properties) {    
    properties.fMetaChecked = meta->isChecked();
    properties.fTrmChecked = trm->isChecked();
    properties.fCustomChecked = custom->isChecked();
    properties.fExpressionChecked = onExpression->isChecked();
    properties.fExpression = expression->text();
    if(onFirst->isChecked()) properties.fSplitOnDash = 1;
    if(onSecond->isChecked()) properties.fSplitOnDash = 2;
    if(onThird->isChecked()) properties.fSplitOnDash = dashNum->value();
   
    properties.fRememberArtist = remember->isChecked();
    properties.fSkipFile =  skipTrackBox->isChecked() || skipDeleteTrackBox->isChecked();
    properties.fDeleteFile = skipDeleteTrackBox->isChecked();
    properties.fRenameFile = renameFileBox->isChecked();
    properties.fDoNothing = doNothing->isChecked();
    
    properties.fApplyToAll = toAllTracks->isChecked() ;
    properties.fApplyToAllInAlbum = toAllTracksInAlbum->isChecked() ;
    properties.fApplyToNone = toNone->isChecked();
    
    properties.fWriteMetatag = writeMetatag->isChecked();
    properties.fMood = mood->currentText();
    properties.fScore = score->value();
}

int SplitTrackDiag::getDashNum() {
    if (onFirst->isChecked()) return 1;
    if (onSecond->isChecked()) return 2;
    return dashNum->value();
}

void SplitTrackDiag::setRenameFile(const bool b) {
    renameFileBox->setChecked(b);
}

bool SplitTrackDiag::renameFile() {
    return renameFileBox->isChecked();
}

bool SplitTrackDiag::flushMetatag() {
    return writeMetatag->isChecked();
}

bool SplitTrackDiag::skipTrack() {
    return skipTrackBox->isChecked();
}

bool SplitTrackDiag::skipDeleteTrack() {
    return skipDeleteTrackBox->isChecked();
}



void SplitTrackDiag::swapArtistTitle()
{
    QString tmp = metaArtist->text();
    metaArtist->setText(metaTitle->text());
    metaTitle->setText(tmp);
    
    tmp = customArtist->text();
    customArtist->setText(customTitle->text());
    customTitle->setText(tmp);
    
    tmp = trmArtist->text();
    trmArtist->setText(trmTitle->text());
    trmTitle->setText(tmp);

}

bool SplitTrackDiag::rememberArtist() {
    return remember->isChecked();
}

void SplitTrackDiag::enableRemember(const bool b) {
    remember->setEnabled(b);
}

void SplitTrackDiag::getTrm()
{
    QString file = trackLine->text();
    QString ext = getExtension(file).upper();
    
    if(ext != "MP3" && ext != "OGG") {
        QMessageBox mb( "Unsupported Extension '" + ext + "'",
        "Sorry, TRM fingerprints only supported for .ogg and .mp3 files.",
        QMessageBox::Warning,
        QMessageBox::Ok | QMessageBox::Default,
        QMessageBox::NoButton, 
        QMessageBox::NoButton,this );
        mb.exec();
        return;
    }
    
    QStringList list;
    list.append(file);
 
    TrmDialog* diag = new TrmDialog(this);
    diag->tracklist->SetFilenames(list);
    QString title,artist,album;
    if (diag->exec() == QDialog::Accepted) {
 diag->tracklist->getFirstSelected(artist, title,album);
 trmArtist->setText(artist);
 trmTitle->setText(title);
 trmAlbum->setText(album);
    }
}

void SplitTrackDiag::setToLargeCase() {
    metaArtist->setText(largeCase(metaArtist->text()));
    metaTitle->setText(largeCase(metaTitle->text()));
    metaComment->setText(largeCase(metaComment->text()));
    
    customArtist->setText(largeCase(customArtist->text()));
    customTitle->setText(largeCase(customTitle->text()));
    customComment->setText(largeCase(customComment->text()));
 
    trmArtist->setText(largeCase(trmArtist->text()));
    trmTitle->setText(largeCase(trmTitle->text()));
    trmComment->setText(largeCase(trmComment->text()));
}

void SplitTrackDiag::close() {
   //following 3 lines patch by Fabrizio Ammollo
    if(skipTrack() || skipDeleteTrack()) {
       this->accept();
       return;
    }
    bool emptyField = false;
    if (meta->isChecked()) {
 emptyField = (empty(metaArtist->text()) || 
        empty(metaTitle->text()));
    } else if (trm->isChecked()) {
 emptyField = (empty(trmArtist->text()) || 
        empty(trmTitle->text()));
    } else if (custom->isChecked()) {
 emptyField = (empty(customArtist->text()) || 
        empty(customTitle->text()));
    } else if (onExpression->isChecked()) {
 emptyField = empty(expression->text());
    } else {
    }
    if (emptyField == false) {
 this->accept();
    } else {
        QMessageBox mb( "Empty field",
        "Empty artist, title or expression!",
        QMessageBox::Warning,
        QMessageBox::Ok | QMessageBox::Default,
        QMessageBox::NoButton, 
        QMessageBox::NoButton,this );
        mb.exec();
    }
}


void SplitTrackDiag::activateTab() {
 int activeTab = -1;
 if (meta->isChecked()) {
  activeTab = 0;
 } else if (custom->isChecked()) {
  activeTab = 1;
 } else if (trm->isChecked()) {
  activeTab = 2;
 } else if (onExpression->isChecked()) {
  activeTab = 3;
 } else {
 }
 
 for(int i = 0; i < tabs->count(); ++i) {
  if(i == activeTab) {
   tabs->setCurrentPage(i);
   tabs->setTabEnabled(tabs->page(i),true);
  } else {
   tabs->setTabEnabled(tabs->page(i),false);
  }
 }
}
