/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#ifndef INC_IOSTREAM
#include <iostream>
#define INC_IOSTREAM
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#ifndef INC_CTYPE
#include <ctype.h>
#define INC_CTYPE
#endif

#include "qregexp.h"
#include "qstringlist.h"
#include "qurl.h"
#include <string>
using std::string;
using std::cout;
//-----------------------------------------------------------------------------------------------------------
//E.g.: take the string "Bla BLA bla bLA 32 3BLA"  and turn it into
//"Bla Bla Bla Bla 32 3bla" 
QString largeCase(const QString& s) {
    QStringList list = QStringList::split(" ",s);
    QStringList::const_iterator end = list.constEnd();
    QString out = "";
    QString word = "";
    
    for ( QStringList::const_iterator it = list.constBegin(); it != end; ++it ) {
	out += (*it).lower().replace( 0, 1, (*it).at(0).upper() ) + " ";
    }
    
    return out.stripWhiteSpace();
}
//-----------------------------------------------------------------------------------------------------------
bool isUpperCase(const QString& in, const bool ignoreExtension) {
	QString s;
	if(ignoreExtension) {
		s = stripExtension(in);
	} else {
		s = in;
	}
	
	//cout<<"in: '" << s << "'\n";
	//cout<<"upper: '" << s.upper() << "'\n";
	return (s.upper() == s);
}
//-----------------------------------------------------------------------------------------------------------
bool empty(const QString& s) {
    return (s.isNull() || s.stripWhiteSpace().isEmpty());
}
//-----------------------------------------------------------------------------------------------------------
//A number is considered empty if it is zero
bool empty(const long s) {
    return (s == 0);
}
//-----------------------------------------------------------------------------------------------------------
QString stripExtension(const QString s) {
    int i = 1;
    QString out = "";

    if (s.length() < 4) return s;
	
    int j = s.length()-1;
    while (( j >= 0) && (s[j] != '.')) {
	--j;
	++i;
    }
	
    if ((j <  0)||(i > 4)) return s;
	
    i = 0;
    while (i != j) {
	out += s[i];
	++i;
    }
	
    return out;	
}
//-----------------------------------------------------------------------------------------------------------
ContentType getContentType(const QString& filename) {
    QString ext = getExtension(filename).upper();
    ContentType type;
    
    if (ext == "MP3") {
	type = MP3; 
    } else if (ext == "OGG") {
	type = OGG;
    } else if (ext == "WMA") {
	type = WMA;
    } else if (ext == "CDA") {
	type = CDA;
    } else if (ext == "WAV") {
	type = WAV;
    } else if (ext == "FLAC") {
	type = FLAC;
    } else {
	type = UNK;
	//assert(false);
    }
    return type;
}
//-----------------------------------------------------------------------------------------------------------
QString getExtension(const QString s) {
    int size = s.length();
    QString out = "";
  	
    if (s.length() < 4) return s;
	
    int j = s.length()-1;
    int i = 1;
    while (( j >= 0) && (s[j] != '.')) {
	--j;
	++i;
    }
	
    if ((j <  0)||(i > 4)) return "";

    if (s[j] == '.') ++j;
	
    while (j < size) {
	out += s[j];
	++j;
    }
    
    return out;	
}
//-----------------------------------------------------------------------------------------------------------
void splitOnFirstSep(QString& s1, const char sep, QString& s2) {
    splitOnNthSep(s1, sep, s2, 1);
}	
//-----------------------------------------------------------------------------------------------------------
QString trim(const QString s) {
    return s.stripWhiteSpace();
}
//-----------------------------------------------------------------------------------------------------------
//removes the track number from a QString, eg: '02 Wyclef - Gentleman' becomes 'Wyclef - Gentleman'
//Portion of QString removed is: (space|.|-|_)*(digit)*(space|.|-|_)*
QString stripTrackNo(const QString& s) {
    int i = 0;
    int size = s.length();
    bool sep = false;
    
    while (i < size && ( isspace(s[i]) || (s[i] == '.') || (s[i] == '_') || (s[i] == '-'))) ++i; 
    while (i < size && ( isdigit(s[i]))) ++i; 
    while (i < size && ( isspace(s[i]) || (s[i] == '.') || (s[i] == '_') || (s[i] == '-'))) {
	++i;
	sep = true;
    }
   
    if ((i == 0)||(!sep)) return s; //no track number to be removed
    
    QString out = "";
    int j = i;
    while (j < size) {
	out += s[j];
	++j;
    }
    return out;
}
//-----------------------------------------------------------------------------------------------------------
//subtract s2 from s1
QString subtract(const QString& s1, const QString& s2){
	if (s1.length() < s2.length()) return "";
	if (empty(s2)) return s1;
	if (s1 == s2) return "";
	
	int pos = s1.find(s2, 0, false);

	if (pos < 0) {
		return s1;
	} else {
		int i = 0;
		int size1 = s1.length();
		int size2 = s2.length();
		QString out = "";

		while ((i < pos) && (i < size1)) {
			out += s1[i];
			++i;
		}
		
		i = i + size2;
		
		while (i < size1) {
			out += s1[i];
			++i;
		}
		return out;
	} 
}
//-----------------------------------------------------------------------------------------------------------
//strips away round and square brackets as well
QString stripTrailingPunct(const QString s) {
	QString out = "";
	QString tmp = trim(s);
	int i = tmp.length()-1;
	uint j = 0;
	
	while( (i >= 0) && (  (s[i].isPunct()) || (s[i] == ')') || (s[i] == '(') || (s[i] == '[') || (s[i] == ']') ) ) {
	    --i;
	}
	
	uint k = i;
	while(j < tmp.length() && j <= k) {
	    out += tmp[j];
	   ++j;
	}
	
	return out;
}
//-----------------------------------------------------------------------------------------------------------
bool hasDash(const QString& s){
    return (s.contains('-',false) > 0);
}
//-----------------------------------------------------------------------------------------------------------
int dashCount(const QString& s){
    return s.contains('-',false);
}	
//-----------------------------------------------------------------------------------------------------------
int sepCount(const QString& s, const char sep){
    return s.contains(sep,false);
}	
//-----------------------------------------------------------------------------------------------------------
QString toString(const bool b, const bool num) {
    if (b) {
	if (num) {
	    return "1";
	} else {
	    return "true";
	}
    } else {
	if (num) {
	    return "0";
	} else {
	    return "false";
	}
    }
}
//-----------------------------------------------------------------------------------------------------------
QString extractComment(QString& s) {
    QString comment = "";
    QString notComment = "";
    uint size = s.length();
    uint openBrackets = 0;
    
    uint i = 0;
    while (i < size) {
	if (s[i] == '(') {
	    while( (i < size) && (s[i] == '(') ) {
		++i;
		++openBrackets;
	    }
	    
	} else if (s[i] == ')') {
    	    while( (i < size) && (s[i] == ')') ) {
		++i;
		--openBrackets;
	    }
 
	} else {
	    if (openBrackets > 0) {
		if (comment != "") comment += ", ";
		while( (i < size) && (s[i] != ')') && (s[i] != '(') ) {
		    comment += s[i];
		    ++i;
		}
		comment = trim(comment);
		
	    } else {
		if (notComment != "") notComment += " ";
		while( (i < size) && (s[i] != '(') ) {
		    notComment += s[i];
		    ++i;
		}
		notComment = trim(notComment);
	    }
	}
    }
    
     s = trim(notComment);
     return trim(comment);
}
//-----------------------------------------------------------------------------------------------------------
void splitOnNthSep(QString& s1, const char sep, QString& s2, const uint n) {
    uint i = 0;
    uint size = s1.length();
    QString tmp = "";
    uint j = 0;
    
    while (( i < size) && (j < n)) {
	if (s1[i] == sep) ++j;
	if (j < n) tmp += s1[i];
	++i;
    }
	
    if (j < n) return;
    
    s2 = "";
    while ( i < size) {
	s2 += s1[i];
	++i;
    }
    s1 = trim(tmp);
    s2 = trim(s2);
}	
//-----------------------------------------------------------------------------------------------------------
int visibleDashCount(const QString s) {
    int size = s.length();
    int count = 0;
    int inComment = 0;
    
    for (int i = 0; i < size ; ++i) {
	if ((s[i] == '(')||(s[i] == '[')) {
	    ++inComment;
	} else if ((s[i] == ')')||(s[i] == ']')) {
	    --inComment;
	} else if ((s[i] == '-')&&(inComment <= 0)) {
	    ++count;
	} else {
	}
    }
    
    return count;
}
//-----------------------------------------------------------------------------------------------------------
void splitOnVisibleDash(const QString in, QString& s1, QString& s2, QString& s3, const uint n) {
    int size = in.length();
    bool found = false;
    int inComment = 0;
    int count = 0;
    uint j = 0;
    
    for (int i = 0; ((i < size) && !found) ; ++i) {
	if ((in[i] == '(')||(in[i] == '[')) {
	    ++inComment;
	} else if ((in[i] == ')')||(in[i] == ']')) {
	    --inComment;
	} else if (in[i] == '-') {
	    ++count;
	    if (inComment <= 0) {
		++j;
		found = (j == n);
	    }
	} else {
	}
    }
 
    s1 = in;
    if (!found) {
	s2 = "";
	s3 = "";
	return;
    }
    
    splitTrackname(in, s1, s2, s3, count, '-');
}
//-----------------------------------------------------------------------------------------------------------
void balanceBrackets(QString& s) {
    int i = sepCount(s,'(');
    int j = sepCount(s,')');
    
    if (i > j) {
	for(int k = 1 ; k <= (i - j) ; ++k) {
	    s += ')';
	}
    } else if (i < j) {
	for(int k = 1 ; k <= (j - i) ; ++k) {
	    s = "(" + s;
	}
    } else {
	//brackets balanced, do nothing
    }
}
//-----------------------------------------------------------------------------------------------------------
QString cleanFilename(const QString& in) {
    QString s = removeIllegalChars(in);
    s = s.replace('_', ' ');
   //eg: replace "bla--- -- -bla" by "bla - bla"
    s = s.replace(QRegExp("(\\-)((\\s)*(\\-)+(\\s)*)*(\\-)")," - ");
    s = s.replace('[', '(');
    s = s.replace(']', ')');
    s = s.stripWhiteSpace();
    return s.simplifyWhiteSpace();
}
//-----------------------------------------------------------------------------------------------------------
QString removeIllegalChars(const QString& in) {
	QString s = in;
	s = s.replace('_', ' ');
	s = s.replace('<', "");
	s = s.replace('>', "");
	s = s.replace('?', "");
	s = s.replace('*', "");
	s = s.replace(':', "");
	s = s.replace('"', "'");
	s = s.replace('|', "");
	s = s.replace('/', "");
	s = s.replace('\\', "");
	s = s.replace(';', "");
	return s;
}
//-----------------------------------------------------------------------------------------------------------
void splitTrackname(QString fname, QString& artist,QString& title, QString& comment, const uint n, const char sep) {
    QString artistComment = "";
    QString titleComment = "";
    artist = "";
    title = "";
    comment = "";
    
    QString s = stripExtension(fname);
    artist = cleanFilename(s);
    
    uint sepcnt = sepCount(fname, sep);
    if (sepcnt == 0) {
	artist = "Unknown";
	title = s;
    } else if (sepcnt < n) {
	artist = s;
	title = "";
    } else {
	splitOnNthSep(artist, sep, title, n);
    }
    //cout<<"After nth split artist: '" << artist << "' title: '" << title << "'\n";
    balanceBrackets(artist);
    balanceBrackets(title);
    artistComment = extractComment(artist);
    titleComment = extractComment(title);
 
    if (artistComment.isEmpty()) {
	comment = titleComment;
    } else if (titleComment.isEmpty()) {
	comment = artistComment;
    } else {
	comment = artistComment + ", " + titleComment;
    }
    
    comment = trim(comment);
    
    return;
}
//-----------------------------------------------------------------------------------------------------------
QString cleanXml(const QString& in) {
    QString s = in;
    
    s = s.replace("&", "&amp;");
    //s.replace("/", "&#47;");
    s = s.replace("\"", "&#34;");
    s = s.replace("'", "&apos;");
    s = s.replace("<", "&#60;");
    s = s.replace(">", "&#62;");
    s = s.replace("$", "&#36;");
    s = s.replace("%", "&#37;");
    s = s.replace("", "&#231;");
    s = s.replace("", "&#224;");
    s = s.replace("", "&#226;");
    s = s.replace("", "&#228;");
    s = s.replace("", "&#232;");
    s = s.replace("", "&#233;");
    s = s.replace("", "&#234;");
    s = s.replace("", "&#235;");
    return s;
    
    /* Qt XML reader does not automatically decode encoded strings
    QUrl::encode(s);
    return s; */
}
//-----------------------------------------------------------------------------------------------------------
QString getNameFromPath(const QString path) {
    QString tmp = path;
    QString fname = "";
    int pos = tmp.findRev('/');		//2 types of dir separators
    if (pos < 0) pos = tmp.findRev('\\');
	
    if (pos >= 0) {
	++pos;
	while (pos < tmp.length()){
	    fname += tmp[pos];					
	    ++pos;
	}
    } else {
	fname = path;
    }
    return fname;
}
//-----------------------------------------------------------------------------------------------------------
QString getPathFromName(const QString name) {
    QString tmp = name;
    QString path = "";
 
    int pos = tmp.findRev('/');		//2 types of dir separators
    if (pos < 0) pos = tmp.findRev('\\');
	
    if (pos >= 0) {
	int j = 0;
	while (j < pos) {
	    path += tmp[j];
	    ++j;
	}	
    } else {
	path = name;
    }
    return path;
}
//-----------------------------------------------------------------------------------------------------------
QString encrypt(const QString in){
    if (empty(in)) return in;
    string s = in.latin1();
    
    int code = -7;
    string out = "";
    
    for (uint i = 0; i < s.length(); ++i) {
	out += (s[i] + code);
    }
    return out.c_str();
}
//-----------------------------------------------------------------------------------------------------------
QString decrypt(const QString in) {
    if (empty(in)) return in;
    string s = in.latin1();
    
    int code = -7;
    string out = "";
    
    for (uint i = 0; i < s.length(); ++i) {
	out += (s[i] - code);
    }
    
    return out.c_str();
}
//-----------------------------------------------------------------------------------------------------------
bool isAlnum(const QString s) {
    for (uint i = 0; i < s.length(); ++i) {
	if (! s[i].isLetterOrNumber() ) return false;
    }
    return true;
}
//-----------------------------------------------------------------------------------------------------------
QString trimDirSeparators(const QString s) {
    if (empty(s)) return "";
    uint i = 0;
    uint j = s.length()-1;
    
    while((i <= j)&&(s[i] == '/' || s[i] == '\\')) ++i; 
    while((j > i)&&(s[j] == '/' || s[j] == '\\')) --j; 
    
    QString out = "";
    
    while(i <= j) {
	out += s[i];
	++i;
    }
    return out;
}
//-----------------------------------------------------------------------------------------------------------
