/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_TEXTOSTREAM_H
#include "textostream.h"
#define INC_TEXTOSTREAM_H
#endif

#include "qfile.h"
#include <stdio.h>
#include <iostream>
using std::cout;
using WriterSpace::TextOStream;
//-----------------------------------------------------------------------------------------------------------
TextOStream::TextOStream(const QString fname, const int mode) : QTextStream() {
    QFile* f = new QFile(fname);
    bool ok = f->open(mode | IO_Translate);
    if (!ok) {
	throw (new IoException("TextOStream:TextOStream","File '" + fname + "' could not be opened"));
    }

    setDevice(f);        
    ok =  device()->isWritable();
    if (!ok) {
	throw (new IoException("TextOStream:TextOStream","File '" + fname + "' could not be opened for writing"));
    }
    fMode = Mode(LONG);
    fName = fname;
    fCdOnly = false;
}
//-----------------------------------------------------------------------------------------------------------
TextOStream::~TextOStream() {
}
//-----------------------------------------------------------------------------------------------------------
void TextOStream::setMode(const Mode m) {
   fMode = m;
}
//-----------------------------------------------------------------------------------------------------------
void TextOStream::setCdOnly(const bool b){
    fCdOnly = b;
}
//-----------------------------------------------------------------------------------------------------------
bool TextOStream::isCdOnly() const{
    return fCdOnly;
}
//-----------------------------------------------------------------------------------------------------------
bool TextOStream::isShort() const{
    switch(fMode) {
	case SHORT:
	return true;
	break;
    default:
	return false;
	break;
    }
}
//-----------------------------------------------------------------------------------------------------------
bool TextOStream::isLong() const {
   switch(fMode) {
	case LONG:
	return true;
	break;
    default:
	return false;
	break;
    }
}
//-----------------------------------------------------------------------------------------------------------
TextOStream::Mode TextOStream::getMode() const {
    return fMode;
}
//-----------------------------------------------------------------------------------------------------------
bool TextOStream::isOpen() const {
    if (device() == 0) return false; //throw (new IoException("TextOStream::isOpen","device() is 0, file is '" + fName + "'"));
    return device()->isOpen();
}
//-----------------------------------------------------------------------------------------------------------
bool TextOStream::atEnd() const {
    if (device() == 0) throw (new IoException("TextIStream::atEnd","device() is 0, file is '" + fName + "'"));
    return device()->atEnd();
}
//-----------------------------------------------------------------------------------------------------------
bool TextOStream::isGood() const {
    if (device() == 0) throw (new IoException("TextIStream::isGood","device() is 0, file is '" + fName + "'"));
    return (device()->isOpen());
}
//-----------------------------------------------------------------------------------------------------------
void TextOStream::flush() {
   if (device() != 0) device()->flush();
}
//-----------------------------------------------------------------------------------------------------------
void TextOStream::close() {
    flush();
   if (device() != 0) device()->close();
}
//-----------------------------------------------------------------------------------------------------------
