/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#ifndef INC_TRACK_H
#define INC_TRACK_H

#ifndef INC_FILE_H
#include "file.h"
#define INC_FILE_H
#endif

#ifndef INC_TEXTOSTREAM_H
#include "textostream.h"
#define INC_TEXTOSTREAM_H
#endif

#include "qstring.h"
#include <iostream>
#include <vector>

using std::vector;
using std::ostream;
using FileSpace::File;

namespace TrackSpace {
    
    class Track: public File {
	private:
	
	int fTrackNo;
	QString fArtist;
	QString fTitle;
	QString fAlbum;
	int fLength; //in seconds
	int fBitrate;  //in kbps
	int fSamplingRate;  //in KHz
	int fYear;
	QString fMood;
	QString fGenre;
	QString fComment;
	int fScore;
	
	public:
	typedef vector<Track*> Tracklist;
	typedef Tracklist::iterator TrackIterator;
	typedef Tracklist::const_iterator ConstTrackIterator;
	typedef Tracklist::reverse_iterator ReverseTrackIterator;
	typedef Tracklist::const_reverse_iterator ConstReverseTrackIterator;
	
	Track();
	Track(const QString artist, const QString title, const int num=0);
	Track(const QString artist, const QString title, const QString comment, const int num = 0);
	Track(const Track&);
	virtual ~Track();
	
	QString getArtist() const;
	void setArtist(const QString);
	
	QString getTitle() const;
	void setTitle(const QString);
	
	QString getAlbum() const;
	void setAlbum(const QString);
	
	QString getMood() const;
	void setMood(const QString);
	
	QString getGenre() const;
	void setGenre(const QString);
	
	QString getComment() const;
	void setComment(const QString);
	void appendComment(const QString);
	
	int getLength() const;
	void setLength(const int);
	void setLength(const QString);
	
	int getBitrate() const;
	void setBitrate(const int);
	void setBitrate(const QString);
	
	int getSamplingRate() const;
	void setSamplingRate(const int);
	void setSamplingRate(const QString);
	
	int getYear() const;
	void setYear(const int);
	void setYear(const QString);
	
	int getScore() const;
	void setScore(const int);
	void setScore(const QString);
	
	int getTrackNo() const;
	void setTrackNo(const int);
	void setTrackNo(const QString);
	
	virtual void writeTxt(TextOStream* ostr) const;
	virtual void writeHtml(TextOStream* ostr) const;
	virtual void writeM3u(TextOStream* ostr) const;
	virtual void writeB4s(TextOStream* ostr) const;
	virtual void writePls(TextOStream* ostr,const uint num = -1) const;
	virtual void writeCsv(TextOStream* ostr,const QStringList& fields) const;
		
	bool fishyQuality() const;
	static void deleteTracks(Tracklist& tlist);
    };
    
    Track* makeTrack(const QString&);
    bool equalLength(const Track& t1, const Track& t2, const int tolerance = 5); //tolerance in sec
    Track* castToTrack(File* f);
    
}//Close namespace TrackSpace


ostream& operator<<(ostream &, TrackSpace::Track&);


#endif

