/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
/****************************************************************************
** Form implementation generated from reading ui file 'trmdialog.ui'
**
** Created: Sat Dec 27 20:44:54 2003
**      by: The User Interface Compiler ()
**
****************************************************************************/

#include "trmdialog.h"

#include <qvariant.h>
#include <qheader.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qstring.h>
#include <settings.h>
#include "domoglobals.h"
#include "stringmanip.h"
/*
 *  Constructs a TrmDialog as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
TrmDialog::TrmDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    fSubmitTrm = false;
    if ( !name )
	setName( "TrmDialog" );
    TrmDialogLayout = new QGridLayout( this, 1, 1, 11, 6, "TrmDialogLayout"); 

    tracklist = new QMBFileView( this, "tracklist" );
    tracklist->setSelectionMode( QListView::Single );
    TrmDialogLayout->addMultiCellWidget( tracklist, 0, 0, 0, 3 );

    lookupButton = new QPushButton( this, "lookupButton" );

    TrmDialogLayout->addWidget( lookupButton, 1, 0 );

    layout58 = new QHBoxLayout( 0, 0, 6, "layout58"); 

    pixmapLabel1 = new QLabel( this, "pixmapLabel1" );
    pixmapLabel1->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, pixmapLabel1->sizePolicy().hasHeightForWidth() ) );
    pixmapLabel1->setPixmap( QPixmap::fromMimeSource( "musicbrainz.png" ) );
    pixmapLabel1->setScaledContents( TRUE );
    layout58->addWidget( pixmapLabel1 );

    textLabel2 = new QLabel( this, "textLabel2" );
    layout58->addWidget( textLabel2 );

    TrmDialogLayout->addMultiCellLayout( layout58, 2, 2, 0, 3 );
    QSpacerItem* spacer = new QSpacerItem( 110, 41, QSizePolicy::Expanding, QSizePolicy::Minimum );
    TrmDialogLayout->addItem( spacer, 1, 1 );

    okButton = new QPushButton( this, "okButton" );
    okButton->setEnabled( FALSE );

    TrmDialogLayout->addWidget( okButton, 1, 3 );

    cancelButton = new QPushButton( this, "cancelButton" );
    cancelButton->setEnabled( TRUE );

    TrmDialogLayout->addWidget( cancelButton, 1, 2 );
    languageChange();
    resize( QSize(534, 309).expandedTo(minimumSizeHint()) );
    clearWState( WState_Polished );

    // signals and slots connections
    connect( okButton, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( okButton, SIGNAL( clicked() ), this, SLOT( close() ) );
    
    connect( cancelButton, SIGNAL( clicked() ), this, SLOT( reject() ) );
    connect( cancelButton, SIGNAL( clicked() ), this, SLOT( close() ) );
    connect( lookupButton, SIGNAL( clicked() ), tracklist, SLOT( QueryServer() ) );
    connect( tracklist, SIGNAL( selectionChanged() ), this, SLOT( checkSelection() ) );
    connect( this, SIGNAL(SaveTRMs()), tracklist, SIGNAL(SaveTRMs()));
    connect( tracklist, SIGNAL( AddedResultsToList(const int&) ), this, SLOT( resultsFound(const int&) ) );
    
    
    Settings set;
    if (!set.getSettings()) {
	DomoGlobals::logger->error("TrmDialog::TrmDialog","Exception during reading of settings");
    } else {
	QString server = set.get(Settings::fMbProxyServer);
	int port = set.getInt(Settings::fMbPort);
	QString login = set.get(Settings::fMbLogin);
	QString pass = set.get(Settings::fMbPassword);
	
	tracklist->setOptions(server, port,
			      set.get(Settings::fMbProxyServer),
			      set.getInt(Settings::fMbProxyPort),
			      "{ARTIST} - {TRACK}.{EXT}",
			      "",
			      true,
			      login,
			      pass);
	DomoGlobals::logger->info("TrmDialog::TrmDialog","Settings read and set");
	
	fSubmitTrm = !(empty(login)||(empty(pass)));
    }

}
/*
 *  Destroys the object and frees any allocated resources
 */
TrmDialog::~TrmDialog()
{
    // no need to delete child widgets, Qt does it all for us
}

void TrmDialog::resultsFound(const int& n) {
    QString tmp;
    tmp.setNum(n);
    
    if (n <= 0) {
	QMessageBox::information(this, "TRM Query","Sorry no matches found...",QMessageBox::Ok);
    } else {
	QMessageBox::information(this, "TRM Query", tmp + " matche(s) found...",QMessageBox::Ok);
    }
}

void TrmDialog::checkSelection() const
{
    QListViewItem* item = tracklist->selectedItem();
    if ((item != 0)&&(item->parent() != 0)) {
	okButton->setEnabled(true);
    } else {
	okButton->setEnabled(false);
    }
    // no need to delete child widgets, Qt does it all for us
}

void TrmDialog::closeEvent(QCloseEvent* e) {
    tracklist->addSelectedFileForSubmission();
    
    if (tracklist->NumSavedTRMs() > 0) {
	if(fSubmitTrm) {
	    int response = 
		   QMessageBox::critical(this,
				      "TRM Submission",
				      "There are some unsubmitted TRMs remaining.  Submit now?",
				      "Submit", "Don't Submit");
	
	    if (response == 0) {
		//submit TRMs
		tracklist->SubmitTRMs();
	    }
	} else {
	    DomoGlobals::logger->info("TrmDialog::closeEvent",
				      "Not asking for TRM submission due to the abscense of Musicbrainz login credentials");
	}	 
    } else {
    }
    
    e->accept();
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void TrmDialog::languageChange()
{
    setCaption( tr( "Audio Fingerprint Based Lookup" ) );
    tracklist->clear();
    lookupButton->setText( tr( "&Perform TRM lookup" ) );
    lookupButton->setAccel( QKeySequence( tr( "Alt+P" ) ) );
    textLabel2->setText( tr( "Powered by <font color=\"#0055ff\"><a href=\"http://www.musicbrainz.org\">http://www.musicbrainz.org</a></font>\n"
"<br> Based on QMBTagger: <a href=\"http://qmbtagger.sf.net\">http://qmbtagger.sf.net</a>" ) );
    okButton->setText( tr( "&OK" ) );
    okButton->setAccel( QKeySequence( tr( "Alt+O" ) ) );
    cancelButton->setText( tr( "&Cancel" ) );
    cancelButton->setAccel( QKeySequence( tr( "Alt+C" ) ) );
}

