/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#include "assert.h"

#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_WRITERB4S_H
#include "writerb4s.h"
#define INC_WRITERB4S_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

using WriterSpace::IWriter;
using FileSpace::File;
using WriterSpace::WriterB4s;

//-----------------------------------------------------------------------------------------------------------
WriterB4s::WriterB4s(const QString s) : IWriter(s){
}
//-----------------------------------------------------------------------------------------------------------
WriterB4s::~WriterB4s(){
}
//-----------------------------------------------------------------------------------------------------------
void WriterB4s::writeHeader(const CD* cd) {
  TextOStream* ostream = getStream();
  if (cd == 0) throw (new IoException("WriterB4s::writeHeader","Pre: cd is 0"));
  if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterB4s::writeHeader","Pre: Ostream is 0 or not good"));
       
    *ostream << "<?xml version=\"1.0\" encoding='UTF-8' standalone=\"yes\"?>\n";
    *ostream << "<WinampXML>\n\n";
    
    *ostream << "<!-- Generated by: GoriTech Domo version " + DomoGlobals::DOMO_VERSION + "\n";
    *ostream << "      Domo is freely available under the GNU GPL at http://domo.sf.net/\n";
    *ostream << "      Please send all bugs/comments/requests to " + DomoGlobals::AUTHOR_MAIL + " -->\n\n";
     
    *ostream << "<playlist num_entries=\"" + toString(cd->getFileCount()) + "\" label=\"" + cleanXml(cd->getTitle()) + "\">\n";
    
  if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterB4s::writeHeader","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void WriterB4s::writeFooter(const QString str) {
  TextOStream* ostream = getStream();
  if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterB4s::writeFooter","Pre: Ostream is 0 or not good"));
   
    *ostream << "</playlist>\n";
    *ostream << "</WinampXML>\n";
      
  if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterB4s::writeFooter","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void WriterB4s::write(CD* m,const bool onePerFile){
    TextOStream* ostream = getStream();
    if (m == 0) throw (new IoException("WriterB4s::write","Pre: cd is 0"));
    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterB4s::write","Pre: Ostream is 0 or not good"));
    
    if (onePerFile) {
	writeHeader(m);
    }
    
    m->writeB4s(ostream);
    
    if (onePerFile) {
	writeFooter();
    }
    
    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterB4s::write","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
