/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_WRITERCSV_H
#include "writercsv.h"
#define INC_WRITERCSV_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#include "qstring.h"

using std::ofstream;
using WriterSpace::IWriter;
using WriterSpace::WriterCsv;
using FileSpace::File;
//-----------------------------------------------------------------------------------------------------------
WriterCsv::WriterCsv(const QString s,QStringList* fields) : IWriter(s) {
    if (fields == 0) throw (new IoException("WriterCsv::WriterCsv","Fields list is 0"));
    fFields = fields;
}
//-----------------------------------------------------------------------------------------------------------
WriterCsv::~WriterCsv() {
    delete fFields;
}
//-----------------------------------------------------------------------------------------------------------
void WriterCsv::writeHeader(const QString str) {
    TextOStream* ostream = getStream();
    if ((ostream == 0) || (fFields == 0) || (!ostream->isGood())) throw (new IoException("WriterCsv::writeHeader","Pre: Ostream is 0 or not good"));
    
    QString s = fFields->join(";");
    *ostream << s << "\n";
    
    if ((ostream == 0) ||  (fFields == 0) || (!ostream->isGood())) throw (new IoException("WriterCsv::writeHeader","Post: Ostream/Fields is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void WriterCsv::writeFooter(const QString str) {
  TextOStream* ostream = getStream();
  if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterCsv::writeFooter","Pre: Ostream is 0 or not good"));
    
    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterCsv::writeFooter","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void WriterCsv::write(CD* m,const bool onePerFile){
   TextOStream* ostream = getStream();
   if (m == 0) throw (new IoException("WriterCsv::write(CD*,bool)","Pre: CD is 0"));
    if ((ostream == 0) ||  (fFields == 0) || (!ostream->isGood())) throw (new IoException("WriterCsv::write(CD*,bool)","Pre: Ostream/Fields is 0 or not good"));
    
    if (onePerFile) {
	writeHeader();
    }
    
    m->writeCsv(ostream,*fFields);
    *ostream << "\n";
    
    if (onePerFile) {
	writeFooter();
    }
    
    if ((ostream == 0) ||  (fFields == 0) || (!ostream->isGood())) throw (new IoException("WriterCsv::write(CD*,bool)","Post: Ostream/Fields is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------




