/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_WRITERHTML_H
#include "writerhtml.h"
#define INC_WRITERHTML_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#include "qurl.h"
#include "qdatetime.h"
#include <iostream>
using std::cout;
using std::vector;
using std::ofstream;
using WriterSpace::IWriter;
using WriterSpace::WriterHtml;

//-----------------------------------------------------------------------------------------------------------
WriterHtml::WriterHtml(const QString s) : IWriter(s){
    fStyleSheet = "";
}
//-----------------------------------------------------------------------------------------------------------
WriterHtml::~WriterHtml() {
}
//-----------------------------------------------------------------------------------------------------------
void WriterHtml::writeHeader(const QString cdType) {
    TextOStream* ostream = getStream();
    
    if ((ostream==0)||(!ostream->isGood())) throw (new IoException("WriterHtml::writeHeader","Pre: Ostream is 0 or not good"));
    
    *ostream << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n";
    *ostream << "<html>\n";
    *ostream << "<head>\n";
    *ostream << "\t<title> DOMO - Music Database </title>\n";
    
    if (fStyleSheet != "") *ostream << "\t<link rel=\"stylesheet\" type=\"text/css\" href=\"" + fStyleSheet + "\" />\n";
    
    *ostream << "</head>\n\n";
    *ostream << "<body>\n";
    *ostream << "<a name=\"top\"></a>\n";
    *ostream << "<center><h2>&copy; GoriTech - DOMO " + DomoGlobals::DOMO_VERSION + " Generated HTML File</h2></center>\n<hr>\n\n";
    
    if (ostream->isCdOnly()) {
	*ostream << "<table border cellpadding=6 cellspacing=0>\n";
	*ostream << "<tr>\n";
	*ostream << "   <th>ID</th>\n";
	*ostream << "   <th>Title</th>\n";
	if (ostream->isLong()) {
	    *ostream << "   <th>File Count</th>\n";
	    *ostream << "   <th>Comment</th>\n";
	}	
	if (cdType == "MP3_CD") {
	    if (ostream->isLong()) {
		*ostream << "   <th>Genre</th>\n";    
	    }
	} else if (cdType == "AUDIO_CD") {
	    if (ostream->isLong()) {
		*ostream << "   <th>Artist</th>\n";    
		*ostream << "   <th>Genre</th>\n";    
		*ostream << "   <th>Year</th>\n";    
	    } else {
		*ostream << "   <th>Artist</th>\n";    
	    }
	} else {
	}
	*ostream << "</tr>\n";
    }
    
    if  ((ostream==0)||(!ostream->isGood())) throw (new IoException("WriterHtml::writeHeader","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void WriterHtml::writeFooter(const QString str) {
    TextOStream* ostream = getStream();
    
    if ((ostream==0)||(!ostream->isGood())) throw (new IoException("WriterHtml::writeFooter","Pre: Ostream is 0 or not good"));
    
    if (ostream->isCdOnly()) {
	*ostream << "\n</table>\n";
    }

    *ostream << "\n<hr>\n";
    *ostream << "<left><em><font size=1>Created: ";
    *ostream << (QDateTime::currentDateTime().toString("dddd dd/MM/yyyy  hh:mm:ss")); 	 
    *ostream << "</font></em></left>\n\n";
    
    *ostream << "<center>\n";
    *ostream << "\t&copy; GoriTech - DOMO " + DomoGlobals::DOMO_VERSION + " HTML Generator - "
	    "<a href=\"http://domo.sf.net\">Contact me</a>\n";
    *ostream << "\t<p>\n";
    *ostream << "\t<em>DOMO is freely available under the <a href=\"http://www.gnu.org/licenses/gpl.html\">GNU GPL</a>"
	    " at <a href=\"http://domo.sf.net\">Sourceforge</a></em>\n";
    *ostream << "</center>\n\n";
    
    *ostream << "</body>\n";
    *ostream << "</html>\n";
    
    if ((ostream==0)||(!ostream->isGood())) throw (new IoException("WriterHtml::writeFooter","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void WriterHtml::writeIndexFile(const QStringList& idList,const QStringList& titleList, const QString fname,const bool singleFile){
   TextOStream* ostream = getStream();
    if  ((ostream==0)||(!ostream->isGood())) throw (new IoException("WriterHtml::writeIndexFile","Pre: Ostream is 0 or not good"));
    if (idList.size() != titleList.size()) throw (new IoException("WriterHtml::writeIndexFile","Pre: IdList.size != titleList.size"));
    
    writeHeader();
    
    QStringList::ConstIterator it1= idList.begin();
    QStringList::ConstIterator it2= titleList.begin();
    QString title = "";
    QString id = "";
    
    
    QString realFname = fname;
    QString realTitle = "";
    	    
    //Url encode patch by Fabrizio Ammollo <fabrizioammollo@fastwebnet.it> 24/2/2004
    //-> commented out as of yet, can't be applied as is
    *ostream << "<left><h3>Click on a title to go straight to the corresponding Entry:</h3></left>\n\n";
    *ostream << "<ol>\n";
    while (it1 != idList.end()) {
	id = (*it1);
	title = (*it2);
	realTitle = title;
	QUrl::encode(realFname);
        QUrl::encode(realTitle);
 	if (singleFile) {
	    *ostream << "\t<li><a href=\"" + realFname + "#" + id + "_" + realTitle + "\">" + title + "</a></li>\n";
	} else {
	    *ostream << "\t<li><a href=\"" + realTitle + ".html#" + id + "_" + realTitle + "\">" + title + "</a></li>\n";
	}
	++it1;
	++it2;
    }
    
    *ostream << "</ol>\n";
    
    writeFooter();
    
    if ((ostream==0)||(!ostream->isGood())) throw (new IoException("WriterHtml::writeIndexFile","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void WriterHtml::write(CD* m, const bool onePerFile){
    if (m == 0) throw (new IoException("WriterHtml::write(CD*,bool)","Pre: CD is 0"));
    
    TextOStream* ostream = getStream();    
    if ((ostream==0)||(!ostream->isGood())) throw (new IoException("WriterHtml::write(CD*,bool)","Pre: Ostream is 0 or not good"));
    
    if (onePerFile) {
	writeHeader(m->getTypeString());
    }

    m->writeHtml(ostream);
    
    if (!ostream->isCdOnly()) *ostream << "\n<a href=\"#top\">top</a><br><br>\n";
 
   if (onePerFile) {
	writeFooter();
    }
   
   if ((ostream==0)||(!ostream->isGood())) throw (new IoException("WriterHtml::write(CD*,bool)","Post: Ostream is 0 or not good"));
    
}
//-----------------------------------------------------------------------------------------------------------
void WriterHtml::write(Collection* c){
   if (c == 0) throw (new IoException("WriterHtml::write(Collection*)","Pre: Collection is 0"));
   
   TextOStream* ostream = getStream();
   if ((ostream==0)||(!ostream->isGood())) throw (new IoException("WriterHtmlt::write(Collection*)","Pre: Ostream is 0 or not good"));
   //get rid of compiler warnings
   if (c) {};
   
   if ((ostream==0)||(!ostream->isGood())) throw (new IoException("WriterHtml::write(Collection*)","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void WriterHtml::setStyleSheet(const QString sheet) {
    fStyleSheet = sheet;
}
//-----------------------------------------------------------------------------------------------------------
