/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_WRITERTXT_H
#include "writertxt.h"
#define INC_WRITERTXT_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#include "qdatetime.h"

using std::ofstream;
using WriterSpace::IWriter;
using WriterSpace::WriterTxt;
using FileSpace::File;

//-----------------------------------------------------------------------------------------------------------
WriterTxt::WriterTxt(const QString s) : IWriter(s) {
}
//-----------------------------------------------------------------------------------------------------------
WriterTxt::~WriterTxt() {
}
//-----------------------------------------------------------------------------------------------------------
void WriterTxt::writeHeader(const QString s) {
    TextOStream* ostream = getStream();

    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterTxt::writeHeader","Pre: Ostream is 0 or not good"));
 
    QString header = "#\n"
		    "# (C) GoriTech - Domo Txt Generator\n"
		    "#\n\n\n";
    
    if (!s.isEmpty()) {
	*ostream << s;	
    } else {
	*ostream << header;
    }

    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterTxt::writeHeader","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void WriterTxt::writeFooter(const QString s) {
   TextOStream* ostream = getStream();
   
   if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterTxt::writeFooter","Pre: Ostream is 0 or not good"));
    
    if (!s.isEmpty()) {
	*ostream << s;
    } else {
	*ostream << "\n";
	*ostream << "#\n";
	*ostream << "# Created on: " << QDateTime::currentDateTime().toString("hh:mm:ss ddd MMMM d yy")  << " By D.O.M.O\n";
	*ostream << "#\n";
	*ostream << "# (C) GoriTech Domo is available for free under the GNU GPL at http://domo.sf.net/\n";
	*ostream << "# Please send all bugs/comments/requests to Dirk Gorissen <" + DomoGlobals::AUTHOR_MAIL +">\n";
	*ostream << "#\n";
    }
    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterTxt::writeFooter","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void WriterTxt::write(const QString s) {
  TextOStream* ostream = getStream();
    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterTxt::write(const QString)","Pre: Ostream is 0 or not good"));
    *ostream << s;
    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterTxt::write(const QString)","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void WriterTxt::writeLn(const QString s) {
  TextOStream* ostream = getStream();
    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterTxt::writeLn(const QString)","Pre: Ostream is 0 or not good"));
    *ostream << s << "\n";
    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterTxt::writeLn(const QString)","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void WriterTxt::write(const File* f) {
  TextOStream* ostream = getStream();
    if (f == 0) throw (new IoException("WriterTxt::write(const File*)","Pre: argument 'File* f' is 0"));
    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterTxt::write(const File*)","Pre: Ostream is 0 or not good"));
    
    f->writeTxt(ostream);
    
    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterTxt::write(const File*)","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void WriterTxt::write(CD* m,const bool onePerFile){
  TextOStream* ostream = getStream();
    if (m == 0) throw (new IoException("WriterTxt::write(CD*,bool)","Pre: CD is 0"));
    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterTxt::write(CD*,bool)","Pre: Ostream is 0 or not good"));
    
    if (onePerFile) {
	writeHeader();
    }
    
    m->writeTxt(ostream);
    
    if (onePerFile) {
	writeFooter();
    }
    
    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterTxt::write(CD*,bool)","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void WriterTxt::write(Collection* c){
  TextOStream* ostream = getStream();
    if (c == 0) throw (new IoException("WriterTxt::write(Collection*)","Pre: Collection is 0"));
    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterTxt::write(Collection*)","Pre: Ostream is 0 or not good"));
    
    if (c) {};
    
    if ((ostream == 0) || (!ostream->isGood())) throw (new IoException("WriterTxt::write(Collection*)","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------




